@@ -57,16 +57,16 @@ int FEAT_LAVA;
 /**
  * Global array for looping through the "keypad directions".
  */
-const s16b ddd[9] =
+const int16_t ddd[9] =
 { 2, 8, 6, 4, 3, 1, 9, 7, 5 };
 
 /**
  * Global arrays for converting "keypad direction" into "offsets".
  */
-const s16b ddx[10] =
+const int16_t ddx[10] =
 { 0, -1, 0, 1, -1, 0, 1, -1, 0, 1 };
 
-const s16b ddy[10] =
+const int16_t ddy[10] =
 { 0, 1, 1, 1, 0, 0, 0, -1, -1, -1 };
 
 
@@ -81,17 +81,17 @@ const struct loc ddgrid[10] =
  * This means that each entry in this array corresponds to the direction
  * with the same array index in ddd[].
  */
-const s16b ddx_ddd[9] =
+const int16_t ddx_ddd[9] =
 { 0, 0, 1, -1, 1, -1, 1, -1, 0 };
 
-const s16b ddy_ddd[9] =
+const int16_t ddy_ddd[9] =
 { 1, -1, 0, 0, 1, 1, -1, -1, 0 };
 
 const struct loc ddgrid_ddd[9] =
 {{0, 1}, {0, -1}, {1, 0}, {-1, 0}, {1, 1}, {-1, 1}, {1, -1}, {-1, -1}, {0, 0}};
 
 /* Can mult these by 45 deg or 1.5 o'clock e.g. [6] -> 270 deg or 9 o'clock */
-const s16b clockwise_ddd[9] =
+const int16_t clockwise_ddd[9] =
 { 8, 9, 6, 3, 2, 1, 4, 7, 5 };
 
 const struct loc clockwise_grid[9] =
@@ -236,7 +236,7 @@ const int *dist_offsets_x[10] =
  * first.  If the left, it must add 10 to the central direction to
  * access the second part of the table.
  */
-const byte side_dirs[20][8] = {
+const uint8_t side_dirs[20][8] = {
 	{0, 0, 0, 0, 0, 0, 0, 0},	/* bias right */
 	{1, 4, 2, 7, 3, 8, 6, 9},
 	{2, 1, 3, 4, 6, 7, 9, 8},
@@ -356,15 +356,15 @@ struct chunk *cave_new(int height, int width) {
 	c->feat_count = mem_zalloc((z_info->f_max + 1) * sizeof(int));
 
 	c->squares = mem_zalloc(c->height * sizeof(struct square*));
-	c->noise.grids = mem_zalloc(c->height * sizeof(u16b*));
-	c->scent.grids = mem_zalloc(c->height * sizeof(u16b*));
+	c->noise.grids = mem_zalloc(c->height * sizeof(uint16_t*));
+	c->scent.grids = mem_zalloc(c->height * sizeof(uint16_t*));
 	for (y = 0; y < c->height; y++) {
 		c->squares[y] = mem_zalloc(c->width * sizeof(struct square));
 		for (x = 0; x < c->width; x++) {
 			c->squares[y][x].info = mem_zalloc(SQUARE_SIZE * sizeof(bitflag));
 		}
-		c->noise.grids[y] = mem_zalloc(c->width * sizeof(u16b));
-		c->scent.grids[y] = mem_zalloc(c->width * sizeof(u16b));
+		c->noise.grids[y] = mem_zalloc(c->width * sizeof(uint16_t));
+		c->scent.grids[y] = mem_zalloc(c->width * sizeof(uint16_t));
 	}
 
 	c->objects = mem_zalloc(OBJECT_LIST_SIZE * sizeof(struct object*));
@@ -381,23 +381,33 @@ struct chunk *cave_new(int height, int width) {
 	return c;
 }
 
+/**
+ * Free a linked list of cave connections.
+ */
+void cave_connectors_free(struct connector *join)
+{
+	while (join) {
+		struct connector *current = join;
+
+		join = current->next;
+		mem_free(current->info);
+		mem_free(current);
+	}
+}
+
 /**
  * Free a chunk
  */
 void cave_free(struct chunk *c) {
+	struct chunk *p_c = (c == cave && player) ? player->cave : NULL;
 	int y, x, i;
 
-	while (c->join) {
-		struct connector *current = c->join;
-		mem_free(current->info);
-		c->join = current->next;
-		mem_free(current);
-	}
+	cave_connectors_free(c->join);
 
 	/* Look for orphaned objects and delete them. */
 	for (i = 1; i < c->obj_max; i++) {
 		if (c->objects[i] && loc_is_zero(c->objects[i]->grid)) {
-			object_delete(&c->objects[i]);
+			object_delete(c, p_c, &c->objects[i]);
 		}
 	}
 
@@ -407,7 +417,7 @@ void cave_free(struct chunk *c) {
 			if (c->squares[y][x].trap)
 				square_free_trap(c, loc(x, y));
 			if (c->squares[y][x].obj)
-				object_pile_free(c->squares[y][x].obj);
+				object_pile_free(c, p_c, c->squares[y][x].obj);
 		}
 		mem_free(c->squares[y]);
 		mem_free(c->noise.grids[y]);
@@ -546,30 +556,68 @@ void object_lists_check_integrity(struct chunk *c, struct chunk *c_k)
 void scatter(struct chunk *c, struct loc *place, struct loc grid, int d,
 			 bool need_los)
 {
-	int tries = 0;
-
-	/* Pick a location, try many times */
-	while (tries < 1000) {
-		/* Pick a new location */
-		struct loc new_grid = rand_loc(grid, d, d);
-		tries++;
-
-		/* Ignore annoying locations */
-		if (!square_in_bounds_fully(c, new_grid)) continue;
-
-		/* Ignore "excessively distant" locations */
-		if ((d > 1) && (distance(grid, new_grid) > d)) continue;
-		
-		/* Require "line of sight" if set */
-		if (need_los && !los(c, grid, new_grid)) continue;
-
-		/* Set the location and return */
-		*place = new_grid;
-		return;
-	}
+	(void) scatter_ext(c, place, 1, grid, d, need_los, NULL);
 }
 
 
+/**
+ * Try to find a given number of distinct, randomly selected, locations that
+ * are within a given distance of a grid, fully in bounds, and, optionally,
+ * are in the line of sight of the given grid and satisfy an additional
+ * condition.
+ * \param c Is the chunk to search.
+ * \param places Points to the storage for the locations found.  That storage
+ * must have space for at least n grids.
+ * \param n Is the number of locations to find.
+ * \param grid Is the location to use as the origin for the search.
+ * \param d Is the maximum distance, in grids, that a location can be from
+ * grid and still be accepted.
+ * \param need_los If true, any locations found will also be in the line of
+ * sight from grid.
+ * \param pred If not NULL, evaluating that function at a found location, lct,
+ * will return true, i.e. (*pred)(c, lct) will be true.
+ * \return Return the number of locations found.  That number will be less
+ * than or equal to n if n is not negative and will be zero if n is negative.
+ */
+int scatter_ext(struct chunk *c, struct loc *places, int n, struct loc grid,
+		int d, bool need_los, bool (*pred)(struct chunk *, struct loc))
+{
+	int result = 0;
+	/* Stores feasible locations. */
+	struct loc *feas = mem_alloc(MIN(c->width, (1 + 2 * MAX(0, d)))
+			* (size_t) MIN(c->height, (1 + 2 * MAX(0, d)))
+			* sizeof(*feas));
+	int nfeas = 0;
+	struct loc g;
+
+	/* Get the feasible locations. */
+	for (g.y = grid.y - d; g.y <= grid.y + d; ++g.y) {
+		for (g.x = grid.x - d; g.x <= grid.x + d; ++g.x) {
+			if (!square_in_bounds_fully(c, g)) continue;
+			if (d > 1 && distance(grid, g) > d) continue;
+			if (need_los && !los(c, grid, g)) continue;
+			if (pred && !(*pred)(c, g)) continue;
+			feas[nfeas] = g;
+			++nfeas;
+		}
+	}
+
+	/* Assemble the result. */
+	while (result < n && nfeas > 0) {
+		/* Choose one at random and append it to the outgoing list. */
+		int choice = randint0(nfeas);
+
+		places[result] = feas[choice];
+		++result;
+		/* Shift the last feasible one to replace the one selected. */
+		--nfeas;
+		feas[choice] = feas[nfeas];
+	}
+
+	mem_free(feas);
+	return result;
+}
+
 /**
  * Get a monster on the current level by its index.
  */
@@ -593,7 +641,11 @@ int cave_monster_count(struct chunk *c) {
 }
 
 /**
- * Return the number of doors/traps around (or under) the character.
+ * Return the number of matching grids around (or under) the character.
+ * \param grid If not NULL, *grid is set to the location of the last match.
+ * \param test Is the predicate to use when testing for a match.
+ * \param under If true, the character's grid is tested as well.
+ * Only tests grids that are known and fully in bounds.
  */
 int count_feats(struct loc *grid,
 				bool (*test)(struct chunk *c, struct loc grid), bool under)
@@ -622,7 +674,7 @@ int count_feats(struct loc *grid,
 		/* Count it */
 		++count;
 
-		/* Remember the location of the last door found */
+		/* Remember the location of the last match */
 		if (grid) {
 			*grid = grid1;
 		}
@@ -632,6 +684,44 @@ int count_feats(struct loc *grid,
 	return count;
 }
 
+/**
+ * Return the number of matching grids around a location.
+ * \param match If not NULL, *match is set to the location of the last match.
+ * \param c Is the chunk to use.
+ * \param grid Is the location whose neighbors will be tested.
+ * \param test Is the predicate to use when testing for a match.
+ * \param under If true, grid is tested as well.
+ */
+int count_neighbors(struct loc *match, struct chunk *c, struct loc grid,
+	bool (*test)(struct chunk *c, struct loc grid), bool under)
+{
+	int dlim = (under) ? 9 : 8;
+	int count = 0; /* Count how many matches */
+	int d;
+	struct loc grid1;
+
+	/* Check the grid's neighbors and, if under is true, grid */
+	for (d = 0; d < dlim; d++) {
+		/* Extract adjacent (legal) location */
+		grid1 = loc_sum(grid, ddgrid_ddd[d]);
+		if (!square_in_bounds(c, grid1)) continue;
+
+		/* Reject those that don't match */
+		if (!((*test)(c, grid1))) continue;
+
+		/* Count it */
+		++count;
+
+		/* Remember the location of the last match */
+		if (match) {
+			*match = grid1;
+		}
+	}
+
+	/* All done */
+	return count;
+}
+
 struct loc cave_find_decoy(struct chunk *c)
 {
 	return c->decoy;
