@@ -26,18 +26,18 @@ struct player;
 struct monster;
 struct monster_group;
 
-extern const s16b ddd[9];
-extern const s16b ddx[10];
-extern const s16b ddy[10];
+extern const int16_t ddd[9];
+extern const int16_t ddx[10];
+extern const int16_t ddy[10];
 extern const struct loc ddgrid[10];
-extern const s16b ddx_ddd[9];
-extern const s16b ddy_ddd[9];
+extern const int16_t ddx_ddd[9];
+extern const int16_t ddy_ddd[9];
 extern const struct loc ddgrid_ddd[9];
-extern const s16b clockwise_ddd[9];
+extern const int16_t clockwise_ddd[9];
 extern const struct loc clockwise_grid[9];
 extern const int *dist_offsets_y[10];
 extern const int *dist_offsets_x[10];
-extern const byte side_dirs[20][8];
+extern const uint8_t side_dirs[20][8];
 
 enum {
 	DIR_UNKNOWN = 0,
@@ -113,21 +113,24 @@ struct feature {
 
 	struct feature *next;
 
-	char *mimic;	/**< Name of feature to mimic */
-	byte priority;	/**< Display priority */
+	char *mimic;		/**< Name of feature to mimic */
+	uint8_t priority;	/**< Display priority */
 
-	byte shopnum;	/**< Which shop does it take you to? */
-	byte dig;      /**< How hard is it to dig through? */
+	uint8_t shopnum;	/**< Which shop does it take you to? */
+	uint8_t dig;		/**< How hard is it to dig through? */
 
 	bitflag flags[TF_SIZE];	/**< Terrain flags */
 
-	byte d_attr;	/**< Default feature attribute */
+	uint8_t d_attr;	/**< Default feature attribute */
 	wchar_t d_char;	/**< Default feature character */
 
 	char *walk_msg;	/**< Message on walking into feature */
 	char *run_msg;	/**< Message on running into feature */
 	char *hurt_msg;	/**< Message on being hurt by feature */
 	char *die_msg;	/**< Message on dying to feature */
+	char *confused_msg; /**< Message on confused monster move into feature */
+	char *look_prefix; /**< Prefix for name in look result */
+	char *look_in_preposition; /**< Preposition in look result when on the terrain */
 	int resist_flag;/**< Monster resist flag for entering feature */
 };
 
@@ -143,8 +146,8 @@ enum grid_light_level
 };
 
 struct grid_data {
-	u32b m_idx;				/* Monster index */
-	u32b f_idx;				/* Feature index */
+	uint32_t m_idx;			/* Monster index */
+	uint32_t f_idx;			/* Feature index */
 	struct object_kind *first_kind;	/* The kind of the first item on the grid */
 	struct trap *trap;		/* Trap */
 	bool multiple_objects;	/* Is there more than one item there? */
@@ -158,39 +161,39 @@ struct grid_data {
 };
 
 struct square {
-	byte feat;
+	uint8_t feat;
 	bitflag *info;
 	int light;
-	s16b mon;
+	int16_t mon;
 	struct object *obj;
 	struct trap *trap;
 };
 
 struct heatmap {
-    u16b **grids;
+	uint16_t **grids;
 };
 
 struct connector {
 	struct loc grid;
-	byte feat;
+	uint8_t feat;
 	bitflag *info;
 	struct connector *next;
 };
 
 struct chunk {
 	char *name;
-	s32b turn;
+	int32_t turn;
 	int depth;
 
-	byte feeling;
-	u32b obj_rating;
-	u32b mon_rating;
+	uint8_t feeling;
+	uint32_t obj_rating;
+	uint32_t mon_rating;
 	bool good_item;
 
 	int height;
 	int width;
 
-	u16b feeling_squares; /* How many feeling squares the player has visited */
+	uint16_t feeling_squares; /* How many feeling squares the player has visited */
 	int *feat_count;
 
 	struct square **squares;
@@ -199,11 +202,11 @@ struct chunk {
 	struct loc decoy;
 
 	struct object **objects;
-	u16b obj_max;
+	uint16_t obj_max;
 
 	struct monster *monsters;
-	u16b mon_max;
-	u16b mon_cnt;
+	uint16_t mon_max;
+	uint16_t mon_cnt;
 	int mon_current;
 	int num_repro;
 
@@ -248,13 +251,13 @@ extern int FEAT_LAVA;
 extern struct chunk *cave;
 /* Stored levels */
 extern struct chunk **chunk_list;
-extern u16b chunk_list_max;
+extern uint16_t chunk_list_max;
 
 /* cave-view.c */
 int distance(struct loc grid1, struct loc grid2);
 bool los(struct chunk *c, struct loc grid1, struct loc grid2);
 void update_view(struct chunk *c, struct player *p);
-bool no_light(void);
+bool no_light(struct player *p);
 
 /* cave-map.c */
 void map_info(struct loc grid, struct grid_data *g);
@@ -264,6 +267,7 @@ void light_room(struct loc grid, bool light);
 void wiz_light(struct chunk *c, struct player *p, bool full);
 void wiz_dark(struct chunk *c, struct player *p, bool full);
 void cave_illuminate(struct chunk *c, bool daytime);
+void expose_to_sun(struct chunk *c, struct loc grid, bool daytime);
 void cave_update_flow(struct chunk *c);
 void cave_forget_flow(struct chunk *c);
 
@@ -285,6 +289,7 @@ bool feat_is_trap_holding(int feat);
 bool feat_is_object_holding(int feat);
 bool feat_is_monster_walkable(int feat);
 bool feat_is_shop(int feat);
+bool feat_is_los(int feat);
 bool feat_is_passable(int feat);
 bool feat_is_projectable(int feat);
 bool feat_is_torch(int feat);
@@ -352,12 +357,11 @@ bool square_iswebbable(struct chunk *c, struct loc grid);
 bool square_is_monster_walkable(struct chunk *c, struct loc grid);
 bool square_ispassable(struct chunk *c, struct loc grid);
 bool square_isprojectable(struct chunk *c, struct loc grid);
-bool square_iswall(struct chunk *c, struct loc grid);
+bool square_allowslos(struct chunk *c, struct loc grid);
 bool square_isstrongwall(struct chunk *c, struct loc grid);
 bool square_isbright(struct chunk *c, struct loc grid);
 bool square_isfiery(struct chunk *c, struct loc grid);
 bool square_islit(struct chunk *c, struct loc grid);
-bool square_islitwall(struct chunk *c, struct loc grid);
 bool square_isdamaging(struct chunk *c, struct loc grid);
 bool square_isnoflow(struct chunk *c, struct loc grid);
 bool square_isnoscent(struct chunk *c, struct loc grid);
@@ -379,9 +383,10 @@ bool square_in_bounds_fully(struct chunk *c, struct loc grid);
 bool square_isbelievedwall(struct chunk *c, struct loc grid);
 bool square_suits_stairs_well(struct chunk *c, struct loc grid);
 bool square_suits_stairs_ok(struct chunk *c, struct loc grid);
+bool square_allows_summon(struct chunk *c, struct loc grid);
 
 
-struct square square(struct chunk *c, struct loc grid);
+const struct square *square(struct chunk *c, struct loc grid);
 struct feature *square_feat(struct chunk *c, struct loc grid);
 int square_light(struct chunk *c, struct loc grid);
 struct monster *square_monster(struct chunk *c, struct loc grid);
@@ -430,6 +435,8 @@ void square_force_floor(struct chunk *c, struct loc grid);
 int square_shopnum(struct chunk *c, struct loc grid);
 int square_digging(struct chunk *c, struct loc grid);
 const char *square_apparent_name(struct chunk *c, struct player *p, struct loc grid);
+const char *square_apparent_look_prefix(struct chunk *c, struct player *p, struct loc grid);
+const char *square_apparent_look_in_preposition(struct chunk *c, struct player *p, struct loc grid);
 
 void square_memorize(struct chunk *c, struct loc grid);
 void square_forget(struct chunk *c, struct loc grid);
@@ -442,12 +449,15 @@ struct loc next_grid(struct loc grid, int dir);
 int lookup_feat(const char *name);
 void set_terrain(void);
 struct chunk *cave_new(int height, int width);
+void cave_connectors_free(struct connector *join);
 void cave_free(struct chunk *c);
 void list_object(struct chunk *c, struct object *obj);
 void delist_object(struct chunk *c, struct object *obj);
 void object_lists_check_integrity(struct chunk *c, struct chunk *c_k);
 void scatter(struct chunk *c, struct loc *place, struct loc grid, int d,
 			 bool need_los);
+int scatter_ext(struct chunk *c, struct loc *places, int n, struct loc grid,
+		int d, bool need_los, bool (*pred)(struct chunk *, struct loc));
 
 struct monster *cave_monster(struct chunk *c, int idx);
 int cave_monster_max(struct chunk *c);
@@ -455,9 +465,9 @@ int cave_monster_count(struct chunk *c);
 
 int count_feats(struct loc *grid,
 				bool (*test)(struct chunk *c, struct loc grid), bool under);
+int count_neighbors(struct loc *match, struct chunk *c, struct loc grid,
+	bool (*test)(struct chunk *c, struct loc grid), bool under);
 struct loc cave_find_decoy(struct chunk *c);
-void prepare_next_level(struct chunk **c, struct player *p);
-bool is_quest(int level);
 
 void cave_known(struct player *p);
 
