@@ -44,7 +44,7 @@
  */
 static void player_pickup_gold(struct player *p)
 {
-	s32b total_gold = 0L;
+	int32_t total_gold = 0L;
 	char name[30] = "";
 
 	struct object *obj = square_object(cave, p->grid), *next;
@@ -74,11 +74,11 @@ static void player_pickup_gold(struct player *p)
 			my_strcpy(name, kind->name, sizeof(name));
 
 		/* Remember whether feedback message is in order */
-		if (!ignore_item_ok(obj))
+		if (!ignore_item_ok(p, obj))
 			verbal = true;
 
 		/* Increment total value */
-		total_gold += (s32b)obj->pval;
+		total_gold += (int32_t)obj->pval;
 
 		/* Delete the gold */
 		if (obj->known) {
@@ -130,7 +130,7 @@ static const struct object *find_stack_object_in_inventory(const struct object *
 	const struct object *gear_obj;
 	for (gear_obj = (start) ? start : player->gear; gear_obj; gear_obj = gear_obj->next) {
 		if (!object_is_equipped(player->body, gear_obj) &&
-				object_stackable(gear_obj, obj, OSTACK_PACK)) {
+				object_similar(gear_obj, obj, OSTACK_PACK)) {
 			/* We found the object */
 			return gear_obj;
 		}
@@ -167,7 +167,7 @@ static int auto_pickup_okay(const struct object *obj)
 	 * inscriptions.  The player option to pickup if in the inventory
 	 * honors those inscriptions.
 	 */
-	int num = inven_carry_num(obj, false);
+	int num = inven_carry_num(player, obj);
 	unsigned obj_has_auto, obj_has_maxauto;
 	int obj_maxauto;
 
@@ -236,7 +236,7 @@ static int auto_pickup_okay(const struct object *obj)
 static void player_pickup_aux(struct player *p, struct object *obj,
 							  int auto_max, bool domsg)
 {
-	int max = inven_carry_num(obj, false);
+	int max = inven_carry_num(p, obj);
 
 	/* Confirm at least some of the object can be picked up */
 	if (max == 0)
@@ -269,7 +269,7 @@ static void player_pickup_aux(struct player *p, struct object *obj,
 		else
 			num = get_quantity(NULL, max);
 		if (!num) return;
-		picked_up = floor_object_for_use(obj, num, false, &dummy);
+		picked_up = floor_object_for_use(p, obj, num, false, &dummy);
 		inven_carry(p, picked_up, true, domsg);
 	}
 }
@@ -300,7 +300,7 @@ static void player_pickup_aux(struct player *p, struct object *obj,
  * \param obj is the object to pick up.
  * \param menu is whether to present a menu to the player
  */
-static byte player_pickup_item(struct player *p, struct object *obj, bool menu)
+static uint8_t player_pickup_item(struct player *p, struct object *obj, bool menu)
 {
 	struct object *current = NULL;
 
@@ -315,7 +315,7 @@ static byte player_pickup_item(struct player *p, struct object *obj, bool menu)
 	bool domsg = true;
 
 	/* Objects picked up.  Used to determine time cost of command. */
-	byte objs_picked_up = 0;
+	uint8_t objs_picked_up = 0;
 
 	/* Always know what's on the floor */
 	square_know_pile(cave, p->grid);
@@ -338,9 +338,9 @@ static byte player_pickup_item(struct player *p, struct object *obj, bool menu)
 	}
 
 	/* Tally objects that can be at least partially picked up.*/
-	floor_num = scan_floor(floor_list, floor_max, OFLOOR_VISIBLE, NULL);
+	floor_num = scan_floor(floor_list, floor_max, p, OFLOOR_VISIBLE, NULL);
 	for (i = 0; i < floor_num; i++)
-	    if (inven_carry_okay(floor_list[i]))
+	    if (inven_carry_num(p, floor_list[i]) > 0)
 			can_pickup++;
 
 	if (!can_pickup) {
@@ -405,7 +405,7 @@ static byte player_pickup_item(struct player *p, struct object *obj, bool menu)
 int do_autopickup(struct player *p)
 {
 	struct object *obj, *next;
-	byte objs_picked_up = 0;
+	uint8_t objs_picked_up = 0;
 
 	/* Nothing to pick up -- return */
 	if (!square_object(cave, p->grid))
@@ -420,7 +420,7 @@ int do_autopickup(struct player *p)
 		next = obj->next;
 
 		/* Ignore all hidden objects and non-objects */
-		if (!ignore_item_ok(obj)) {
+		if (!ignore_item_ok(p, obj)) {
 			int auto_num;
 
 			/* Hack -- disturb */
