@@ -21,21 +21,12 @@
 
 #else
 
-/* Necessary? */
-#ifdef NDS
-# include <fat.h>
-# include <unistd.h>
-# include <reent.h>
-# include <sys/iosupport.h>
-# include <errno.h>
-#endif
-
 /**
  * Using C99, assume we have stdint and stdbool
  */
 # if (defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L) \
   || (defined(_MSC_VER) && _MSC_VER >= 1600L)
-#  define HAVE_STDINT_H
+#  define HAVE_STDINT_H 1
 # endif
 
 # if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
@@ -70,14 +61,15 @@
 # ifndef HAVE_DIRENT_H
 #  define HAVE_DIRENT_H
 # endif
-#endif
 
 /**
- * Define SETGID if we are running as a central install on a multiuser
- * system that has setgid support.
+ * May need to be tightened:  without autoconf.h assume all Unixes have mkdir().
  */
-/* #define SETGID */
+# if !defined(HAVE_MKDIR) && !defined(HAVE_CONFIG_H)
+#   define HAVE_MKDIR
+# endif
 
+#endif
 
 /**
  * Every system seems to use its own symbol as a path separator.
@@ -148,6 +140,10 @@
 /**
  * errr is an error code
  *
+ * byte/s16b/u16b/s32b/u32b/s64b/u64b have been deprecated; use the
+ * standard uint8_t, int16_t, uint16_t, ... types instead.  Those typedefs
+ * will still be defined if SKIP_ANGBAND_OLD_INT_TYPEDEFS is not set.
+ *
  * A "byte" is an unsigned byte of memory.
  * s16b/u16b are exactly 2 bytes (where possible)
  * s32b/u32b are exactly 4 bytes (where possible)
@@ -155,6 +151,7 @@
 
 typedef int errr;
 
+#ifndef SKIP_ANGBAND_OLD_INT_TYPEDEFS
 /* Use guaranteed-size types */
 typedef uint8_t byte;
 
@@ -166,7 +163,7 @@ typedef int32_t s32b;
 
 typedef uint64_t u64b;
 typedef int64_t s64b;
-
+#endif
 
 /** Debugging macros ***/
 
