@@ -49,6 +49,7 @@
 #include "savefile.h"
 #include "store.h"
 #include "trap.h"
+#include "ui-term.h"
 
 /**
  * Setting this to 1 and recompiling gives a chance to recover a savefile 
@@ -60,32 +61,32 @@
 /**
  * Dungeon constants
  */
-static byte square_size = 0;
+static uint8_t square_size = 0;
 
 /**
  * Player constants
  */
-static byte hist_size = 0;
+static uint8_t hist_size = 0;
 
 /**
  * Object constants
  */
-static byte obj_mod_max = 0;
-static byte of_size = 0;
-static byte elem_max = 0;
-static byte brand_max;
-static byte slay_max;
-static byte curse_max;
+static uint8_t obj_mod_max = 0;
+static uint8_t of_size = 0;
+static uint8_t elem_max = 0;
+static uint8_t brand_max;
+static uint8_t slay_max;
+static uint8_t curse_max;
 
 /**
  * Monster constants
  */
-static byte mflag_size = 0;
+static uint8_t mflag_size = 0;
 
 /**
  * Trap constants
  */
-static byte trf_size = 0;
+static uint8_t trf_size = 0;
 
 /**
  * Shorthand function pointer for rd_item version
@@ -99,12 +100,12 @@ static struct object *rd_item(void)
 {
 	struct object *obj = object_new();
 
-	byte tmp8u;
-	u16b tmp16u;
-	byte effect;
+	uint8_t tmp8u;
+	uint16_t tmp16u;
+	uint8_t effect;
 	size_t i;
 	char buf[128];
-	byte ver = 1;
+	uint8_t ver = 1;
 
 	rd_u16b(&tmp16u);
 	rd_byte(&ver);
@@ -239,7 +240,7 @@ static struct object *rd_item(void)
 
 	/* Check we have a kind */
 	if ((!obj->tval && !obj->sval) || !obj->kind) {
-		object_delete(&obj);
+		object_delete(NULL, NULL, &obj);
 		return NULL;
 	}
 
@@ -260,10 +261,11 @@ static struct object *rd_item(void)
  */
 static bool rd_monster(struct chunk *c, struct monster *mon)
 {
-	byte tmp8u;
-	u16b tmp16u;
+	uint8_t tmp8u;
+	uint16_t tmp16u;
 	char race_name[80];
 	size_t j;
+	bool delete = false;
 
 	/* Read the monster race */
 	rd_u16b(&tmp16u);
@@ -311,7 +313,7 @@ static bool rd_monster(struct chunk *c, struct monster *mon)
 		/* Find and set the mimicked object */
 		struct object *square_obj = square_object(c, mon->grid);
 
-		/* Try and find the mimicked object; if we fail, create a new one */
+		/* Try and find the mimicked object; if we fail, delete the monster */
 		while (square_obj) {
 			if (square_obj->mimicking_m_idx == tmp16u) break;
 			square_obj = square_obj->next;
@@ -319,7 +321,7 @@ static bool rd_monster(struct chunk *c, struct monster *mon)
 		if (square_obj) {
 			mon->mimicked_obj = square_obj;
 		} else {
-			mon_create_mimicked_object(c, mon, tmp16u);
+			delete = true;
 		}
 	}
 
@@ -345,6 +347,11 @@ static bool rd_monster(struct chunk *c, struct monster *mon)
 	rd_byte(&tmp8u);
 	mon->group_info[SUMMON_GROUP].role = tmp8u;
 
+	/* Now delete the monster if necessary */
+	if (delete) {
+		delete_monster(mon->grid);
+	}
+
 	return true;
 }
 
@@ -355,7 +362,7 @@ static bool rd_monster(struct chunk *c, struct monster *mon)
 static void rd_trap(struct trap *trap)
 {
 	int i;
-	byte tmp8u;
+	uint8_t tmp8u;
 	char buf[80];
 
 	rd_string(buf, sizeof(buf));
@@ -384,7 +391,7 @@ static void rd_trap(struct trap *trap)
 int rd_randomizer(void)
 {
 	int i;
-	u32b noop;
+	uint32_t noop;
 
 	/* current value for the simple RNG */
 	rd_u32b(&Rand_value);
@@ -419,9 +426,7 @@ int rd_randomizer(void)
  */
 int rd_options(void)
 {
-	byte b;
-
-	u16b tmp16u;
+	uint8_t b;
 
 	/*** Special info */
 
@@ -434,13 +439,22 @@ int rd_options(void)
 	player->opts.hitpoint_warn = b;
 
 	/* Read lazy movement delay */
-	rd_u16b(&tmp16u);
-	player->opts.lazymove_delay = (tmp16u < 1000) ? tmp16u : 0;
+	rd_byte(&b);
+	player->opts.lazymove_delay = b;
+
+	/* Read sidebar mode (if it's an actual game) */
+	if (angband_term[0]) {
+		rd_byte(&b);
+		if (b >= SIDEBAR_MAX) b = SIDEBAR_LEFT;
+		SIDEBAR_MODE = b;
+	} else {
+		strip_bytes(1);
+	}
 
 
 	/* Read options */
 	while (1) {
-		byte value;
+		uint8_t value;
 		char name[40];
 		rd_string(name, sizeof name);
 
@@ -461,9 +475,9 @@ int rd_messages(void)
 {
 	int i;
 	char buf[128];
-	u16b tmp16u;
+	uint16_t tmp16u;
 
-	s16b num;
+	int16_t num;
 
 	/* Total */
 	rd_s16b(&num);
@@ -488,7 +502,7 @@ int rd_messages(void)
  */
 int rd_monster_memory(void)
 {
-	u16b nkill, ntheft;
+	uint16_t nkill, ntheft;
 	char buf[128];
 	int i;
 
@@ -537,7 +551,7 @@ int rd_monster_memory(void)
 int rd_object_memory(void)
 {
 	size_t i;
-	u16b tmp16u;
+	uint16_t tmp16u;
 
 	/* Object Memory */
 	rd_u16b(&tmp16u);
@@ -591,7 +605,7 @@ int rd_object_memory(void)
 
 	/* Read the kind knowledge */
 	for (i = 0; i < tmp16u; i++) {
-		byte tmp8u;
+		uint8_t tmp8u;
 		struct object_kind *kind = &k_info[i];
 
 		rd_byte(&tmp8u);
@@ -612,7 +626,7 @@ int rd_object_memory(void)
 int rd_quests(void)
 {
 	int i;
-	u16b tmp16u;
+	uint16_t tmp16u;
 
 	/* Load the Quests */
 	rd_u16b(&tmp16u);
@@ -624,7 +638,7 @@ int rd_quests(void)
 	/* Load the Quests */
 	player_quests_reset(player);
 	for (i = 0; i < tmp16u; i++) {
-		u16b cur_num;
+		uint16_t cur_num;
 		rd_byte(&player->quests[i].level);
 		rd_u16b(&cur_num);
 		player->quests[i].cur_num = cur_num;
@@ -640,8 +654,8 @@ int rd_quests(void)
 int rd_player(void)
 {
 	int i;
-	byte num;
-	byte stat_max = 0;
+	uint8_t tmp8u, num;
+	uint8_t stat_max = 0;
 	char buf[80];
 	struct player_race *r;
 	struct player_shape *s;
@@ -783,7 +797,11 @@ int rd_player(void)
 				  sizeof(player->died_from));
 
 	/* More info */
-	strip_bytes(7);
+	rd_byte(&tmp8u);
+	player->old_grid.y = tmp8u;
+	rd_byte(&tmp8u);
+	player->old_grid.x = tmp8u;
+	strip_bytes(5);
 	rd_byte(&player->unignoring);
 	rd_s16b(&player->deep_descent);
 
@@ -801,7 +819,7 @@ int rd_player(void)
 
 		/* Initialize any entries not read */
 		if (num < TMD_MAX)
-			memset(player->timed + num, 0, (TMD_MAX - num) * sizeof(s16b));
+			memset(player->timed + num, 0, (TMD_MAX - num) * sizeof(int16_t));
 	} else {
 		/* Probably in trouble anyway */
 		for (i = 0; i < TMD_MAX; i++)
@@ -830,10 +848,10 @@ int rd_player(void)
 int rd_ignore(void)
 {
 	size_t i, j;
-	byte tmp8u = 24;
-	u16b file_e_max;
-	u16b itype_size;
-	u16b inscriptions;
+	uint8_t tmp8u = 24;
+	uint16_t file_e_max;
+	uint16_t itype_size;
+	uint16_t inscriptions;
 
 	/* Read how many ignore bytes we have */
 	rd_byte(&tmp8u);
@@ -881,7 +899,7 @@ int rd_ignore(void)
 	/* Read the aware object autoinscriptions array */
 	for (i = 0; i < inscriptions; i++) {
 		char tmp[80];
-		byte tval, sval;
+		uint8_t tval, sval;
 		struct object_kind *k;
 
 		rd_string(tmp, sizeof(tmp));
@@ -901,7 +919,7 @@ int rd_ignore(void)
 	/* Read the unaware object autoinscriptions array */
 	for (i = 0; i < inscriptions; i++) {
 		char tmp[80];
-		byte tval, sval;
+		uint8_t tval, sval;
 		struct object_kind *k;
 
 		rd_string(tmp, sizeof(tmp));
@@ -921,7 +939,7 @@ int rd_ignore(void)
 	/* Read the rune autoinscriptions array */
 	for (i = 0; i < inscriptions; i++) {
 		char tmp[80];
-		s16b runeid;
+		int16_t runeid;
 
 		rd_s16b(&runeid);
 		rd_string(tmp, sizeof(tmp));
@@ -935,7 +953,7 @@ int rd_ignore(void)
 int rd_misc(void)
 {
 	size_t i;
-	byte tmp8u;
+	uint8_t tmp8u;
 	
 	/* Read the randart seed */
 	rd_u32b(&seed_randart);
@@ -961,9 +979,13 @@ int rd_misc(void)
 
 	/* Handle randart file parsing */
 	if (OPT(player, birth_randarts)) {
-		cleanup_parser(&artifact_parser);
-		activate_randart_file();
-		run_parser(&randart_parser);
+		if (randart_file_exists()) {
+			cleanup_parser(&artifact_parser);
+			activate_randart_file();
+			run_parser(&randart_parser);
+		} else {
+			do_randart(seed_randart, true);
+		}
 		deactivate_randart_file();
 	}
 
@@ -1014,7 +1036,7 @@ int rd_misc(void)
 int rd_artifacts(void)
 {
 	int i;
-	u16b tmp16u;
+	uint16_t tmp16u;
 
 	/* Load the Artifacts */
 	rd_u16b(&tmp16u);
@@ -1025,14 +1047,14 @@ int rd_artifacts(void)
 
 	/* Read the artifact flags */
 	for (i = 0; i < tmp16u; i++) {
-		byte tmp8u;
+		uint8_t tmp8u;
 
 		rd_byte(&tmp8u);
-		a_info[i].created = tmp8u;
+		aup_info[i].created = tmp8u ? true : false;
 		rd_byte(&tmp8u);
-		a_info[i].seen = tmp8u;
+		aup_info[i].seen = tmp8u ? true : false;
 		rd_byte(&tmp8u);
-		a_info[i].everseen = tmp8u;
+		aup_info[i].everseen = tmp8u ? true : false;
 		rd_byte(&tmp8u);
 	}
 
@@ -1044,7 +1066,7 @@ int rd_artifacts(void)
 int rd_player_hp(void)
 {
 	int i;
-	u16b tmp16u;
+	uint16_t tmp16u;
 
 	/* Read the player_hp array */
 	rd_u16b(&tmp16u);
@@ -1067,7 +1089,7 @@ int rd_player_hp(void)
 int rd_player_spells(void)
 {
 	int i;
-	u16b tmp16u;
+	uint16_t tmp16u;
 	
 	int cnt;
 	
@@ -1101,7 +1123,7 @@ int rd_player_spells(void)
  */
 static int rd_gear_aux(rd_item_t rd_item_version, struct object **gear)
 {
-	byte code;
+	uint8_t code;
 	struct object *last_gear_obj = NULL;
 
 	/* Get the first item code */
@@ -1161,8 +1183,7 @@ int rd_gear(void)
 		player->upkeep->total_weight += (obj->number * obj->weight);
 	}
 
-	/* Maybe we have to duplicate also upkeep and body */
-	calc_inventory(player->upkeep, player->gear, player->body);
+	calc_inventory(player);
 
 	return 0;
 }
@@ -1174,14 +1195,14 @@ int rd_gear(void)
 static int rd_stores_aux(rd_item_t rd_item_version)
 {
 	int i;
-	u16b tmp16u;
+	uint16_t tmp16u;
 
 	/* Read the stores */
 	rd_u16b(&tmp16u);
 	for (i = 0; i < tmp16u; i++) {
 		struct store *store = &stores[i];
 
-		byte own, num;
+		uint8_t own, num;
 
 		/* Read the basic info */
 		rd_byte(&own);
@@ -1248,16 +1269,16 @@ static int rd_dungeon_aux(struct chunk **c)
 	struct chunk *c1;
 	int i, n, y, x;
 
-	u16b height, width;
+	uint16_t height, width;
 
-	byte count;
-	byte tmp8u;
-	u16b tmp16u;
+	uint8_t count;
+	uint8_t tmp8u;
+	uint16_t tmp16u;
 	char name[100];
 
 	/* Header info */
 	rd_string(name, sizeof(name));
-	if (streq(name, "arena")) {
+	if (streq(name, "arena") && (*c == cave)) {
 		player->upkeep->arena_level = true;
 	}
 	rd_u16b(&height);
@@ -1326,7 +1347,6 @@ static int rd_dungeon_aux(struct chunk **c)
 	if (OPT(player, birth_levels_persist)) {
 		rd_byte(&tmp8u);
 		while (tmp8u != 0xff) {
-			size_t n;
 			struct connector *current = mem_zalloc(sizeof *current);
 			current->info = mem_zalloc(square_size * sizeof(bitflag));
 			current->grid.x = tmp8u;
@@ -1392,7 +1412,7 @@ static int rd_objects_aux(rd_item_t rd_item_version, struct chunk *c)
 static int rd_monsters_aux(struct chunk *c)
 {
 	int i;
-	u16b limit;
+	uint16_t limit;
 
 	/* Only if the player's alive */
 	if (player->is_dead)
@@ -1467,8 +1487,8 @@ static int rd_traps_aux(struct chunk *c)
 
 int rd_dungeon(void)
 {
-	u16b depth;
-	u16b py, px;
+	uint16_t depth;
+	uint16_t py, px;
 
 	/* Header info */
 	rd_u16b(&depth);
@@ -1583,7 +1603,7 @@ int rd_traps(void)
 int rd_chunks(void)
 {
 	int j;
-	u16b chunk_max;
+	uint16_t chunk_max;
 
 	if (player->is_dead)
 		return 0;
@@ -1613,10 +1633,11 @@ int rd_chunks(void)
 		if (OPT(player, birth_levels_persist)) {
 			char buf[80];
 			int i;
-			byte tmp8u;
-			u16b tmp16u;
+			uint8_t tmp8u;
+			uint16_t tmp16u;
 
 			rd_string(buf, sizeof(buf));
+			string_free(c->name);
 			c->name = string_make(buf);
 			rd_s32b(&c->turn);
 			rd_u16b(&tmp16u);
@@ -1635,6 +1656,21 @@ int rd_chunks(void)
 				rd_u16b(&tmp16u);
 				c->feat_count[i] = tmp16u;
 			}
+		} else if (c->name) {
+			struct level *lev = level_by_name(c->name);
+
+			if (lev) {
+				c->depth = lev->depth;
+			} else if (suffix(c->name, " known")) {
+				size_t offset = strlen(c->name) -
+					strlen(" known");
+				c->name[offset] = '\0';
+				lev = level_by_name(c->name);
+				if (lev) {
+					c->depth = lev->depth;
+				}
+				c->name[offset] = ' ';
+			}
 		}
 
 		chunk_list_add(c);
@@ -1658,7 +1694,7 @@ int rd_chunks(void)
 
 int rd_history(void)
 {
-	u32b tmp32u;
+	uint32_t tmp32u;
 	size_t i, j;
 	
 	history_clear(player);
@@ -1672,10 +1708,10 @@ int rd_history(void)
 
 	rd_u32b(&tmp32u);
 	for (i = 0; i < tmp32u; i++) {
-		s32b turnno;
-		s16b dlev, clev;
+		int32_t turnno;
+		int16_t dlev, clev;
 		bitflag type[HIST_SIZE];
-		struct artifact *art = NULL;
+		const struct artifact *art = NULL;
 		int aidx = 0;
 		char name[80];
 		char text[80];
