@@ -183,6 +183,7 @@ static int same_colortable[BASIC_COLORS];
 /* Screen info: use one big Term 0, or other subwindows? */
 static bool bold_extended = false;
 static bool use_default_background = false;
+static bool keep_terminal_colors = false;
 static int term_count = 1;
 
 /**
@@ -328,7 +329,11 @@ static errr Term_xtra_gcu_alive(int v) {
 	return 0;
 }
 
-const char help_gcu[] = "Text mode, subopts\n              -B     Use brighter bold characters\n              -D     Use terminal default background color\n              -nN    Use N terminals (up to 6)";
+const char help_gcu[] = "Text mode, subopts\n"
+	"              -B     Use brighter bold characters\n"
+	"              -D     Use terminal default background color\n"
+	"              -K     Keep terminal's color table when changing colors\n"
+	"              -nN    Use N terminals (up to 6)";
 
 /**
  * Usage:
@@ -728,36 +733,81 @@ static int create_color(int i, int scale) {
 
 
 /**
- * React to changes
+ * Adjust the color tables if there's more than 16 available.
  */
-static errr Term_xtra_gcu_react(void) {
+static void handle_extended_color_tables(void) {
 #ifdef A_COLOR
 	if (COLORS == 256 || COLORS == 88) {
-		/* If we have more than 16 colors, find the best matches. These numbers
-		 * correspond to xterm/rxvt's builtin color numbers--they do not
-		 * correspond to curses' constants OR with curses' color pairs.
-		 *
-		 * XTerm has 216 (6*6*6) RGB colors, with each RGB setting 0-5.
-		 * RXVT has 64 (4*4*4) RGB colors, with each RGB setting 0-3.
-		 *
-		 * Both also have the basic 16 ANSI colors, plus some extra grayscale
-		 * colors which we do not use.
-		 */
+		int isbold = bold_extended ?  A_BRIGHT : A_NORMAL;
 		int i;
-		int scale = COLORS == 256 ? 6 : 4;
 
-		for (i = 0; i < BASIC_COLORS; i++) {
-			int fg = create_color(i, scale);
-			int isbold = bold_extended ? A_BRIGHT : A_NORMAL;
-			init_pair(i + 1, fg, bg_color);
-			colortable[i] = COLOR_PAIR(i + 1) | isbold;
-			init_pair(BASIC_COLORS + i, fg, fg);
-			same_colortable[i] = COLOR_PAIR(BASIC_COLORS + i) | isbold;
+		if (keep_terminal_colors) {
+			/*
+			 * If we have more than 16 colors, find the best
+			 * matches in the terminal's default color table.
+			 * These numbers correspond to xterm/rxvt's builtin
+			 * color numbers--they do not correspond to curses'
+			 * constants OR with curses' color pairs.
+			 *
+			 * XTerm has 216 (6*6*6) RGB colors, with each RGB
+			 * setting 0-5.
+			 * RXVT has 64 (4*4*4) RGB colors, with each RGB
+			 * setting 0-3.
+			 *
+			 * Both also have the basic 16 ANSI colors, plus some
+			 * extra grayscale colors which we do not use.
+			 */
+			int scale = COLORS == 256 ? 6 : 4;
+
+			bg_color = create_color(COLOUR_DARK, scale);
+			for (i = 0; i < BASIC_COLORS; i++) {
+				int fg = create_color(i, scale);
+				init_pair(i + 1, fg, bg_color);
+				colortable[i] = COLOR_PAIR(i + 1) | isbold;
+				init_pair(BASIC_COLORS + i, fg, fg);
+				same_colortable[i] =
+					COLOR_PAIR(BASIC_COLORS + i) | isbold;
+			}
+		} else {
+			bg_color = 0;
+			for (i = 0; i < BASIC_COLORS; i++) {
+				/*
+				 * Scale components to a range of 0 - 1000 per
+				 * per init_color()'s documentation.
+				 */
+				init_color(i,
+					(angband_color_table[i][1] * 1001) / 256,
+					(angband_color_table[i][2] * 1001) / 256,
+					(angband_color_table[i][3] * 1001) / 256);
+				init_pair(i + 1, i, bg_color);
+				colortable[i] = COLOR_PAIR(i + 1) | isbold;
+				init_pair(BASIC_COLORS + i, i, i);
+				same_colortable[i] =
+					COLOR_PAIR(BASIC_COLORS + i) | isbold;
+			}
+		}
+
+		for (i = 0; i < term_count; ++i) {
+			if (data[i].win) {
+				wbkgdset(data[i].win, ' ' |
+					colortable[COLOUR_DARK]);
+			}
+		}
+		if (data[0].win) {
+			/*
+			 * Adjust the background color on the standard screen
+			 * as well so separators between the terminals have
+			 * the same background as the rest.
+			 */
+			chtype term0_bkg = getbkgd(data[0].win);
+
+			if (getbkgd(stdscr) != term0_bkg) {
+				wbkgd(stdscr, term0_bkg);
+				wrefresh(stdscr);
+			}
 		}
 	}
 #endif
-
-	return 0;
 }
 
 
@@ -796,7 +846,7 @@ static errr Term_xtra_gcu(int n, int v) {
 		case TERM_XTRA_DELAY: if (v > 0) usleep(1000 * v); return 0;
 
 		/* React to events */
-		case TERM_XTRA_REACT: Term_xtra_gcu_react(); return 0;
+		case TERM_XTRA_REACT: handle_extended_color_tables(); return 0;
 	}
 
 	/* Unknown event */
@@ -823,12 +873,19 @@ static errr Term_wipe_gcu(int x, int y, int n) {
 
 	wmove(td->win, y, x);
 
-	if (x + n >= td->t.wid)
+	if (x + n >= td->t.wid) {
 		/* Clear to end of line */
 		wclrtoeol(td->win);
-	else
+	} else {
 		/* Clear some characters */
+		if (can_use_color) {
+			wattrset(td->win, colortable[COLOUR_DARK] | A_NORMAL);
+		}
 		whline(td->win, ' ', n);
+		if (can_use_color) {
+			wattrset(td->win, A_NORMAL);
+		}
+	}
 
 	return 0;
 }
@@ -1006,6 +1063,8 @@ errr init_gcu(int argc, char **argv) {
 			else if (term_count < 1) term_count = 1;
 		} else if (prefix(argv[i], "-D")) {
 			use_default_background = true;
+		} else if (streq(argv[i], "-K")) {
+			keep_terminal_colors = true;
 		}
 	}
 
@@ -1014,8 +1073,14 @@ errr init_gcu(int argc, char **argv) {
 
 	/* We do it like this to prevent a link error with curseses that
 	 * lack ESCDELAY. */
-	if (!getenv("ESCDELAY"))
-		putenv("ESCDELAY=20");
+	if (!getenv("ESCDELAY")) {
+#if _POSIX_C_SOURCE < 200112L
+		static char escdelbuf[80] = "ESCDELAY=20";
+		putenv(escdelbuf);
+#else
+		setenv("ESCDELAY", "20", 1);
+#endif
+	}
 
 	/* Initialize */
 	if (initscr() == NULL) return (-1);
@@ -1031,6 +1096,7 @@ errr init_gcu(int argc, char **argv) {
 	/* Do we have color, and enough color, available? */
 	can_use_color = ((start_color() != ERR) && has_colors() &&
 					 (COLORS >= 8) && (COLOR_PAIRS >= 8));
+	if (!can_change_color()) keep_terminal_colors = true;
 
 #ifdef HAVE_USE_DEFAULT_COLORS
 	/* Should we use curses' "default color" */
@@ -1119,6 +1185,7 @@ errr init_gcu(int argc, char **argv) {
 		same_colortable[COLOUR_MUSTARD]     = (COLOR_PAIR(PAIR_YELLOW));
 		same_colortable[COLOUR_BLUE_SLATE]  = (COLOR_PAIR(PAIR_BLUE));
 		same_colortable[COLOUR_DEEP_L_BLUE] = (COLOR_PAIR(PAIR_BLUE));
+		handle_extended_color_tables();
 	}
 #endif
 
@@ -1198,17 +1265,17 @@ errr init_gcu(int argc, char **argv) {
 
         for (i = 1; i < argc; i++)
         {
-            if (strcmp(argv[i], "-spacer") == 0)
+            if (streq(argv[i], "-spacer"))
             {
                 i++;
                 if (i >= argc)
                     quit("Missing size specifier for -spacer");
                 sscanf(argv[i], "%dx%d", &spacer_cx, &spacer_cy);
             }
-            else if (strcmp(argv[i], "-right") == 0 || strcmp(argv[i], "-left") == 0)
+            else if (streq(argv[i], "-right") || streq(argv[i], "-left"))
             {
                 const char *arg, *tmp;
-                bool left = strcmp(argv[i], "-left") == 0;
+                bool left = streq(argv[i], "-left");
                 int  cx, cys[MAX_TERM_DATA] = {0}, ct, j, x, y;
 
                 i++;
@@ -1255,10 +1322,10 @@ errr init_gcu(int argc, char **argv) {
                     term_ct++;
                 }
             }
-            else if (strcmp(argv[i], "-top") == 0 || strcmp(argv[i], "-bottom") == 0)
+            else if (streq(argv[i], "-top") || streq(argv[i], "-bottom"))
             {
                 const char *arg, *tmp;
-                bool top = strcmp(argv[i], "-top") == 0;
+                bool top = streq(argv[i], "-top");
                 int  cy, cxs[MAX_TERM_DATA] = {0}, ct, j, x, y;
 
                 i++;
