@@ -304,14 +304,14 @@ struct infowin
 	Window win;
 	long mask;
 
-	s16b ox, oy;
+	int16_t ox, oy;
 
-	s16b x, y;
-	s16b x_save, y_save;
-	s16b w, h;
-	u16b b;
+	int16_t x, y;
+	int16_t x_save, y_save;
+	int16_t w, h;
+	uint16_t b;
 
-	byte byte1;
+	uint8_t byte1;
 
 	unsigned int mapped:1;
 	unsigned int redraw:1;
@@ -378,12 +378,12 @@ struct infofnt
 
 	const char *name;
 
-	s16b wid;
-	s16b twid;
-	s16b hgt;
-	s16b asc;
+	int16_t wid;
+	int16_t twid;
+	int16_t hgt;
+	int16_t asc;
 
-	byte off;
+	uint8_t off;
 
 	unsigned int mono:1;
 	unsigned int nuke:1;
@@ -494,7 +494,8 @@ static int gamma_val = 0;
 /**
  * Hack -- Convert an RGB value to an X11 Pixel, or die.
  */
-static u32b create_pixel(Display *dpy, byte red, byte green, byte blue)
+static uint32_t create_pixel(Display *dpy, uint8_t red, uint8_t green,
+		uint8_t blue)
 {
 	Colormap cmap = DefaultColormapOfScreen(DefaultScreenOfDisplay(dpy));
 
@@ -584,7 +585,7 @@ static unsigned int xkb_mask_modifier( XkbDescPtr xkb, const char *name )
 {
 	unsigned int mask=0;
 	
-	if (strcmp(name, "Caps Lock") == 0) return 2;
+	if (streq(name, "Caps Lock")) return 2;
 	
 	for (int i = 0; (!mask) && (i < XkbNumVirtualMods); i++ ) {
 		char* modStr;
@@ -595,7 +596,7 @@ static unsigned int xkb_mask_modifier( XkbDescPtr xkb, const char *name )
 		}
 		modStr = XGetAtomName( xkb->dpy, xkb->names->vmods[i] );
 		if (modStr) {
-			if (!strcmp(name, modStr))
+			if (streq(name, modStr))
 				XkbVirtualModsToReal( xkb, 1 << i, &mask );
 
 			XFree(modStr);
@@ -1333,6 +1334,33 @@ static errr Infoclr_change_fg(Pixell fg)
 
 
 
+/**
+ * Change the 'bg' for an infoclr
+ *
+ * Inputs:
+ *	bg:   The Pixell for the requested Background (see above)
+ */
+static errr Infoclr_change_bg(Pixell bg)
+{
+	infoclr *iclr = Infoclr;
+
+
+	/*** Simple error checking of opr and clr ***/
+
+	/* Check the 'Pixells' for realism */
+	if (bg > Metadpy->zg) return (-1);
+
+	/*** Change ***/
+
+	/* Change */
+	XSetBackground(Metadpy->dpy, iclr->gc, bg);
+
+	/* Success */
+	return (0);
+}
+
+
+
 /**
  * Nuke an old 'infofnt'.
  */
@@ -1566,7 +1594,7 @@ static infoclr *xor;
 /**
  * Color info (unused, red, green, blue).
  */
-static byte color_table_x11[MAX_COLORS][4];
+static uint8_t color_table_x11[MAX_COLORS][4];
 
 
 /**
@@ -1614,7 +1642,7 @@ static void react_keypress(XKeyEvent *ev)
 	int mx = (ev->state & m->super_mask) ? true : false;
 	int kp = false;
 
-	byte mods = (mo ? KC_MOD_ALT : 0) | (mx ? KC_MOD_META : 0);
+	uint8_t mods = (mo ? KC_MOD_ALT : 0) | (mx ? KC_MOD_META : 0);
 
 	/* Check for "normal" keypresses */
 	n = XLookupString(ev, buf, 125, &ks, NULL);
@@ -1802,7 +1830,7 @@ static errr CheckEvent(bool wait)
 			else z = 0;
 
 			/* Save a byte in ui-term/Term_mousepress for some reason */
-			u32b state = ((XButtonEvent*) xev)->state;
+			uint32_t state = ((XButtonEvent*) xev)->state;
 			if(state & ShiftMask) {
 				z |= (KC_MOD_SHIFT << 4);
 			}
@@ -1955,6 +1983,8 @@ static errr Term_xtra_x11_react(void)
 	int i;
 
 	if (Metadpy->color) {
+		XSetWindowAttributes xattr;
+
 		/* Check the colors */
 		for (i = 0; i < MAX_COLORS; i++) {
 			if ((color_table_x11[i][0] != angband_color_table[i][0]) ||
@@ -1978,8 +2008,32 @@ static errr Term_xtra_x11_react(void)
 				/* Change the foreground */
 				Infoclr_set(clr[i]);
 				Infoclr_change_fg(pixel);
+
+				if (i == COLOUR_DARK) {
+					int j;
+
+					Metadpy->bg = pixel;
+					/* Change the background */
+					for (j = 0; j < MAX_COLORS; ++j) {
+						Infoclr_set(clr[j]);
+						Infoclr_change_bg(pixel);
+					}
+				} else if (i == COLOUR_WHITE) {
+					Metadpy->fg = pixel;
+				}
 			}
 		}
+
+		xattr.background_pixel = Metadpy->bg;
+		for (i = 0; i < ANGBAND_TERM_MAX; ++i) {
+			term_data *td;
+
+			if (!angband_term[i]) continue;
+			td = (term_data*)(angband_term[i]->data);
+			if (!td || !td->win) continue;
+			XChangeWindowAttributes(Metadpy->dpy, td->win->win,
+				CWBackPixel, &xattr);
+		}
 	}
 
 	/* Success */
@@ -2485,6 +2539,7 @@ static errr term_data_init(term_data *td, int i)
 		wmh->window_group = data[0].win->win;
 	}
 	XSetWMHints(Metadpy->dpy, Infowin->win, wmh);
+	XFree(wmh);
 
 	/* Map the window */
 	Infowin_map();
