@@ -73,6 +73,7 @@ static blow_tag_t blow_tag_lookup(const char *tag)
  */
 char *monster_blow_method_action(struct blow_method *method, int midx)
 {
+	const char punct[] = ".!?;:,'";
 	char buf[1024] = "\0";
 	const char *next;
 	const char *s;
@@ -114,7 +115,14 @@ char *monster_blow_method_action(struct blow_method *method, int midx)
 				case BLOW_TAG_TARGET: {
 					char m_name[80];
 					if (midx > 0) {
-						monster_desc(m_name, sizeof(m_name), t_mon, MDESC_TARG);
+						int mdesc_mode = MDESC_TARG;
+
+						if (!strchr(punct, *in_cursor)) {
+							mdesc_mode |= MDESC_COMMA;
+						}
+						monster_desc(m_name,
+							sizeof(m_name), t_mon,
+							mdesc_mode);
 						strnfcat(buf, sizeof(buf), &end, m_name);
 					} else {
 						strnfcat(buf, sizeof(buf), &end, "you");
@@ -124,9 +132,10 @@ char *monster_blow_method_action(struct blow_method *method, int midx)
 				case BLOW_TAG_OF_TARGET: {
 					char m_name[80];
 					if (midx > 0) {
-						monster_desc(m_name, sizeof(m_name), t_mon, MDESC_TARG);
+						monster_desc(m_name,
+							sizeof(m_name), t_mon,
+							MDESC_TARG | MDESC_POSS);
 						strnfcat(buf, sizeof(buf), &end, m_name);
-						strnfcat(buf, sizeof(buf), &end, "'s");
 					} else {
 						strnfcat(buf, sizeof(buf), &end, "your");
 					}
@@ -199,7 +208,7 @@ static void steal_player_item(melee_effect_handler_context_t *context)
         if (obj->artifact) continue;
 
         /* Get a description */
-        object_desc(o_name, sizeof(o_name), obj, ODESC_FULL);
+        object_desc(o_name, sizeof(o_name), obj, ODESC_FULL, context->p);
 
 		/* Is it one of a stack being stolen? */
 		if (obj->number > 1)
@@ -222,7 +231,8 @@ static void steal_player_item(melee_effect_handler_context_t *context)
 				o_name, I2A(index));
 
 			/* Steal and carry */
-			stolen = gear_object_for_use(obj, 1, false, &none_left);
+			stolen = gear_object_for_use(context->p, obj, 1,
+				false, &none_left);
 			(void)monster_carry(cave, context->mon, stolen);
 		}
 
@@ -512,7 +522,7 @@ static void melee_effect_experience(melee_effect_handler_context_t *context,
 	if (player_of_has(context->p, OF_HOLD_LIFE) && (randint0(100) < chance)) {
 		msg("You keep hold of your life force!");
 	} else {
-		s32b d = drain_amount +
+		int32_t d = drain_amount +
 			(context->p->exp/100) * z_info->life_drain_percent;
 		if (player_of_has(context->p, OF_HOLD_LIFE)) {
 			msg("You feel your life slipping away!");
@@ -681,7 +691,7 @@ static void melee_effect_handler_EAT_GOLD(melee_effect_handler_context_t *contex
         /* Occasional blink anyway */
         if (randint0(3)) context->blinked = true;
     } else {
-        s32b gold = (current_player->au / 10) + randint1(25);
+        int32_t gold = (current_player->au / 10) + randint1(25);
         if (gold < 2) gold = 2;
         if (gold > 5000) gold = (current_player->au / 20) + randint1(3000);
         if (gold > current_player->au) gold = current_player->au;
@@ -714,7 +724,7 @@ static void melee_effect_handler_EAT_GOLD(melee_effect_handler_context_t *contex
             /* Set origin to stolen, so it is not confused with
              * dropped treasure in monster_death */
             obj->origin = ORIGIN_STOLEN;
-			obj->origin_depth = current_player->depth;
+            obj->origin_depth = convert_depth_to_origin(current_player->depth);
 
             /* Give the gold to the monster */
             monster_carry(cave, context->mon, obj);
@@ -793,20 +803,22 @@ static void melee_effect_handler_EAT_FOOD(melee_effect_handler_context_t *contex
 		if (!tval_is_edible(obj)) continue;
 
 		if (obj->number == 1) {
-			object_desc(o_name, sizeof(o_name), obj, ODESC_BASE);
+			object_desc(o_name, sizeof(o_name), obj, ODESC_BASE,
+				context->p);
 			msg("Your %s (%c) was eaten!", o_name, I2A(index));
 		} else {
 			object_desc(o_name, sizeof(o_name), obj,
-						ODESC_PREFIX | ODESC_BASE);
+				ODESC_PREFIX | ODESC_BASE, context->p);
 			msg("One of your %s (%c) was eaten!", o_name,
 				I2A(index));
 		}
 
 		/* Steal and eat */
-		eaten = gear_object_for_use(obj, 1, false, &none_left);
+		eaten = gear_object_for_use(context->p, obj, 1, false,
+			&none_left);
 		if (eaten->known)
-			object_delete(&eaten->known);
-		object_delete(&eaten);
+			object_delete(player->cave, NULL, &eaten->known);
+		object_delete(cave, player->cave, &eaten);
 
 		/* Obvious */
 		context->obvious = true;
