@@ -60,6 +60,31 @@ static spell_tag_t spell_tag_lookup(const char *tag)
 		return SPELL_TAG_NONE;
 }
 
+/**
+ * Lookup a race-specific message for a spell.
+ *
+ * \param r is the race.
+ * \param s_idx is the spell index.
+ * \param msg_type is the type of message.
+ * \return the text of the message if there's a race-specific one or NULL if
+ * there is not.
+ */
+static const char *find_alternate_spell_message(const struct monster_race *r,
+		int s_idx, enum monster_altmsg_type msg_type)
+{
+	const struct monster_altmsg *am = r->spell_msgs;
+
+	while (1) {
+		if (!am) {
+			return NULL;
+		}
+		if (am->index == s_idx && am->msg_type == msg_type) {
+			 return am->message;
+		}
+		am = am->next;
+	}
+}
+
 /**
  * Print a monster spell message.
  *
@@ -70,6 +95,7 @@ static void spell_message(struct monster *mon,
 						  const struct monster_spell *spell,
 						  bool seen, bool hits)
 {
+	const char punct[] = ".!?;:,'";
 	char buf[1024] = "\0";
 	const char *next;
 	const char *s;
@@ -78,6 +104,7 @@ static void spell_message(struct monster *mon,
 	size_t end = 0;
 	struct monster_spell_level *level = spell->level;
 	struct monster *t_mon = NULL;
+	bool is_leading;
 
 	/* Get the right level of message */
 	while (level->next && mon->race->spell_power >= level->next->power) {
@@ -94,15 +121,34 @@ static void spell_message(struct monster *mon,
 		if (t_mon) {
 			return;
 		} else {
-			in_cursor = level->blind_message;
+			in_cursor = find_alternate_spell_message(mon->race,
+				spell->index, MON_ALTMSG_UNSEEN);
+			if (in_cursor == NULL) {
+				in_cursor = level->blind_message;
+			} else if (in_cursor[0] == '\0') {
+				return;
+			}
 		}
 	} else if (!hits) {
-		in_cursor = level->miss_message;
+		in_cursor = find_alternate_spell_message(mon->race,
+			spell->index, MON_ALTMSG_MISS);
+		if (in_cursor == NULL) {
+			in_cursor = level->miss_message;
+		} else if (in_cursor[0] == '\0') {
+			return;
+		}
 	} else {
-		in_cursor = level->message;
+		in_cursor = find_alternate_spell_message(mon->race,
+			spell->index, MON_ALTMSG_SEEN);
+		if (in_cursor == NULL) {
+			in_cursor = level->message;
+		} else if (in_cursor[0] == '\0') {
+			return;
+		}
 	}
 
 	next = strchr(in_cursor, '{');
+	is_leading = (next == in_cursor);
 	while (next) {
 		/* Copy the text leading up to this { */
 		strnfcat(buf, 1024, &end, "%.*s", next - in_cursor, in_cursor);
@@ -119,7 +165,17 @@ static void spell_message(struct monster *mon,
 			switch (spell_tag_lookup(tag)) {
 				case SPELL_TAG_NAME: {
 					char m_name[80];
-					monster_desc(m_name, sizeof(m_name), mon, MDESC_STANDARD);
+					int mdesc_mode = (MDESC_IND_HID |
+						MDESC_PRO_HID);
+
+					if (is_leading) {
+						mdesc_mode |= MDESC_CAPITAL;
+					}
+					if (!strchr(punct, *in_cursor)) {
+						mdesc_mode |= MDESC_COMMA;
+					}
+					monster_desc(m_name, sizeof(m_name),
+						mon, mdesc_mode);
 
 					strnfcat(buf, sizeof(buf), &end, m_name);
 					break;
@@ -137,10 +193,15 @@ static void spell_message(struct monster *mon,
 
 				case SPELL_TAG_TARGET: {
 					char m_name[80];
-					struct monster *t_mon;
 					if (mon->target.midx > 0) {
-						t_mon = cave_monster(cave, mon->target.midx);
-						monster_desc(m_name, sizeof(m_name), t_mon, MDESC_TARG);
+						int mdesc_mode = MDESC_TARG;
+
+						if (!strchr(punct, *in_cursor)) {
+							mdesc_mode |= MDESC_COMMA;
+						}
+						monster_desc(m_name,
+							sizeof(m_name), t_mon,
+							mdesc_mode);
 						strnfcat(buf, sizeof(buf), &end, m_name);
 					} else {
 						strnfcat(buf, sizeof(buf), &end, "you");
@@ -179,6 +240,7 @@ static void spell_message(struct monster *mon,
 		}
 
 		next = strchr(in_cursor, '{');
+		is_leading = false;
 	}
 	strnfcat(buf, 1024, &end, in_cursor);
 
@@ -215,6 +277,38 @@ static void spell_check_for_fail_rune(const struct monster_spell *spell)
 	}
 }
 
+/**
+ * Calculate the base to-hit value for a monster spell based on race only
+ * See also: chance_of_monster_hit_base
+ *
+ * \param race The monster race
+ * \param spell The spell
+ */
+static int chance_of_spell_hit_base(const struct monster_race *race,
+	const struct monster_spell *spell)
+{
+	return MAX(race->level, 1) * 3 + spell->hit;
+}
+
+/**
+ * Calculate the to-hit value of a monster spell for a specific monster
+ *
+ * \param mon The monster
+ * \param spell The spell
+ */
+static int chance_of_spell_hit(const struct monster *mon,
+	const struct monster_spell *spell)
+{
+	int to_hit = chance_of_spell_hit_base(mon->race, spell);
+
+	/* Apply confusion hit reduction for each level of confusion */
+	for (int i = 0; i < monster_effect_level(mon, MON_TMD_CONF); i++) {
+		to_hit = to_hit * (100 - CONF_HIT_REDUCTION) / 100;
+	}
+
+	return to_hit;
+}
+
 /**
  * Process a monster spell 
  *
@@ -228,7 +322,7 @@ void do_mon_spell(int index, struct monster *mon, bool seen)
 
 	bool ident = false;
 	bool hits;
-	int target_mon = mon->target.midx;
+	int target_midx = mon->target.midx;
 
 	/* See if it hits */
 	if (spell->hit == 100) {
@@ -236,19 +330,11 @@ void do_mon_spell(int index, struct monster *mon, bool seen)
 	} else if (spell->hit == 0) {
 		hits = false;
 	} else {
-		int rlev = MAX(mon->race->level, 1);
-		int conf_level = monster_effect_level(mon, MON_TMD_CONF);
-		int accuracy = 100;
-		while (conf_level) {
-			accuracy *= (100 - CONF_HIT_REDUCTION);
-			accuracy /= 100;
-			conf_level--;
-		}
-		if (target_mon > 0) {
-			hits = check_hit_monster(cave_monster(cave, target_mon),
-									 spell->hit, rlev, accuracy);
+		if (target_midx > 0) {
+			hits = test_hit(chance_of_spell_hit(mon, spell),
+				cave_monster(cave, target_midx)->race->ac);
 		} else {
-			hits = check_hit(player, spell->hit, rlev, accuracy);
+			hits = check_hit(player, chance_of_spell_hit(mon, spell));
 		}
 	}
 
@@ -265,7 +351,7 @@ void do_mon_spell(int index, struct monster *mon, bool seen)
 		}
 
 		/* Try a saving throw if available */
-		if (level->save_message && (target_mon <= 0) &&
+		if (level->save_message && (target_midx <= 0) &&
 				randint0(100) < player->state.skills[SKILL_SAVE]) {
 			msg("%s", level->save_message);
 			spell_check_for_fail_rune(spell);
@@ -283,7 +369,7 @@ void do_mon_spell(int index, struct monster *mon, bool seen)
  * Types of monster spells used for spell selection.
  */
 static const struct mon_spell_info {
-	u16b index;				/* Numerical index (RSF_FOO) */
+	uint16_t index;				/* Numerical index (RSF_FOO) */
 	int type;				/* Type bitflag */
 } mon_spell_types[] = {
     #define RSF(a, b)	{ RSF_##a, b },
