@@ -217,14 +217,14 @@ static void path_analyse(struct chunk *c, struct loc grid)
 	}
 
 	/* Plot the path. */
-	path_n = project_path(path_g, z_info->max_range, player->grid, grid,
-						  PROJECT_NONE);
+	path_n = project_path(cave, path_g, z_info->max_range, player->grid,
+		grid, PROJECT_NONE);
 
 	/* Project along the path */
 	for (i = 0; i < path_n - 1; ++i) {
 		/* Forget grids which would block los */
-		if (square_iswall(player->cave, path_g[i])) {
-			sqinfo_off(square(c, path_g[i]).info, SQUARE_SEEN);
+		if (!square_allowslos(player->cave, path_g[i])) {
+			sqinfo_off(square(c, path_g[i])->info, SQUARE_SEEN);
 			square_forget(c, path_g[i]);
 			square_light_spot(c, path_g[i]);
 		}
@@ -395,7 +395,7 @@ void update_mon(struct monster *mon, struct chunk *c, bool full)
 	/* If a mimic looks like an ignored item, it's not seen */
 	if (monster_is_mimicking(mon)) {
 		struct object *obj = mon->mimicked_obj;
-		if (ignore_item_ok(obj))
+		if (ignore_item_ok(player, obj))
 			easy = flag = false;
 	}
 
@@ -428,7 +428,8 @@ void update_mon(struct monster *mon, struct chunk *c, bool full)
 		}
 	} else if (monster_is_visible(mon)) {
 		/* Not visible but was previously seen - treat mimics differently */
-		if (!mon->mimicked_obj || ignore_item_ok(mon->mimicked_obj)) {
+		if (!mon->mimicked_obj
+				|| ignore_item_ok(player, mon->mimicked_obj)) {
 			/* Mark as not visible */
 			mflag_off(mon->mflag, MFLAG_VISIBLE);
 
@@ -514,6 +515,52 @@ static void player_leaving(struct loc grid1, struct loc grid2)
 	hit_trap(grid1, 1);
 }
 
+/**
+ * Is a helper function to move a mimicked object when the mimic (not known
+ * to the player) is moved.  Assumes that the caller will be calling
+ * square_light_spot() for the source grid.
+ */
+static void move_mimicked_object(struct chunk *c, struct monster *mon,
+	struct loc src, struct loc dest)
+{
+	struct object *mimicked = mon->mimicked_obj;
+	/*
+	 * Move a copy so, if necessary, the original can remain as a
+	 * placeholder for the known version of the object in the player's
+	 * view of the cave.
+	 */
+	struct object *moved = object_new();
+	bool dummy = true;
+
+	assert(mimicked);
+	object_copy(moved, mimicked);
+	moved->oidx = 0;
+	mimicked->mimicking_m_idx = 0;
+	if (mimicked->known) {
+		moved->known = object_new();
+		object_copy(moved->known, mimicked->known);
+		moved->known->oidx = 0;
+		moved->known->grid = loc(0,0);
+	}
+	if (floor_carry(c, dest, moved, &dummy)) {
+		mon->mimicked_obj = moved;
+	} else {
+		/* Could not move the object so cancel mimicry. */
+		moved->mimicking_m_idx = 0;
+		mon->mimicked_obj = NULL;
+		/* Give object to monster if appropriate; otherwise, delete. */
+		if (!rf_has(mon->race->flags, RF_MIMIC_INV) ||
+			!monster_carry(c, mon, moved)) {
+			struct chunk *p_c = (c == cave) ? player->cave : NULL;
+			if (moved->known) {
+				object_delete(p_c, NULL, &moved->known);
+			}
+			object_delete(c, p_c, &moved);
+		}
+	}
+	square_delete_object(c, src, mimicked, true, false);
+}
+
 /**
  * Swap the players/monsters (if any) at two locations.
  */
@@ -535,14 +582,28 @@ void monster_swap(struct loc grid1, struct loc grid2)
 	if (m1 > 0) {
 		/* Monster */
 		mon = cave_monster(cave, m1);
-		mon->grid = grid2;
 
 		/* Update monster */
+		if (monster_is_mimicking(mon)) {
+			/*
+			 * Become aware if the player can see the mimic before
+			 * or after the swap.
+			 */
+			if (monster_is_in_view(mon) ||
+				(m2 >= 0 && los(cave, pgrid, grid2)) ||
+				(m2 < 0 && los(cave, grid1, grid2))) {
+				become_aware(cave, mon);
+			} else {
+				move_mimicked_object(cave, mon, grid1, grid2);
+				player->upkeep->redraw |= (PR_ITEMLIST);
+			}
+		}
+		mon->grid = grid2;
 		update_mon(mon, cave, true);
 
 		/* Affect light? */
 		if (mon->race->light != 0)
-			player->upkeep->update |= PU_UPDATE_VIEW;
+			player->upkeep->update |= PU_UPDATE_VIEW | PU_MONSTERS;
 
 		/* Redraw monster list */
 		player->upkeep->redraw |= (PR_MONLIST);
@@ -561,23 +622,35 @@ void monster_swap(struct loc grid1, struct loc grid2)
 		player->upkeep->redraw |= (PR_MONLIST);
 
 		/* Don't allow command repeat if moved away from item used. */
-		if (cmdq_does_previous_use_floor_item()) {
-			cmd_disable_repeat();
-		}
+		cmd_disable_repeat_floor_item();
 	}
 
 	/* Monster 2 */
 	if (m2 > 0) {
 		/* Monster */
 		mon = cave_monster(cave, m2);
-		mon->grid = grid1;
 
 		/* Update monster */
+		if (monster_is_mimicking(mon)) {
+			/*
+			 * Become aware if the player can see the mimic before
+			 * or after the swap.
+			 */
+			if (monster_is_in_view(mon) ||
+				(m1 >= 0 && los(cave, pgrid, grid1)) ||
+				(m1 < 0 && los(cave, grid2, grid1))) {
+				become_aware(cave, mon);
+			} else {
+				move_mimicked_object(cave, mon, grid2, grid1);
+				player->upkeep->redraw |= (PR_ITEMLIST);
+			}
+		}
+		mon->grid = grid1;
 		update_mon(mon, cave, true);
 
 		/* Affect light? */
 		if (mon->race->light != 0)
-			player->upkeep->update |= PU_UPDATE_VIEW;
+			player->upkeep->update |= PU_UPDATE_VIEW | PU_MONSTERS;
 
 		/* Redraw monster list */
 		player->upkeep->redraw |= (PR_MONLIST);
@@ -596,9 +669,7 @@ void monster_swap(struct loc grid1, struct loc grid2)
 		player->upkeep->redraw |= (PR_MONLIST);
 
 		/* Don't allow command repeat if moved away from item used. */
-		if (cmdq_does_previous_use_floor_item()) {
-			cmd_disable_repeat();
-		}
+		cmd_disable_repeat_floor_item();
 	}
 
 	/* Redraw */
@@ -633,10 +704,12 @@ bool monster_can_see(struct chunk *c, struct monster *mon, struct loc grid)
 /**
  * Make player fully aware of the given mimic.
  *
+ * \param c Is the chunk with the monster.
+ * \param mon Is the monster.
  * When a player becomes aware of a mimic, we update the monster memory
  * and delete the "fake item" that the monster was mimicking.
  */
-void become_aware(struct monster *mon)
+void become_aware(struct chunk *c, struct monster *mon)
 {
 	struct monster_lore *lore = get_lore(mon->race);
 
@@ -651,10 +724,10 @@ void become_aware(struct monster *mon)
 		if (mon->mimicked_obj) {
 			struct object *obj = mon->mimicked_obj;
 			char o_name[80];
-			object_desc(o_name, sizeof(o_name), obj, ODESC_BASE);
+			object_desc(o_name, sizeof(o_name), obj, ODESC_BASE, player);
 
 			/* Print a message */
-			if (square_isseen(cave, obj->grid))
+			if (square_isseen(c, obj->grid))
 				msg("The %s was really a monster!", o_name);
 
 			/* Clear the mimicry */
@@ -676,11 +749,12 @@ void become_aware(struct monster *mon)
 					given->known->oidx = 0;
 					given->known->grid = loc(0, 0);
 				}
-				if (! monster_carry(cave, mon, given)) {
+				if (!monster_carry(c, mon, given)) {
+					struct chunk *p_c = (c == cave) ? player->cave : NULL;
 					if (given->known) {
-						object_delete(&given->known);
+						object_delete(p_c, NULL, &given->known);
 					}
-					object_delete(&given);
+					object_delete(c, p_c, &given);
 				}
 			}
 
@@ -688,16 +762,21 @@ void become_aware(struct monster *mon)
 			 * Delete the mimicked object; noting and lighting
 			 * done below outside of the if block.
 			 */
-			square_delete_object(cave, obj->grid, obj, false, false);
+			square_delete_object(c, obj->grid, obj, false, false);
+
+			/* Since mimicry affects visibility, update that. */
+			update_mon(mon, c, false);
 		}
 
 		/* Update monster and item lists */
-		player->upkeep->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
+		if (mon->race->light != 0) {
+			player->upkeep->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
+		}
 		player->upkeep->redraw |= (PR_MONLIST | PR_ITEMLIST);
 	}
 
-	square_note_spot(cave, mon->grid);
-	square_light_spot(cave, mon->grid);
+	square_note_spot(c, mon->grid);
+	square_light_spot(c, mon->grid);
 }
 
 /**
@@ -868,7 +947,7 @@ struct monster *choose_nearby_injured_kin(struct chunk *c,
  * If `stats` is true, then we skip updating the monster memory. This is
  * used by stats-generation code, for efficiency.
  */
-void monster_death(struct monster *mon, bool stats)
+void monster_death(struct monster *mon, struct player *p, bool stats)
 {
 	int dump_item = 0;
 	int dump_gold = 0;
@@ -921,37 +1000,35 @@ void monster_death(struct monster *mon, bool stats)
 		lore_treasure(mon, dump_item, dump_gold);
 
 	/* Update monster list window */
-	player->upkeep->redraw |= PR_MONLIST;
-
-	/* Affect light? */
-	if (mon->race->light != 0)
-		player->upkeep->update |= PU_UPDATE_VIEW;
+	p->upkeep->redraw |= PR_MONLIST;
 
 	/* Check if we finished a quest */
-	quest_check(mon);
+	quest_check(p, mon);
 }
 
 /**
  * Handle the consequences of the killing of a monster by the player
  */
-static void player_kill_monster(struct monster *mon, const char *note)
+static void player_kill_monster(struct monster *mon, struct player *p,
+		const char *note)
 {
-	s32b div, new_exp, new_exp_frac;
+	int32_t div, new_exp, new_exp_frac;
 	struct monster_lore *lore = get_lore(mon->race);
 	char m_name[80];
 	char buf[80];
+	int desc_mode = MDESC_DEFAULT | ((note) ? MDESC_COMMA : 0);
 
 	/* Assume normal death sound */
 	int soundfx = MSG_KILL;
 
 	/* Extract monster name */
-	monster_desc(m_name, sizeof(m_name), mon, MDESC_DEFAULT);
+	monster_desc(m_name, sizeof(m_name), mon, desc_mode);
 
 	/* Shapechanged monsters revert on death */
 	if (mon->original_race) {
 		monster_revert_shape(mon);
 		lore = get_lore(mon->race);
-		monster_desc(m_name, sizeof(m_name), mon, MDESC_DEFAULT);
+		monster_desc(m_name, sizeof(m_name), mon, desc_mode);
 	}
 
 	/* Play a special sound if the monster was unique */
@@ -967,18 +1044,16 @@ static void player_kill_monster(struct monster *mon, const char *note)
 		if (strlen(note) <= 1) {
 			/* Death by Spell attack - messages handled by project_m() */
 		} else {
-			char *str = format("%s%s", m_name, note);
-			my_strcap(str);
-
 			/* Make sure to flush any monster messages first */
-			notice_stuff(player);
+			notice_stuff(p);
 
 			/* Death by Missile attack */
-			msgt(soundfx, "%s", str);
+			my_strcap(m_name);
+			msgt(soundfx, "%s%s", m_name, note);
 		}
 	} else {
 		/* Make sure to flush any monster messages first */
-		notice_stuff(player);
+		notice_stuff(p);
 
 		if (!monster_is_visible(mon))
 			/* Death by physical attack -- invisible monster */
@@ -992,26 +1067,27 @@ static void player_kill_monster(struct monster *mon, const char *note)
 	}
 
 	/* Player level */
-	div = player->lev;
+	div = p->lev;
 
 	/* Give some experience for the kill */
 	new_exp = ((long)mon->race->mexp * mon->race->level) / div;
 
 	/* Handle fractional experience */
 	new_exp_frac = ((((long)mon->race->mexp * mon->race->level) % div)
-					* 0x10000L / div) + player->exp_frac;
+					* 0x10000L / div) + p->exp_frac;
 
 	/* Keep track of experience */
 	if (new_exp_frac >= 0x10000L) {
 		new_exp++;
-		player->exp_frac = (u16b)(new_exp_frac - 0x10000L);
+		p->exp_frac = (uint16_t)(new_exp_frac - 0x10000L);
 	} else {
-		player->exp_frac = (u16b)new_exp_frac;
+		p->exp_frac = (uint16_t)new_exp_frac;
 	}
 
 	/* When the player kills a Unique, it stays dead */
 	if (rf_has(mon->race->flags, RF_UNIQUE)) {
 		char unique_name[80];
+		assert(mon->original_race == NULL);
 		mon->race->max_num = 0;
 
 		/*
@@ -1023,20 +1099,20 @@ static void player_kill_monster(struct monster *mon, const char *note)
 
 		/* Log the slaying of a unique */
 		strnfmt(buf, sizeof(buf), "Killed %s", unique_name);
-		history_add(player, buf, HIST_SLAY_UNIQUE);
+		history_add(p, buf, HIST_SLAY_UNIQUE);
 	}
 
 	/* Gain experience */
-	player_exp_gain(player, new_exp);
+	player_exp_gain(p, new_exp);
 
 	/* Generate treasure */
-	monster_death(mon, false);
+	monster_death(mon, p, false);
 
 	/* Bloodlust bonus */
-	if (player->timed[TMD_BLOODLUST]) {
-		player_inc_timed(player, TMD_BLOODLUST, 10, false, false);
-		player_over_exert(player, PY_EXERT_CONF, 5, 3);
-		player_over_exert(player, PY_EXERT_HALLU, 5, 10);
+	if (p->timed[TMD_BLOODLUST]) {
+		player_inc_timed(p, TMD_BLOODLUST, 10, false, false);
+		player_over_exert(p, PY_EXERT_CONF, 5, 3);
+		player_over_exert(p, PY_EXERT_HALLU, 5, 10);
 	}
 
 	/* Recall even invisible uniques or winners */
@@ -1049,7 +1125,7 @@ static void player_kill_monster(struct monster *mon, const char *note)
 
 		/* Update lore and tracking */
 		lore_update(mon->race, lore);
-		monster_race_track(player->upkeep, mon->race);
+		monster_race_track(p->upkeep, mon->race);
 	}
 
 	/* Delete the monster */
@@ -1119,8 +1195,9 @@ bool mon_take_nonplayer_hit(int dam, struct monster *t_mon,
 {
 	assert(t_mon);
 
-	/* "Unique" monsters can only be "killed" by the player */
-	if (rf_has(t_mon->race->flags, RF_UNIQUE)) {
+	/* "Unique" or arena monsters can only be "killed" by the player */
+	if (rf_has(t_mon->race->flags, RF_UNIQUE)
+			|| player->upkeep->arena_level) {
 		/* Reduce monster hp to zero, but don't kill it. */
 		if (dam > t_mon->hp) dam = t_mon->hp;
 	}
@@ -1141,7 +1218,7 @@ bool mon_take_nonplayer_hit(int dam, struct monster *t_mon,
 		add_monster_message(t_mon, die_msg, false);
 
 		/* Generate treasure, etc */
-		monster_death(t_mon, false);
+		monster_death(t_mon, player, false);
 
 		/* Delete the monster */
 		delete_monster_idx(t_mon->midx);
@@ -1179,38 +1256,42 @@ bool mon_take_nonplayer_hit(int dam, struct monster *t_mon,
  * worth more than subsequent monsters.  This would also need to
  * induce changes in the monster recall code.  XXX XXX XXX
  **/
-bool mon_take_hit(struct monster *mon, int dam, bool *fear, const char *note)
+bool mon_take_hit(struct monster *mon, struct player *p, int dam, bool *fear,
+		const char *note)
 {
 	/* Redraw (later) if needed */
-	if (player->upkeep->health_who == mon)
-		player->upkeep->redraw |= (PR_HEALTH);
+	if (p->upkeep->health_who == mon)
+		p->upkeep->redraw |= (PR_HEALTH);
 
-	/* Wake it up, make it aware of the player */
-	monster_wake(mon, false, 100);
-	mon_clear_timed(mon, MON_TMD_HOLD, MON_TMD_FLG_NOTIFY);
+	/* If the hit doesn't kill, wake it up, make it aware of the player */
+	if (dam <= mon->hp) {
+		monster_wake(mon, false, 100);
+		mon_clear_timed(mon, MON_TMD_HOLD, MON_TMD_FLG_NOTIFY);
+	}
 
 	/* Become aware of its presence */
 	if (monster_is_camouflaged(mon))
-		become_aware(mon);
+		become_aware(cave, mon);
 
 	/* No damage, we're done */
 	if (dam == 0) return false;
 
 	/* Covering tracks is no longer possible */
-	player->timed[TMD_COVERTRACKS] = 0;
+	p->timed[TMD_COVERTRACKS] = 0;
 
 	/* Hurt it */
 	mon->hp -= dam;
 	if (mon->hp < 0) {
 		/* Deal with arena monsters */
-		if (player->upkeep->arena_level) {
-			player->upkeep->generate_level = true;
+		if (p->upkeep->arena_level) {
+			p->upkeep->generate_level = true;
+			p->upkeep->health_who = mon;
 			(*fear) = false;
 			return true;
 		}
 
 		/* It is dead now */
-		player_kill_monster(mon, note);
+		player_kill_monster(mon, p, note);
 
 		/* Not afraid */
 		(*fear) = false;
@@ -1232,7 +1313,7 @@ void kill_arena_monster(struct monster *mon)
 	assert(old_mon);
 	update_mon(old_mon, cave, true);
 	old_mon->hp = -1;
-	player_kill_monster(old_mon, " is defeated!");
+	player_kill_monster(old_mon, player, " is defeated!");
 }
 
 /**
@@ -1258,10 +1339,10 @@ void monster_take_terrain_damage(struct monster *mon)
 /**
  * Terrain is currently damaging monster
  */
-bool monster_taking_terrain_damage(struct monster *mon)
+bool monster_taking_terrain_damage(struct chunk *c, struct monster *mon)
 {
-	if (square_isdamaging(cave, mon->grid) &&
-		!rf_has(mon->race->flags, square_feat(cave, mon->grid)->resist_flag)) {
+	if (square_isdamaging(c, mon->grid) &&
+		!rf_has(mon->race->flags, square_feat(c, mon->grid)->resist_flag)) {
 		return true;
 	}
 
@@ -1286,7 +1367,7 @@ bool monster_carry(struct chunk *c, struct monster *mon, struct object *obj)
 	/* Scan objects already being held for combination */
 	for (held_obj = mon->held_obj; held_obj; held_obj = held_obj->next) {
 		/* Check for combination */
-		if (object_similar(held_obj, obj, OSTACK_MONSTER)) {
+		if (object_mergeable(held_obj, obj, OSTACK_MONSTER)) {
 			/* Combine the items */
 			object_absorb(held_obj, obj);
 
@@ -1318,26 +1399,21 @@ bool monster_carry(struct chunk *c, struct monster *mon, struct object *obj)
  */
 struct object *get_random_monster_object(struct monster *mon)
 {
-	struct object *obj = mon->held_obj;
-	int count = 0;
+    struct object *obj, *pick = NULL;
+    int i = 1;
 
-	if (!obj) return NULL;
+    /* Pick a random object */
+    for (obj = mon->held_obj; obj; obj = obj->next)
+    {
+        /* Check it isn't a quest artifact */
+        if (obj->artifact && kf_has(obj->kind->kind_flags, KF_QUEST_ART))
+            continue;
 
-	/* Count the objects */
-	while (obj) {
-		count++;
-		obj = obj->next;
-	}
+        if (one_in_(i)) pick = obj;
+        i++;
+    }
 
-	/* Now pick one... */
-	obj = mon->held_obj;
-	count -= randint1(count);
-	while (count) {
-		obj = obj->next;
-		count--;
-	}
-
-	return obj;
+    return pick;
 }
 
 /**
@@ -1398,17 +1474,20 @@ void steal_monster_item(struct monster *mon, int midx)
 			if (tval_is_money(obj)) {
 				msg("You steal %d gold pieces worth of treasure.", obj->pval);
 				player->au += obj->pval;
+				player->upkeep->redraw |= (PR_GOLD);
 				delist_object(cave, obj);
-				object_delete(&obj);
+				object_delete(cave, player->cave, &obj);
 			} else {
 				object_grab(player, obj);
 				delist_object(player->cave, obj->known);
 				delist_object(cave, obj);
-				/* Drop immediately if ignored to prevent pack overflow */
-				if (ignore_item_ok(obj)) {
+				/* Drop immediately if ignored,
+				   or if inventory already full to prevent pack overflow */
+				if (ignore_item_ok(player, obj) || !inven_carry_okay(obj)) {
 					char o_name[80];
 					object_desc(o_name, sizeof(o_name), obj,
-								ODESC_PREFIX | ODESC_FULL);
+						ODESC_PREFIX | ODESC_FULL,
+						player);
 					drop_near(cave, &obj, 0, player->grid, true, true);
 					msg("You drop %s.", o_name);
 				} else {
@@ -1430,7 +1509,7 @@ void steal_monster_item(struct monster *mon, int midx)
 				(void)strnfmt(o_name, sizeof(o_name), "treasure");
 			} else {
 				object_desc(o_name, sizeof(o_name), obj,
-							ODESC_PREFIX | ODESC_FULL);
+					ODESC_PREFIX | ODESC_FULL, player);
 			}
 			msg("You fail to steal %s from %s.", o_name, m_name);
 			/* Monster wakes, may notice */
@@ -1527,7 +1606,7 @@ bool monster_change_shape(struct monster *mon)
 			get_mon_num_prep(monster_base_shape_okay);
 
 			/* Pick a random race */
-			race = get_mon_num(player->depth + 5);
+			race = get_mon_num(player->depth + 5, player->depth);
 
 			/* Reset allocation table */
 			get_mon_num_prep(NULL);
@@ -1569,7 +1648,7 @@ bool monster_change_shape(struct monster *mon)
 	/* Print a message immediately, update visuals */
 	if (monster_is_obvious(mon)) {
 		char m_name[80];
-		monster_desc(m_name, sizeof(m_name), mon, MDESC_IND_HID);
+		monster_desc(m_name, sizeof(m_name), mon, MDESC_STANDARD);
 		msgt(MSG_GENERIC, "%s %s", m_name, "shimmers and changes!");
 		if (player->upkeep->health_who == mon)
 			player->upkeep->redraw |= (PR_HEALTH);
@@ -1580,13 +1659,14 @@ bool monster_change_shape(struct monster *mon)
 
 	/* Set the race */
 	if (race) {
-		mon->original_race = mon->race;
+		if (!mon->original_race) mon->original_race = mon->race;
 		mon->race = race;
 		mon->mspeed += mon->race->speed - mon->original_race->speed;
 	}
 
 	/* Emergency teleport if needed */
-	if (!monster_passes_walls(mon) && square_iswall(cave, mon->grid)) {
+	if (!monster_passes_walls(mon) &&
+		!square_is_monster_walkable(cave, mon->grid)) {
 		effect_simple(EF_TELEPORT, source_monster(mon->midx), "1", 0, 0, 0,
 					  mon->grid.y, mon->grid.x, NULL);
 	}
@@ -1602,7 +1682,7 @@ bool monster_revert_shape(struct monster *mon)
 	if (mon->original_race) {
 		if (monster_is_obvious(mon)) {
 			char m_name[80];
-			monster_desc(m_name, sizeof(m_name), mon, MDESC_IND_HID);
+			monster_desc(m_name, sizeof(m_name), mon, MDESC_STANDARD);
 			msgt(MSG_GENERIC, "%s %s", m_name, "shimmers and changes!");
 			if (player->upkeep->health_who == mon)
 				player->upkeep->redraw |= (PR_HEALTH);
@@ -1615,7 +1695,8 @@ bool monster_revert_shape(struct monster *mon)
 		mon->original_race = NULL;
 
 		/* Emergency teleport if needed */
-		if (!monster_passes_walls(mon) && square_iswall(cave, mon->grid)) {
+		if (!monster_passes_walls(mon) &&
+			!square_is_monster_walkable(cave, mon->grid)) {
 			effect_simple(EF_TELEPORT, source_monster(mon->midx), "1", 0, 0, 0,
 						  mon->grid.y, mon->grid.x, NULL);
 		}
