@@ -20,10 +20,10 @@
 #include "angband.h"
 #include "cmds.h"
 #include "effects.h"
+#include "effects-info.h"
 #include "game-world.h"
 #include "init.h"
 #include "monster.h"
-#include "mon-summon.h"
 #include "mon-util.h"
 #include "obj-curse.h"
 #include "obj-gear.h"
@@ -36,7 +36,6 @@
 #include "obj-util.h"
 #include "player-attack.h"
 #include "player-calcs.h"
-#include "player-timed.h"
 #include "project.h"
 #include "z-textblock.h"
 
@@ -64,18 +63,6 @@ static const struct origin_type {
 	#undef ORIGIN
 };
 
-static struct {
-	int index;
-	int args;
-	int efinfo_flag;
-	const char *desc;
-} base_descs[] = {
-	{ EF_NONE, 0, EFINFO_NONE, "" },
-	#define EFFECT(x, a, b, c, d, e) { EF_##x, c, d, e },
-	#include "list-effects.h"
-	#undef EFFECT
-};
-
 
 /**
  * ------------------------------------------------------------------------
@@ -93,7 +80,7 @@ static void info_out_list(textblock *tb, const char *list[], size_t count)
 	size_t i;
 
 	for (i = 0; i < count; i++) {
-		textblock_append(tb, list[i]);
+		textblock_append(tb, "%s", list[i]);
 		if (i != (count - 1)) textblock_append(tb, ", ");
 	}
 
@@ -137,17 +124,13 @@ static bool describe_curses(textblock *tb, const struct object *obj,
 	for (i = 1; i < z_info->curse_max; i++) {
 		if (c[i].power) {
 			textblock_append(tb, "It ");
-			textblock_append_c(tb, COLOUR_L_RED, curses[i].desc);
+			textblock_append_c(tb, COLOUR_L_RED, "%s", curses[i].desc);
 			if (c[i].power == 100) {
 				textblock_append(tb, "; this curse cannot be removed");
 			}
 			textblock_append(tb, ".\n");
 		}
 	}
-	/* Say if curse removal has been tried */
-	if (of_has(obj->flags, OF_FRAGILE)) {
-		textblock_append(tb, "Attempting to uncurse it may destroy it.\n");
-	}
 
 	return true;
 }
@@ -394,7 +377,7 @@ static bool describe_slays(textblock *tb, const struct object *obj)
 	for (i = 1; i < z_info->slay_max; i++) {
 		if (!s[i]) continue;
 
-		textblock_append(tb, slays[i].name);
+		textblock_append(tb, "%s", slays[i].name);
 		if (slays[i].multiplier > 3)
 			textblock_append(tb, " (powerfully)");
 		if (count > 1)
@@ -434,7 +417,7 @@ static bool describe_brands(textblock *tb, const struct object *obj)
 
 		if (brands[i].multiplier < 3)
 			textblock_append(tb, "weak ");
-		textblock_append(tb, brands[i].name);
+		textblock_append(tb, "%s", brands[i].name);
 		if (count > 1)
 			textblock_append(tb, ", ");
 		else
@@ -831,7 +814,7 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 								&crit_add, &crit_div);
 
 		dam += (obj->known->to_d * 10);
-		dam *= 1 + player->lev / 12;
+		dam *= 2 + obj->weight / 12;
 	}
 
 	if (ammo) multiplier = player->state.ammo_mult;
@@ -870,7 +853,7 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 	/* Get damage for each brand on the objects */
 	for (i = 1; i < z_info->brand_max; i++) {
 		/* Must have the brand, possibly from a spell */
-		if (player_has_temporary_brand(i)) {
+		if (player_has_temporary_brand(player, i)) {
 			*nonweap_slay = true;
 		} else if (!total_brands[i]) {
 			continue;
@@ -896,7 +879,7 @@ static bool obj_known_damage(const struct object *obj, int *normal_damage,
 	/* Get damage for each slay on the objects */
 	for (i = 1; i < z_info->slay_max; i++) {
 		/* Must have the slay, possibly from a spell */
-		if (player_has_temporary_slay(i)) {
+		if (player_has_temporary_slay(player, i)) {
 			*nonweap_slay = true;
 		} else if (!total_slays[i]) {
 			continue;
@@ -1002,7 +985,7 @@ static bool o_obj_known_damage(const struct object *obj, int *normal_damage,
 		dice += o_calculate_missile_crits(player->state, obj, bow);
 	} else {
 		dice += o_calculate_missile_crits(player->state, obj, NULL);
-		dice *= 2 + player->lev / 12;
+		dice *= 2 + obj->weight / 12;
 	}
 
 	if (ammo) multiplier = player->state.ammo_mult;
@@ -1056,11 +1039,13 @@ static bool o_obj_known_damage(const struct object *obj, int *normal_damage,
 	for (i = 1; i < z_info->brand_max; i++) {
 		int brand_average, add = brands[i].o_multiplier - 10;
 
-		/* Must have the brand */
-		if (total_brands[i])
-			has_brands_or_slays = true;
-		else
+		/* Must have the brand, possibly from a spell */
+		if (player_has_temporary_brand(player, i)) {
+			*nonweap_slay = true;
+		} else if (!total_brands[i]) {
 			continue;
+		}
+		has_brands_or_slays = true;
 
 		/* Include brand in stated average (x10), deflate (/1000) */
 		brand_average = die_average * brands[i].o_multiplier;
@@ -1087,11 +1072,13 @@ static bool o_obj_known_damage(const struct object *obj, int *normal_damage,
 	for (i = 1; i < z_info->slay_max; i++) {
 		int slay_average, add = slays[i].o_multiplier - 10;
 
-		/* Must have the slay */
-		if (total_slays[i])
-			has_brands_or_slays = true;
-		else
+		/* Must have the slay, possibly from a spell */
+		if (player_has_temporary_slay(player, i)) {
+			*nonweap_slay = true;
+		} else if (!total_slays[i]) {
 			continue;
+		}
+		has_brands_or_slays = true;
 
 		/* Include slay in stated average (x10), deflate (/1000) */
 		slay_average = die_average * slays[i].o_multiplier;
@@ -1288,7 +1275,11 @@ static bool describe_damage(textblock *tb, const struct object *obj, bool throw)
 			textblock_append(tb, " %s", lastnm);
 		}
 
-		textblock_append(tb, (nsort == 1) ? " and " : ", and ");
+		if (nsort == 0) {
+			has_brands_or_slays = false;
+		} else {
+			textblock_append(tb, (nsort == 1) ? " and " : ", and ");
+		}
 		mem_free(sortind);
 	}
 
@@ -1395,7 +1386,7 @@ static bool describe_combat(textblock *tb, const struct object *obj)
 
 	if (ammo) {
 		textblock_append(tb, "When fired, hits targets up to ");
-		textblock_append_c(tb, COLOUR_L_GREEN, format("%d", range));
+		textblock_append_c(tb, COLOUR_L_GREEN, "%d", range);
 		textblock_append(tb, " feet away.\n");
 	}
 
@@ -1431,8 +1422,8 @@ static bool obj_known_digging(struct object *obj, int deciturns[])
 	struct player_state state;
 	int i;
 	int chances[DIGGING_MAX];
-	int slot = wield_slot(obj);
-	struct object *current = slot_object(player, slot);
+	int slot;
+	struct object *current;
 
 	/* Doesn't remotely resemble a digger */
 	if (!tval_is_wearable(obj) ||
@@ -1444,6 +1435,8 @@ static bool obj_known_digging(struct object *obj, int deciturns[])
 		return false;
 
 	/* Pretend we're wielding the object */
+	slot = wield_slot(obj);
+	current = slot_object(player, slot);
 	player->body.slots[slot].obj = obj;
 
 	/* Calculate the player's hypothetical state */
@@ -1579,7 +1572,7 @@ static bool describe_light(textblock *tb, const struct object *obj,
 
 	if (tval_is_light(obj)) {
 		textblock_append(tb, "Intensity ");
-		textblock_append_c(tb, COLOUR_L_GREEN, format("%d", intensity));
+		textblock_append_c(tb, COLOUR_L_GREEN, "%d", intensity);
 		textblock_append(tb, " light.");
 
 		if (!obj->artifact && !uses_fuel)
@@ -1674,7 +1667,6 @@ static bool obj_known_effect(const struct object *obj, struct effect **effect,
 static bool describe_effect(textblock *tb, const struct object *obj,
 		bool only_artifacts, bool subjective)
 {
-	char desc[200];
 	struct effect *effect = NULL;
 	bool aimed = false;
 	int min_time, max_time, failure_chance;
@@ -1709,264 +1701,33 @@ static bool describe_effect(textblock *tb, const struct object *obj,
 	/* Activations get a special message */
 	if (obj->activation && obj->activation->desc) {
 		textblock_append(tb, "When activated, it ");
-		textblock_append(tb, obj->activation->desc);
+		textblock_append(tb, "%s", obj->activation->desc);
 	} else {
-		int random_choices = 0;
-		bool random_breath = (effect && (effect->index == EF_RANDOM) &&
-							  effect->next &&
-							  (effect->next->index == EF_BREATH));
-		char breaths[120];
+		int level = obj->artifact ?
+			obj->artifact->level : obj->kind->level;
+		int boost = MAX((player->state.skills[SKILL_DEVICE] - level) / 2, 0);
+		const char *prefix;
+		textblock *tbe;
 
-		my_strcpy(breaths, "", sizeof(breaths));
-
-		/* Get descriptions for all the effects */
-		effect = object_effect(obj);
-		if (!effect_desc(effect)) return false;
-
-		if (aimed)
-			textblock_append(tb, "When aimed, it ");
+		if (obj->activation)
+			prefix = "When activated, it ";
+		else if (aimed)
+			prefix = "When aimed, it ";
 		else if (tval_is_edible(obj))
-			textblock_append(tb, "When eaten, it ");
+			prefix = "When eaten, it ";
 		else if (tval_is_potion(obj))
-			textblock_append(tb, "When quaffed, it ");
+			prefix = "When quaffed, it ";
 		else if (tval_is_scroll(obj))
-			textblock_append(tb, "When read, it ");
+			prefix = "When read, it ";
 		else
-			textblock_append(tb, "When activated, it ");
+			prefix = "When activated, it ";
 
-		/* Print a colourised description */
-		while (effect) {
-			char *next_char = desc;
-			int roll = 0;
-			random_value value = { 0, 0, 0, 0 };
-			char dice_string[20];
-
-			int level = obj->artifact ? obj->artifact->level : obj->kind->level;
-			int boost = MAX(player->state.skills[SKILL_DEVICE] - level, 0);
-
-			if (effect->dice != NULL)
-				roll = dice_roll(effect->dice, &value);
-
-			/* Deal with special random effect */
-			if (effect->index == EF_RANDOM)
-				random_choices = roll + 1;
-
-			/* Get the possible dice strings */
-			if (value.dice && value.base)
-				strnfmt(dice_string, sizeof(dice_string), "%d+%dd%d",
-						value.base, value.dice, value.sides);
-			else if (value.dice)
-				strnfmt(dice_string, sizeof(dice_string), "%dd%d",
-						value.dice, value.sides);
-			else
-				strnfmt(dice_string, sizeof(dice_string), "%d", value.base);
-
-			/* Check all the possible types of description format */
-			switch (base_descs[effect->index].efinfo_flag) {
-				/* Healing sometimes has a minimum percentage */
-			case EFINFO_HURT: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect), dice_string);
-				break;
-			}
-			case EFINFO_HEAL: {
-				char min_string[50];
-				if (value.m_bonus)
-					strnfmt(min_string, sizeof(min_string),
-							" (or %d%%, whichever is greater)", value.m_bonus);
-				else
-					strnfmt(min_string, sizeof(min_string), "");
-				strnfmt(desc, sizeof(desc), effect_desc(effect), dice_string,
-						min_string);
-				break;
-			}
-
-			case EFINFO_CONST: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect), value.base/2);
-				break;
-			}
-			case EFINFO_FOOD: {
-				char *fed = effect->subtype ? "leaves you nourished" :
-					"feeds you";
-				strnfmt(desc, sizeof(desc), effect_desc(effect), fed,
-						value.base * z_info->food_value, value.base);
-				break;
-			}
-			case EFINFO_CURE: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						timed_effects[effect->subtype].desc);
-				break;
-			}
-			case EFINFO_TIMED: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						timed_effects[effect->subtype].desc, dice_string);
-				break;
-			}
-			case EFINFO_STAT: {
-				int stat = effect->subtype;
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						lookup_obj_property(OBJ_PROPERTY_STAT, stat)->name);
-				break;
-			}
-			case EFINFO_SEEN: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->subtype].desc);
-				break;
-			}
-			case EFINFO_SUMM: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						summon_desc(effect->subtype));
-				break;
-			}
-
-			/* Only currently used for the player, but can handle monsters */
-			case EFINFO_TELE: {
-				char *dist = value.m_bonus ?
-					" a level dependent distance" :
-					format(" %d grids", value.base);
-
-				if (effect->subtype) {
-					strnfmt(desc, sizeof(desc), effect_desc(effect),
-							"a monster", dist);
-				} else {
-					strnfmt(desc, sizeof(desc), effect_desc(effect), "you",
-							dist);
-				}
-				break;
-			}
-			case EFINFO_QUAKE: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						effect->radius);
-				break;
-			}
-
-			/* Object generated balls are elemental */
-			case EFINFO_BALL: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->subtype].player_desc,
-						effect->radius, dice_string);
-				if (boost)
-					my_strcat(desc, format(", which your device skill increases by %d per cent", boost),
-							  sizeof(desc));
-				break;
-			}
-
-			case EFINFO_SPOT: {
-				int i_radius = effect->other ? effect->other : effect->radius;
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->subtype].player_desc,
-						effect->radius, i_radius, dice_string);
-				break;
-			}
-
-			/* Object generated breaths are elemental */
-			case EFINFO_BREATH: {
-				/* Special treatment for several random breaths */
-				if (random_breath) {
-					my_strcat(breaths,
-							  projections[effect->subtype].player_desc,
-							  sizeof(breaths));
-					if (random_choices > 3) {
-						my_strcat(breaths, ", ", sizeof(breaths));
-					} else if (random_choices == 3) {
-						my_strcat(breaths, " or ", sizeof(breaths));
-					}
-					random_choices--;
-
-					if ((!effect->next) || (effect->next->index != EF_BREATH)) {
-						random_breath = false;
-					}
-					strnfmt(desc, sizeof(desc), effect_desc(effect), breaths,
-							effect->other, dice_string);
-				} else {
-					strnfmt(desc, sizeof(desc), effect_desc(effect),
-							projections[effect->subtype].player_desc,
-							effect->other, dice_string);
-				}
-				break;
-			}
-
-			case EFINFO_SHORT: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect), 
-						projections[effect->subtype].player_desc,
-						effect->radius +
-						effect->other ? effect->other / player->lev : 0,
-						dice_string);
-				break;
-			}
-
-			/* Currently no object generated lashes */
-			case EFINFO_LASH: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->subtype].lash_desc,
-						effect->subtype);
-				break;
-			}
-
-			/* Bolts that inflict status */
-			case EFINFO_BOLT: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->subtype].desc);
-				break;
-			}
-			/* Bolts and beams that damage */
-			case EFINFO_BOLTD: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->subtype].desc, dice_string);
-				if (boost)
-					my_strcat(desc, format(", which your device skill increases by %d per cent", boost),
-							  sizeof(desc));
-				break;
-			}
-			case EFINFO_TOUCH: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						projections[effect->subtype].desc);
-				break;
-			}
-			case EFINFO_TAP: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect),
-						dice_string);
-				break;
-			}
-			case EFINFO_NONE: {
-				strnfmt(desc, sizeof(desc), effect_desc(effect));
-				break;
-			}
-			default: {
-				msg("Bad effect description passed to describe_effect(). Please report this bug.");
-				return false;
-			}
-			}
-
-			do {
-				if (random_breath && effect->index != EF_RANDOM) break;
-				if (isdigit((unsigned char) *next_char))
-					textblock_append_c(tb, COLOUR_L_GREEN, "%c", *next_char);
-				else
-					textblock_append(tb, "%c", *next_char);
-			} while (*next_char++);
-
-			/* Random choices need special treatment - note that this code
-			 * assumes that RANDOM and the random choices will be the last
-			 * effect in the object/activation description */
-			if (random_breath) {
-				/* Handled in effect description */
-				;
-			} else if (random_choices >= 1) {
-				if (effect->index == EF_RANDOM)
-					;
-				else if (random_choices > 2)
-					textblock_append(tb, ", ");
-				else if (random_choices == 2)
-					textblock_append(tb, " or ");
-				random_choices--;
-			} else if (effect->next) {
-				if (effect->next->next && (effect->next->index != EF_RANDOM))
-					textblock_append(tb, ", ");
-				else
-					textblock_append(tb, " and ");
-			}
-			effect = effect->next;
+		tbe = effect_describe(effect, prefix, boost, false);
+		if (! tbe) {
+			return false;
 		}
+		textblock_append_textblock(tb, tbe);
+		textblock_free(tbe);
 	}
 
 	textblock_append(tb, ".\n");
@@ -2015,6 +1776,7 @@ static bool describe_origin(textblock *tb, const struct object *obj, bool terse)
 	const char *dropper = NULL;
 	const char *article;
 	bool unique = false;
+	bool comma = false;
 
 	/* Only give this info in chardumps if wieldable */
 	if (terse && !obj_can_wear(obj))
@@ -2039,6 +1801,9 @@ static bool describe_origin(textblock *tb, const struct object *obj, bool terse)
 		if (rf_has(obj->origin_race->flags, RF_UNIQUE)) {
 			unique = true;
 		}
+		if (rf_has(obj->origin_race->flags, RF_NAME_COMMA)) {
+			comma = true;
+		}
 	} else {
 		dropper = "monster lost to history";
 	}
@@ -2049,12 +1814,15 @@ static bool describe_origin(textblock *tb, const struct object *obj, bool terse)
 		my_strcpy(name, article, sizeof(name));
 		my_strcat(name, dropper, sizeof(name));
 	}
+	if (comma) {
+		my_strcat(name, ",", sizeof(name));
+	}
 
 	/* Print an appropriate description */
 	switch (origins[origin].args)
 	{
 		case -1: return false;
-		case 0: textblock_append(tb, origins[origin].desc); break;
+		case 0: textblock_append(tb, "%s", origins[origin].desc); break;
 		case 1: textblock_append(tb, origins[origin].desc, loot_spot);
 				break;
 		case 2:
