@@ -38,27 +38,27 @@
  * obj_alloc[ilv * (z_info->k_max + 1) + z_info->k_max], that an item whose
  * index is less than itm occurs at level, ilv.
  */
-static u32b *obj_alloc;
+static uint32_t *obj_alloc;
 
 /**
  * Has the same layout and interpretation as obj_alloc, but only items that
  * are good or better contribute to the cumulative probability distribution.
  */
-static u32b *obj_alloc_great;
+static uint32_t *obj_alloc_great;
 
 /**
  * Store the total allocation value for each tval by level.  The value at
  * ilv * TV_MAX + tval is the total for tval at the level, ilv.
  */
-static u32b *obj_total_tval;
+static uint32_t *obj_total_tval;
 
 /**
  * Same layout and interpretation as obj_total_tval, but only items that are
  * good or better contribute.
  */
-static u32b *obj_total_tval_great;
+static uint32_t *obj_total_tval_great;
 
-static s16b alloc_ego_size = 0;
+static int16_t alloc_ego_size = 0;
 static alloc_entry *alloc_ego_table;
 
 struct money {
@@ -77,10 +77,10 @@ static void alloc_init_objects(void) {
 	int k_max = z_info->k_max;
 
 	/* Allocate */
-	obj_alloc = mem_alloc((z_info->max_obj_depth + 1) * (k_max + 1) * sizeof(*obj_alloc));
-	obj_alloc_great = mem_alloc((z_info->max_obj_depth + 1) * (k_max + 1) * sizeof(*obj_alloc_great));
-	obj_total_tval = mem_zalloc((z_info->max_obj_depth + 1) * TV_MAX * sizeof(*obj_total_tval));
-	obj_total_tval_great = mem_zalloc((z_info->max_obj_depth + 1) * TV_MAX * sizeof(*obj_total_tval));
+	obj_alloc = mem_alloc_alt((z_info->max_obj_depth + 1) * (k_max + 1) * sizeof(*obj_alloc));
+	obj_alloc_great = mem_alloc_alt((z_info->max_obj_depth + 1) * (k_max + 1) * sizeof(*obj_alloc_great));
+	obj_total_tval = mem_zalloc_alt((z_info->max_obj_depth + 1) * TV_MAX * sizeof(*obj_total_tval));
+	obj_total_tval_great = mem_zalloc_alt((z_info->max_obj_depth + 1) * TV_MAX * sizeof(*obj_total_tval));
 
 	/* The cumulative chance starts at zero for each level. */
 	for (lev = 0; lev <= z_info->max_obj_depth; lev++) {
@@ -101,7 +101,7 @@ static void alloc_init_objects(void) {
 
 			/* Add to the cumulative prob. in the standard table */
 			if ((lev < min) || (lev > max)) rarity = 0;
-			assert(rarity >= 0 && obj_alloc[(lev * (k_max + 1)) + item] <= (u32b)-1 - rarity);
+			assert(rarity >= 0 && obj_alloc[(lev * (k_max + 1)) + item] <= (uint32_t)-1 - rarity);
 			obj_alloc[(lev * (k_max + 1)) + item + 1] =
 				obj_alloc[(lev * (k_max + 1)) + item] + rarity;
 
@@ -216,10 +216,10 @@ static void cleanup_obj_make(void) {
 	}
 	mem_free(money_type);
 	mem_free(alloc_ego_table);
-	mem_free(obj_total_tval_great);
-	mem_free(obj_total_tval);
-	mem_free(obj_alloc_great);
-	mem_free(obj_alloc);
+	mem_free_alt(obj_total_tval_great);
+	mem_free_alt(obj_total_tval);
+	mem_free_alt(obj_alloc_great);
+	mem_free_alt(obj_alloc);
 }
 
 /*** Make an ego item ***/
@@ -312,7 +312,7 @@ static int random_high_resist(struct object *obj, int *resist)
  * tbl[i] <= p < tbl[i + 1].  p must be less than tbl[n - 1] and tbl[0] must be
  * zero.
  */
-static int binary_search_probtable(const u32b *tbl, int n, u32b p)
+static int binary_search_probtable(const uint32_t *tbl, int n, uint32_t p)
 {
 	int ilow = 0, ihigh = n;
 
@@ -574,22 +574,20 @@ void copy_artifact_data(struct object *obj, const struct artifact *art)
  * We *prefer* to create the special artifacts in order, but this is
  * normally outweighed by the "rarity" rolls for those artifacts.
  */
-static struct object *make_artifact_special(int level)
+static struct object *make_artifact_special(int level, int tval)
 {
 	int i;
 	struct object *new_obj;
 
 	/* No artifacts, do nothing */
-	if (OPT(player, birth_no_artifacts))
-		return NULL;
+	if (OPT(player, birth_no_artifacts)) return NULL;
 
 	/* No artifacts in the town */
-	if (!player->depth)
-		return NULL;
+	if (!player->depth) return NULL;
 
 	/* Check the special artifacts */
 	for (i = 0; i < z_info->a_max; ++i) {
-		struct artifact *art = &a_info[i];
+		const struct artifact *art = &a_info[i];
 		struct object_kind *kind = lookup_kind(art->tval, art->sval);
 
 		/* Skip "empty" artifacts */
@@ -598,11 +596,14 @@ static struct object *make_artifact_special(int level)
 		/* Make sure the kind was found */
 		if (!kind) continue;
 
+		/* Make sure it's the right tval (if given) */
+		if (tval && (tval != art->tval)) continue;
+
 		/* Skip non-special artifacts */
 		if (!kf_has(kind->kind_flags, KF_INSTA_ART)) continue;
 
 		/* Cannot make an artifact twice */
-		if (art->created) continue;
+		if (is_artifact_created(art)) continue;
 
 		/* Enforce minimum "depth" (loosely) */
 		if (art->alloc_min > player->depth) {
@@ -639,7 +640,7 @@ static struct object *make_artifact_special(int level)
 		copy_artifact_data(new_obj, art);
 
 		/* Mark the artifact as "created" */
-		art->created = true;
+		mark_artifact_created(art, true);
 
 		/* Success */
 		return new_obj;
@@ -662,22 +663,19 @@ static struct object *make_artifact_special(int level)
 static bool make_artifact(struct object *obj)
 {
 	int i;
-	bool art_ok = true;
 
 	/* Make sure birth no artifacts isn't set */
-	if (OPT(player, birth_no_artifacts)) art_ok = false;
-
-	if (!art_ok) return (false);
+	if (OPT(player, birth_no_artifacts)) return false;
 
 	/* No artifacts in the town */
-	if (!player->depth) return (false);
+	if (!player->depth) return false;
 
 	/* Paranoia -- no "plural" artifacts */
-	if (obj->number != 1) return (false);
+	if (obj->number != 1) return false;
 
 	/* Check the artifact list (skip the "specials") */
 	for (i = 0; !obj->artifact && i < z_info->a_max; i++) {
-		struct artifact *art = &a_info[i];
+		const struct artifact *art = &a_info[i];
 		struct object_kind *kind = lookup_kind(art->tval, art->sval);
 
 		/* Skip "empty" items */
@@ -690,7 +688,7 @@ static bool make_artifact(struct object *obj)
 		if (kf_has(kind->kind_flags, KF_INSTA_ART)) continue;
 
 		/* Cannot make an artifact twice */
-		if (art->created) continue;
+		if (is_artifact_created(art)) continue;
 
 		/* Must have the correct fields */
 		if (art->tval != obj->tval) continue;
@@ -718,7 +716,7 @@ static bool make_artifact(struct object *obj)
 
 	if (obj->artifact) {
 		copy_artifact_data(obj, obj->artifact);
-		obj->artifact->created = true;
+		mark_artifact_created(obj->artifact, true);
 		return true;
 	}
 
@@ -748,7 +746,7 @@ bool make_fake_artifact(struct object *obj, const struct artifact *artifact)
 
 	/* Create the artifact */
 	object_prep(obj, kind, 0, MAXIMISE);
-	obj->artifact = (struct artifact *)artifact;
+	obj->artifact = artifact;
 	copy_artifact_data(obj, artifact);
 
 	return (true);
@@ -937,7 +935,7 @@ int apply_magic(struct object *obj, int lev, bool allow_artifacts, bool good,
 				bool great, bool extra_roll)
 {
 	int i;
-	s16b power = 0;
+	int16_t power = 0;
 
 	/* Chance of being `good` and `great` */
 	/* This has changed over the years:
@@ -1077,8 +1075,8 @@ bool kind_is_good(const struct object_kind *kind)
  */
 static struct object_kind *get_obj_num_by_kind(int level, bool good, int tval)
 {
-	const u32b *objects;
-	u32b total, value;
+	const uint32_t *objects;
+	uint32_t total, value;
 	int item;
 
 	assert(level >= 0 && level <= z_info->max_obj_depth);
@@ -1105,7 +1103,7 @@ static struct object_kind *get_obj_num_by_kind(int level, bool good, int tval)
 	 */
 	for (item = 0; item < z_info->k_max; item++) {
 		if (objkind_byid(item)->tval == tval) {
-			u32b prob = objects[item + 1] - objects[item];
+			uint32_t prob = objects[item + 1] - objects[item];
 
 			if (value < prob) break;
 			value -= prob;
@@ -1123,8 +1121,8 @@ static struct object_kind *get_obj_num_by_kind(int level, bool good, int tval)
  */
 struct object_kind *get_obj_num(int level, bool good, int tval)
 {
-	const u32b *objects;
-	u32b value;
+	const uint32_t *objects;
+	uint32_t value;
 	int item;
 
 	/* Occasional level boost */
@@ -1170,7 +1168,7 @@ struct object_kind *get_obj_num(int level, bool good, int tval)
  * \return a pointer to the newly allocated object, or NULL on failure.
  */
 struct object *make_object(struct chunk *c, int lev, bool good, bool great,
-						   bool extra_roll, s32b *value, int tval)
+		bool extra_roll, int32_t *value, int tval)
 {
 	int base, tries = 3;
 	struct object_kind *kind = NULL;
@@ -1178,7 +1176,7 @@ struct object *make_object(struct chunk *c, int lev, bool good, bool great,
 
 	/* Try to make a special artifact */
 	if (one_in_(good ? 10 : 1000)) {
-		new_obj = make_artifact_special(lev);
+		new_obj = make_artifact_special(lev, tval);
 		if (new_obj) {
 			if (value) *value = object_value_real(new_obj, 1);
 			return new_obj;
@@ -1212,7 +1210,7 @@ struct object *make_object(struct chunk *c, int lev, bool good, bool great,
 	apply_magic(new_obj, lev, true, good, great, extra_roll);
 
 	/* Generate multiple items */
-	if (kind->gen_mult_prob >= randint1(100))
+	if (!new_obj->artifact && kind->gen_mult_prob >= randint1(100))
 		new_obj->number = randcalc(kind->stack_size, lev, RANDOMISE);
 
 	if (new_obj->number > new_obj->kind->base->max_stack)
@@ -1245,7 +1243,7 @@ void acquirement(struct loc grid, int level, int num, bool great)
 		if (!nice_obj) continue;
 
 		nice_obj->origin = ORIGIN_ACQUIRE;
-		nice_obj->origin_depth = player->depth;
+		nice_obj->origin_depth = convert_depth_to_origin(player->depth);
 
 		/* Drop the object */
 		drop_near(cave, &nice_obj, 0, grid, true, false);
@@ -1288,7 +1286,7 @@ struct object_kind *money_kind(const char *name, int value)
  * \param coin_type the name of the type of money object to make
  * \return a pointer to the newly minted cash (cannot fail)
  */
-struct object *make_gold(int lev, char *coin_type)
+struct object *make_gold(int lev, const char *coin_type)
 {
 	/* This average is 16 at dlev0, 80 at dlev40, 176 at dlev100. */
 	int avg = (16 * lev)/10 + 16;
@@ -1308,7 +1306,7 @@ struct object *make_gold(int lev, char *coin_type)
 		value *= 5;
 	}
 
-	/* Cap gold at max short (or alternatively make pvals s32b) */
+	/* Cap gold at max short (or alternatively make pvals int32_t) */
 	if (value >= SHRT_MAX) {
 		value = SHRT_MAX - randint0(200);
 	}
