@@ -78,26 +78,48 @@ int breakage_chance(const struct object *obj, bool hit_target) {
 }
 
 /**
- * Return the player's chance to hit with a particular weapon.
+ * Calculate the player's base melee to-hit value without regard to a specific
+ * monster.
+ * See also: chance_of_missile_hit_base
+ *
+ * \param p The player
+ * \param weapon The player's weapon
  */
-int chance_of_melee_hit(const struct player *p, const struct object *weapon)
+int chance_of_melee_hit_base(const struct player *p,
+		const struct object *weapon)
 {
-	int chance, bonus = p->state.to_h;
-
-	if (weapon)
-		bonus += weapon->to_h;
-	chance = p->state.skills[SKILL_TO_HIT_MELEE] + bonus * BTH_PLUS_ADJ;
-	return chance;
+	int bonus = p->state.to_h + (weapon ? weapon->to_h : 0);
+	return p->state.skills[SKILL_TO_HIT_MELEE] + bonus * BTH_PLUS_ADJ;
 }
 
 /**
- * Return the player's chance to hit with a particular missile and
- * (optionally) launcher.
+ * Calculate the player's melee to-hit value against a specific monster.
+ * See also: chance_of_missile_hit
+ *
+ * \param p The player
+ * \param weapon The player's weapon
+ * \param mon The monster
  */
-static int chance_of_missile_hit(const struct player *p,
+static int chance_of_melee_hit(const struct player *p,
+		const struct object *weapon, const struct monster *mon)
+{
+	int chance = chance_of_melee_hit_base(p, weapon);
+	/* Non-visible targets have a to-hit penalty of 50% */
+	return monster_is_visible(mon) ? chance : chance / 2;
+}
+
+/**
+ * Calculate the player's base missile to-hit value without regard to a specific
+ * monster.
+ * See also: chance_of_melee_hit_base
+ *
+ * \param p The player
+ * \param missile The missile to launch
+ * \param launcher The launcher to use (optional)
+ */
+static int chance_of_missile_hit_base(const struct player *p,
 								 const struct object *missile,
-								 const struct object *launcher,
-								 struct loc grid)
+								 const struct object *launcher)
 {
 	int bonus = missile->to_h;
 	int chance;
@@ -118,30 +140,83 @@ static int chance_of_missile_hit(const struct player *p,
 		chance = p->state.skills[SKILL_TO_HIT_BOW] + bonus * BTH_PLUS_ADJ;
 	}
 
-	return chance - distance(p->grid, grid);
+	return chance;
 }
 
 /**
- * Determine if the player "hits" a monster.
+ * Calculate the player's missile to-hit value against a specific monster.
+ * See also: chance_of_melee_hit
+ *
+ * \param p The player
+ * \param missile The missile to launch
+ * \param launcher Optional launcher to use (thrown weapons use no launcher)
+ * \param mon The monster
  */
-bool test_hit(int chance, int ac, int vis) {
-	int k = randint0(100);
-
-	/* There is an automatic 12% chance to hit,
-	 * and 5% chance to miss.
-	 */
-	if (k < 17) return k < 12;
-
-	/* Penalize invisible targets */
-	if (!vis) chance /= 2;
-
-	/* Starting a bit higher up on the scale */
-	if (chance < 9) chance = 9;
-
-	/* Power competes against armor */
-	return randint0(chance) >= (ac * 2 / 3);
+static int chance_of_missile_hit(const struct player *p,
+	const struct object *missile, const struct object *launcher,
+	const struct monster *mon)
+{
+	int chance = chance_of_missile_hit_base(p, missile, launcher);
+	/* Penalize for distance */
+	chance -= distance(p->grid, mon->grid);
+	/* Non-visible targets have a to-hit penalty of 50% */
+	return monster_is_obvious(mon) ? chance : chance / 2;
 }
 
+/**
+ * Determine if a hit roll is successful against the target AC.
+ * See also: hit_chance
+ *
+ * \param to_hit To total to-hit value to use
+ * \param ac The AC to roll against
+ */
+bool test_hit(int to_hit, int ac)
+{
+	random_chance c;
+	hit_chance(&c, to_hit, ac);
+	return random_chance_check(c);
+}
+
+/**
+ * Return a random_chance by reference, which represents the likelihood of a
+ * hit roll succeeding for the given to_hit and ac values. The hit calculation
+ * will:
+ *
+ * Always hit 12% of the time
+ * Always miss 5% of the time
+ * Put a floor of 9 on the to-hit value
+ * Roll between 0 and the to-hit value
+ * The outcome must be >= AC*2/3 to be considered a hit
+ *
+ * \param chance The random_chance to return-by-reference
+ * \param to_hit The to-hit value to use
+ * \param ac The AC to roll against
+ */
+void hit_chance(random_chance *chance, int to_hit, int ac)
+{
+	/* Percentages scaled to 10,000 to avoid rounding error */
+	const int HUNDRED_PCT = 10000;
+	const int ALWAYS_HIT = 1200;
+	const int ALWAYS_MISS = 500;
+
+	/* Put a floor on the to_hit */
+	to_hit = MAX(9, to_hit);
+
+	/* Calculate the hit percentage */
+	chance->numerator = MAX(0, to_hit - ac * 2 / 3);
+	chance->denominator = to_hit;
+
+	/* Convert the ratio to a scaled percentage */
+	chance->numerator = HUNDRED_PCT * chance->numerator / chance->denominator;
+	chance->denominator = HUNDRED_PCT;
+
+	/* The calculated rate only applies when the guaranteed hit/miss don't */
+	chance->numerator = chance->numerator *
+			(HUNDRED_PCT - ALWAYS_MISS - ALWAYS_HIT) / HUNDRED_PCT;
+
+	/* Add in the guaranteed hit */
+	chance->numerator += ALWAYS_HIT;
+}
 
 /**
  * ------------------------------------------------------------------------
@@ -152,7 +227,7 @@ bool test_hit(int chance, int ac, int vis) {
  * Much of this table is not intended ever to be used, and is included
  * only to handle possible inflation elsewhere. -LM-
  */
-byte deadliness_conversion[151] =
+uint8_t deadliness_conversion[151] =
   {
     0,
     5,  10,  14,  18,  22,  26,  30,  33,  36,  39,
@@ -218,6 +293,7 @@ static bool is_debuffed(const struct monster *monster)
 {
 	return monster->m_timed[MON_TMD_CONF] > 0 ||
 			monster->m_timed[MON_TMD_HOLD] > 0 ||
+			monster->m_timed[MON_TMD_FEAR] > 0 ||
 			monster->m_timed[MON_TMD_STUN] > 0;
 }
 
@@ -229,7 +305,7 @@ static bool is_debuffed(const struct monster *monster)
 static int critical_shot(const struct player *p,
 		const struct monster *monster,
 		int weight, int plus,
-		int dam, u32b *msg_type)
+		int dam, uint32_t *msg_type)
 {
 	int debuff_to_hit = is_debuffed(monster) ? DEBUFF_CRITICAL_HIT : 0;
 	int chance = weight + (p->state.to_h + plus + debuff_to_hit) * 4 + p->lev * 2;
@@ -258,13 +334,13 @@ static int critical_shot(const struct player *p,
  * Factor in item weight, total plusses, and player level.
  */
 static int o_critical_shot(const struct player *p,
-						   const struct monster *monster,
-						   const struct object *missile,
-						   const struct object *launcher,
-						   u32b *msg_type)
+		const struct monster *monster,
+		const struct object *missile,
+		const struct object *launcher,
+		uint32_t *msg_type)
 {
 	int debuff_to_hit = is_debuffed(monster) ? DEBUFF_CRITICAL_HIT : 0;
-	int power = chance_of_missile_hit(p, missile, launcher, monster->grid)
+	int power = chance_of_missile_hit_base(p, missile, launcher)
 		+ debuff_to_hit;
 	int add_dice = 0;
 
@@ -301,7 +377,7 @@ static int o_critical_shot(const struct player *p,
 static int critical_melee(const struct player *p,
 		const struct monster *monster,
 		int weight, int plus,
-		int dam, u32b *msg_type)
+		int dam, uint32_t *msg_type)
 {
 	int debuff_to_hit = is_debuffed(monster) ? DEBUFF_CRITICAL_HIT : 0;
 	int power = weight + randint1(650);
@@ -337,11 +413,11 @@ static int critical_melee(const struct player *p,
  * Factor in weapon weight, total plusses, player level.
  */
 static int o_critical_melee(const struct player *p,
-							const struct monster *monster,
-							const struct object *obj, u32b *msg_type)
+		const struct monster *monster,
+		const struct object *obj, uint32_t *msg_type)
 {
 	int debuff_to_hit = is_debuffed(monster) ? DEBUFF_CRITICAL_HIT : 0;
-	int power = (chance_of_melee_hit(p, obj) + debuff_to_hit) / 3;
+	int power = (chance_of_melee_hit_base(p, obj) + debuff_to_hit) / 3;
 	int add_dice = 0;
 
 	/* Test for critical hit - chance power / (power + 240) */
@@ -377,15 +453,15 @@ static int o_critical_melee(const struct player *p,
  */
 static int melee_damage(const struct monster *mon, struct object *obj, int b, int s)
 {
-	int dmg = damroll(obj->dd, obj->ds);
+	int dmg = (obj) ? damroll(obj->dd, obj->ds) : 1;
 
 	if (s) {
 		dmg *= slays[s].multiplier;
 	} else if (b) {
-		dmg *= get_monster_brand_multiplier(mon, &brands[b]);
+		dmg *= get_monster_brand_multiplier(mon, &brands[b], false);
 	}
 
-	dmg += obj->to_d;
+	if (obj) dmg += obj->to_d;
 
 	return dmg;
 }
@@ -397,21 +473,21 @@ static int melee_damage(const struct monster *mon, struct object *obj, int b, in
  * criticals add extra dice.
  */
 static int o_melee_damage(struct player *p, const struct monster *mon,
-		struct object *obj, int b, int s, u32b *msg_type)
+		struct object *obj, int b, int s, uint32_t *msg_type)
 {
-	int dice = obj->dd;
+	int dice = (obj) ? obj->dd : 1;
 	int sides, dmg, add = 0;
 	bool extra;
 
 	/* Get the average value of a single damage die. (x10) */
-	int die_average = (10 * (obj->ds + 1)) / 2;
+	int die_average = (10 * (((obj) ? obj->ds : 1) + 1)) / 2;
 
 	/* Adjust the average for slays and brands. (10x inflation) */
 	if (s) {
 		die_average *= slays[s].o_multiplier;
 		add = slays[s].o_multiplier - 10;
 	} else if (b) {
-		int bmult = get_monster_brand_multiplier(mon, &brands[b]);
+		int bmult = get_monster_brand_multiplier(mon, &brands[b], true);
 
 		die_average *= bmult;
 		add = bmult - 10;
@@ -420,7 +496,8 @@ static int o_melee_damage(struct player *p, const struct monster *mon,
 	}
 
 	/* Apply deadliness to average. (100x inflation) */
-	apply_deadliness(&die_average, MIN(obj->to_d + p->state.to_d, 150));
+	apply_deadliness(&die_average,
+		MIN(((obj) ? obj->to_d : 0) + p->state.to_d, 150));
 
 	/* Calculate the actual number of sides to each die. */
 	sides = (2 * die_average) - 10000;
@@ -428,8 +505,11 @@ static int o_melee_damage(struct player *p, const struct monster *mon,
 	sides /= 10000;
 	sides += (extra ? 1 : 0);
 
-	/* Get number of critical dice */
-	dice += o_critical_melee(p, mon, obj, msg_type);
+	/*
+	 * Get number of critical dice; for now, excluding criticals for
+	 * unarmed combat
+	 */
+	if (obj) dice += o_critical_melee(p, mon, obj, msg_type);
 
 	/* Roll out the damage. */
 	dmg = damroll(dice, sides);
@@ -454,7 +534,7 @@ static int ranged_damage(struct player *p, const struct monster *mon,
 
 	/* If we have a slay or brand, modify the multiplier appropriately */
 	if (b) {
-		mult += get_monster_brand_multiplier(mon, &brands[b]);
+		mult += get_monster_brand_multiplier(mon, &brands[b], false);
 	} else if (s) {
 		mult += slays[s].multiplier;
 	}
@@ -482,8 +562,8 @@ static int ranged_damage(struct player *p, const struct monster *mon,
  * damage dice, criticals add extra dice.
  */
 static int o_ranged_damage(struct player *p, const struct monster *mon,
-						   struct object *missile, struct object *launcher,
-						   int b, int s, u32b *msg_type)
+		struct object *missile, struct object *launcher,
+		int b, int s, uint32_t *msg_type)
 {
 	int mult = (launcher ? p->state.ammo_mult : 1);
 	int dice = missile->dd;
@@ -498,7 +578,7 @@ static int o_ranged_damage(struct player *p, const struct monster *mon,
 
 	/* Adjust the average for slays and brands. (10x inflation) */
 	if (b) {
-		int bmult = get_monster_brand_multiplier(mon, &brands[b]);
+		int bmult = get_monster_brand_multiplier(mon, &brands[b], true);
 
 		die_average *= bmult;
 		add = bmult - 10;
@@ -622,18 +702,16 @@ bool py_attack_real(struct player *p, struct loc grid, bool *fear)
 	struct object *obj = equipped_item_by_slot_name(p, "weapon");
 
 	/* Information about the attack */
-	int chance = chance_of_melee_hit(p, obj);
 	int drain = 0;
 	int splash = 0;
 	bool do_quake = false;
 	bool success = false;
 
-	/* Default to punching for one damage */
 	char verb[20];
-	int dmg = 1;
-	u32b msg_type = MSG_HIT;
+	uint32_t msg_type = MSG_HIT;
+	int j, b, s, weight, dmg;
 
-	/* Default to punching for one damage */
+	/* Default to punching */
 	my_strcpy(verb, "punch", sizeof(verb));
 
 	/* Extract monster name (or "it") */
@@ -657,7 +735,7 @@ bool py_attack_real(struct player *p, struct loc grid, bool *fear)
 	mon_clear_timed(mon, MON_TMD_HOLD, MON_TMD_FLG_NOTIFY);
 
 	/* See if the player hit */
-	success = test_hit(chance, mon->race->ac, monster_is_visible(mon));
+	success = test_hit(chance_of_melee_hit(p, obj, mon), mon->race->ac);
 
 	/* If a miss, skip this hit */
 	if (!success) {
@@ -672,43 +750,50 @@ bool py_attack_real(struct player *p, struct loc grid, bool *fear)
 		return false;
 	}
 
-	/* Handle normal weapon */
 	if (obj) {
-		int j;
-		int b = 0, s = 0;
-		int weight = obj->weight;
-
+		/* Handle normal weapon */
+		weight = obj->weight;
 		my_strcpy(verb, "hit", sizeof(verb));
+	} else {
+		weight = 0;
+	}
 
-		/* Best attack from all slays or brands on all non-launcher equipment */
-		for (j = 2; j < p->body.count; j++) {
-			struct object *obj_local = slot_object(p, j);
-			if (obj_local)
-				improve_attack_modifier(obj_local, mon, &b, &s, verb, false);
-		}
+	/* Best attack from all slays or brands on all non-launcher equipment */
+	b = 0;
+	s = 0;
+	for (j = 2; j < p->body.count; j++) {
+		struct object *obj_local = slot_object(p, j);
+		if (obj_local)
+			improve_attack_modifier(p, obj_local, mon, &b, &s,
+				verb, false);
+	}
 
-		/* Get the best attack from all slays or brands - weapon or temporary */
-		improve_attack_modifier(obj, mon, &b, &s, verb, false);
-		improve_attack_modifier(NULL, mon, &b, &s, verb, false);
+	/* Get the best attack from all slays or brands - weapon or temporary */
+	if (obj) {
+		improve_attack_modifier(p, obj, mon, &b, &s, verb, false);
+	}
+	improve_attack_modifier(p, NULL, mon, &b, &s, verb, false);
 
-		/* Get the damage */
-		if (!OPT(p, birth_percent_damage)) {
-			dmg = melee_damage(mon, obj, b, s);
-			dmg = critical_melee(p, mon, weight, obj->to_h, dmg, &msg_type);
-		} else {
-			dmg = o_melee_damage(p, mon, obj, b, s, &msg_type);
-		}
+	/* Get the damage */
+	if (!OPT(p, birth_percent_damage)) {
+		dmg = melee_damage(mon, obj, b, s);
+		/* For now, exclude criticals on unarmed combat */
+		if (obj) dmg = critical_melee(p, mon, weight, obj->to_h,
+			dmg, &msg_type);
+	} else {
+		dmg = o_melee_damage(p, mon, obj, b, s, &msg_type);
+	}
 
-		/* Splash damage and earthquakes */
-		splash = (weight * dmg) / 100;
-		if (player_of_has(p, OF_IMPACT) && dmg > 50) {
-			do_quake = true;
-			equip_learn_flag(p, OF_IMPACT);
-		}
+	/* Splash damage and earthquakes */
+	splash = (weight * dmg) / 100;
+	if (player_of_has(p, OF_IMPACT) && dmg > 50) {
+		do_quake = true;
+		equip_learn_flag(p, OF_IMPACT);
 	}
 
 	/* Learn by use */
 	equip_learn_on_melee_attack(p);
+	learn_brand_slay_from_melee(p, obj, mon);
 
 	/* Apply the player damage bonuses */
 	if (!OPT(p, birth_percent_damage)) {
@@ -753,7 +838,7 @@ bool py_attack_real(struct player *p, struct loc grid, bool *fear)
 
 	/* Damage, check for hp drain, fear and death */
 	drain = MIN(mon->hp, dmg);
-	stop = mon_take_hit(mon, dmg, fear, NULL);
+	stop = mon_take_hit(mon, p, dmg, fear, NULL);
 
 	/* Small chance of bloodlust side-effects */
 	if (p->timed[TMD_BLOODLUST] && one_in_(50)) {
@@ -782,7 +867,7 @@ bool py_attack_real(struct player *p, struct loc grid, bool *fear)
 /**
  * Attempt a shield bash; return true if the monster dies
  */
-bool attempt_shield_bash(struct player *p, struct monster *mon, bool *fear)
+static bool attempt_shield_bash(struct player *p, struct monster *mon, bool *fear)
 {
 	struct object *weapon = slot_object(p, slot_by_name(p, "weapon"));
 	struct object *shield = slot_object(p, slot_by_name(p, "arm"));
@@ -841,7 +926,7 @@ bool attempt_shield_bash(struct player *p, struct monster *mon, bool *fear)
 	}
 
 	/* Damage, check for fear and death. */
-	if (mon_take_hit(mon, bash_dam, fear, NULL)) return true;
+	if (mon_take_hit(mon, p, bash_dam, fear, NULL)) return true;
 
 	/* Stunning. */
 	if (bash_quality + p->lev > randint1(200 + mon->race->level * 8)) {
@@ -887,7 +972,7 @@ void py_attack(struct player *p, struct loc grid)
 
 	/* Reward BGs with 5% of max SPs, min 1/2 point */
 	if (player_has(p, PF_COMBAT_REGEN)) {
-		s32b sp_gain = (s32b)(MAX(p->msp, 10) << 16) / 20;
+		int32_t sp_gain = (int32_t)(MAX(p->msp, 10) << 16) / 20;
 		player_adjust_mana_precise(p, sp_gain);
 	}
 
@@ -938,8 +1023,6 @@ static void ranged_helper(struct player *p,	struct object *obj, int dir,
 {
 	int i, j;
 
-	char o_name[80];
-
 	int path_n;
 	struct loc path_g[256];
 
@@ -972,18 +1055,15 @@ static void ranged_helper(struct player *p,	struct object *obj, int dir,
 	/* Sound */
 	sound(MSG_SHOOT);
 
-	/* Describe the object */
-	object_desc(o_name, sizeof(o_name), obj, ODESC_FULL | ODESC_SINGULAR);
-
 	/* Actually "fire" the object -- Take a partial turn */
 	p->upkeep->energy_use = (z_info->move_energy * 10 / shots);
 
 	/* Calculate the path */
-	path_n = project_path(path_g, range, grid, target, 0);
+	path_n = project_path(cave, path_g, range, grid, target, 0);
 
 	/* Calculate potenital piercing */
-	if (player->timed[TMD_POWERSHOT] && tval_is_sharp_missile(obj)) {
-		pierce = player->state.ammo_mult;
+	if (p->timed[TMD_POWERSHOT] && tval_is_sharp_missile(obj)) {
+		pierce = p->state.ammo_mult;
 	}
 
 	/* Hack -- Handle stuff */
@@ -1008,7 +1088,7 @@ static void ranged_helper(struct player *p,	struct object *obj, int dir,
 		/* Try the attack on the monster at (x, y) if any */
 		mon = square_monster(cave, path_g[i]);
 		if (mon) {
-			int visible = monster_is_visible(mon);
+			int visible = monster_is_obvious(mon);
 
 			bool fear = false;
 			const char *note_dies = monster_is_destroyed(mon) ? 
@@ -1016,12 +1096,14 @@ static void ranged_helper(struct player *p,	struct object *obj, int dir,
 
 			struct attack_result result = attack(p, obj, grid);
 			int dmg = result.dmg;
-			u32b msg_type = result.msg_type;
+			uint32_t msg_type = result.msg_type;
 			char hit_verb[20];
 			my_strcpy(hit_verb, result.hit_verb, sizeof(hit_verb));
 			mem_free(result.hit_verb);
 
 			if (result.success) {
+				char o_name[80];
+
 				hit_target = true;
 
 				missile_learn_on_ranged_attack(p, obj);
@@ -1029,6 +1111,13 @@ static void ranged_helper(struct player *p,	struct object *obj, int dir,
 				/* Learn by use for other equipped items */
 				equip_learn_on_ranged_attack(p);
 
+				/*
+				 * Describe the object (have most up-to-date
+				 * knowledge now).
+				 */
+				object_desc(o_name, sizeof(o_name), obj,
+					ODESC_FULL | ODESC_SINGULAR, p);
+
 				/* No negative damage; change verb if no damage done */
 				if (dmg <= 0) {
 					dmg = 0;
@@ -1064,16 +1153,16 @@ static void ranged_helper(struct player *p,	struct object *obj, int dir,
 					}
 
 					/* Track this monster */
-					if (monster_is_visible(mon)) {
+					if (monster_is_obvious(mon)) {
 						monster_race_track(p->upkeep, mon->race);
 						health_track(p->upkeep, mon);
 					}
 				}
 
 				/* Hit the monster, check for death */
-				if (!mon_take_hit(mon, dmg, &fear, note_dies)) {
+				if (!mon_take_hit(mon, p, dmg, &fear, note_dies)) {
 					message_pain(mon, dmg);
-					if (fear && monster_is_visible(mon)) {
+					if (fear && monster_is_obvious(mon)) {
 						add_monster_message(mon, MON_MSG_FLEE_IN_TERROR, true);
 					}
 				}
@@ -1090,10 +1179,11 @@ static void ranged_helper(struct player *p,	struct object *obj, int dir,
 	}
 
 	/* Get the missile */
-	if (object_is_carried(p, obj))
-		missile = gear_object_for_use(obj, 1, true, &none_left);
-	else
-		missile = floor_object_for_use(obj, 1, true, &none_left);
+	if (object_is_carried(p, obj)) {
+		missile = gear_object_for_use(p, obj, 1, true, &none_left);
+	} else {
+		missile = floor_object_for_use(p, obj, 1, true, &none_left);
+	}
 
 	/* Terminate piercing */
 	if (p->timed[TMD_POWERSHOT]) {
@@ -1115,19 +1205,18 @@ static struct attack_result make_ranged_shot(struct player *p,
 	struct attack_result result = {false, 0, 0, hit_verb};
 	struct object *bow = equipped_item_by_slot_name(p, "shooting");
 	struct monster *mon = square_monster(cave, grid);
-	int chance = chance_of_missile_hit(p, ammo, bow, grid);
 	int b = 0, s = 0;
 
 	my_strcpy(hit_verb, "hits", 20);
 
-	/* Did we hit it (penalize distance travelled) */
-	if (!test_hit(chance, mon->race->ac, monster_is_visible(mon)))
+	/* Did we hit it */
+	if (!test_hit(chance_of_missile_hit(p, ammo, bow, mon), mon->race->ac))
 		return result;
 
 	result.success = true;
 
-	improve_attack_modifier(ammo, mon, &b, &s, result.hit_verb, true);
-	improve_attack_modifier(bow, mon, &b, &s, result.hit_verb, true);
+	improve_attack_modifier(p, ammo, mon, &b, &s, result.hit_verb, true);
+	improve_attack_modifier(p, bow, mon, &b, &s, result.hit_verb, true);
 
 	if (!OPT(p, birth_percent_damage)) {
 		result.dmg = ranged_damage(p, mon, ammo, bow, b, s);
@@ -1138,6 +1227,7 @@ static struct attack_result make_ranged_shot(struct player *p,
 	}
 
 	missile_learn_on_ranged_attack(p, bow);
+	learn_brand_slay_from_launch(p, ammo, bow, mon);
 
 	return result;
 }
@@ -1152,18 +1242,17 @@ static struct attack_result make_ranged_throw(struct player *p,
 	char *hit_verb = mem_alloc(20 * sizeof(char));
 	struct attack_result result = {false, 0, 0, hit_verb};
 	struct monster *mon = square_monster(cave, grid);
-	int chance = chance_of_missile_hit(p, obj, NULL, grid);
 	int b = 0, s = 0;
 
 	my_strcpy(hit_verb, "hits", 20);
 
 	/* If we missed then we're done */
-	if (!test_hit(chance, mon->race->ac, monster_is_visible(mon)))
+	if (!test_hit(chance_of_missile_hit(p, obj, NULL, mon), mon->race->ac))
 		return result;
 
 	result.success = true;
 
-	improve_attack_modifier(obj, mon, &b, &s, result.hit_verb, true);
+	improve_attack_modifier(p, obj, mon, &b, &s, result.hit_verb, true);
 
 	if (!OPT(p, birth_percent_damage)) {
 		result.dmg = ranged_damage(p, mon, obj, NULL, b, s);
@@ -1177,10 +1266,22 @@ static struct attack_result make_ranged_throw(struct player *p,
 	if (of_has(obj->flags, OF_EXPLODE))
 		result.dmg *= 3;
 
+	learn_brand_slay_from_throw(p, obj, mon);
+
 	return result;
 }
 
 
+/**
+ * Help do_cmd_throw():  restrict which equipment can be thrown.
+ */
+static bool restrict_for_throwing(const struct object *obj)
+{
+	return !object_is_equipped(player->body, obj) ||
+			(tval_is_melee_weapon(obj) && obj_can_takeoff(obj));
+}
+
+
 /**
  * Fire an object from the quiver, pack or floor at a target.
  */
@@ -1194,29 +1295,19 @@ void do_cmd_fire(struct command *cmd) {
 	struct object *bow = equipped_item_by_slot_name(player, "shooting");
 	struct object *obj;
 
-	if (player_is_shapechanged(player)) {
-		msg("You cannot do this while in %s form.",	player->shape->name);
-		if (get_check("Do you want to change back? " )) {
-			player_resume_normal_shape(player);
-		} else {
-			return;
-		}
+	if (!player_get_resume_normal_shape(player, cmd)) {
+		return;
 	}
 
 	/* Get arguments */
 	if (cmd_get_item(cmd, "item", &obj,
 			/* Prompt */ "Fire which ammunition?",
-			/* Error  */ "You have no ammunition to fire.",
+			/* Error  */ "You have no suitable ammunition to fire.",
 			/* Filter */ obj_can_fire,
 			/* Choice */ USE_INVEN | USE_QUIVER | USE_FLOOR | QUIVER_TAGS)
 		!= CMD_OK)
 		return;
 
-	if (cmd_get_target(cmd, "target", &dir) == CMD_OK)
-		player_confuse_dir(player, &dir, false);
-	else
-		return;
-
 	/* Require a usable launcher */
 	if (!bow || !player->state.ammo_tval) {
 		msg("You have nothing to fire with.");
@@ -1235,13 +1326,19 @@ void do_cmd_fire(struct command *cmd) {
 		return;
 	}
 
+	if (cmd_get_target(cmd, "target", &dir) == CMD_OK)
+		player_confuse_dir(player, &dir, false);
+	else
+		return;
+
 	ranged_helper(player, obj, dir, range, shots, attack, ranged_hit_types,
 				  (int) N_ELEMENTS(ranged_hit_types));
 }
 
 
 /**
- * Throw an object from the quiver, pack or floor.
+ * Throw an object from the quiver, pack, floor, or, in limited circumstances,
+ * the equipment.
  */
 void do_cmd_throw(struct command *cmd) {
 	int dir;
@@ -1253,21 +1350,22 @@ void do_cmd_throw(struct command *cmd) {
 	int range;
 	struct object *obj;
 
-	if (player_is_shapechanged(player)) {
-		msg("You cannot do this while in %s form.",	player->shape->name);
-		if (get_check("Do you want to change back? " )) {
-			player_resume_normal_shape(player);
-		} else {
-			return;
-		}
+	if (!player_get_resume_normal_shape(player, cmd)) {
+		return;
 	}
 
-	/* Get arguments */
+	/*
+	 * Get arguments.  Never default to showing the equipment as the first
+	 * list (since throwing the equipped weapon leaves that slot empty will
+	 * have to choose another source anyways).
+	 */
+	if (player->upkeep->command_wrk == USE_EQUIP)
+		player->upkeep->command_wrk = USE_INVEN;
 	if (cmd_get_item(cmd, "item", &obj,
 			/* Prompt */ "Throw which item?",
 			/* Error  */ "You have nothing to throw.",
-			/* Filter */ NULL,
-			/* Choice */ USE_QUIVER | USE_INVEN | USE_FLOOR | SHOW_THROWING)
+			/* Filter */ restrict_for_throwing,
+			/* Choice */ USE_EQUIP | USE_QUIVER | USE_INVEN | USE_FLOOR | SHOW_THROWING)
 		!= CMD_OK)
 		return;
 
@@ -1276,16 +1374,14 @@ void do_cmd_throw(struct command *cmd) {
 	else
 		return;
 
+	if (object_is_equipped(player->body, obj)) {
+		assert(obj_can_takeoff(obj) && tval_is_melee_weapon(obj));
+		inven_takeoff(obj);
+	}
 
 	weight = MAX(obj->weight, 10);
 	range = MIN(((str + 20) * 10) / weight, 10);
 
-	/* Make sure the player isn't throwing wielded items */
-	if (object_is_equipped(player->body, obj)) {
-		msg("You cannot throw wielded items.");
-		return;
-	}
-
 	ranged_helper(player, obj, dir, range, shots, attack, ranged_hit_types,
 				  (int) N_ELEMENTS(ranged_hit_types));
 }
