@@ -30,6 +30,7 @@
 #include "obj-gear.h"
 #include "obj-ignore.h"
 #include "obj-knowledge.h"
+#include "obj-pile.h"
 #include "obj-power.h"
 #include "obj-tval.h"
 #include "obj-util.h"
@@ -1019,163 +1020,221 @@ int equipped_item_slot(struct player_body body, struct object *item)
  * Put the player's inventory and quiver into easily accessible arrays.  The
  * pack may be overfull by one item
  */
-void calc_inventory(struct player_upkeep *upkeep, struct object *gear,
-					struct player_body body)
+void calc_inventory(struct player *p)
 {
-	int i;
-	int old_inven_cnt = upkeep->inven_cnt;
-	struct object **old_quiver = mem_zalloc(z_info->quiver_size *
-												sizeof(struct object *));
-	struct object **old_pack = mem_zalloc(z_info->pack_size *
-											  sizeof(struct object *));
+	int old_inven_cnt = p->upkeep->inven_cnt;
+	int n_stack_split = 0;
+	int n_pack_remaining = z_info->pack_size - pack_slots_used(p);
+	int n_max = 1 + z_info->pack_size + z_info->quiver_size
+		+ p->body.count;
+	struct object **old_quiver = mem_zalloc(z_info->quiver_size
+		* sizeof(*old_quiver));
+	struct object **old_pack = mem_zalloc(z_info->pack_size
+		* sizeof(*old_pack));
+	bool *assigned = mem_alloc(n_max * sizeof(*assigned));
+	struct object *current;
+	int i, j;
+
+	/*
+	 * Equipped items are already taken care of.  Only the others need
+	 * to be tested for assignment to the quiver or pack.
+	 */
+	for (current = p->gear, j = 0; current; current = current->next, ++j) {
+		assert(j < n_max);
+		assigned[j] = object_is_equipped(p->body, current);
+	}
+	for (; j < n_max; ++j) {
+		assigned[j] = false;
+	}
 
 	/* Prepare to fill the quiver */
-	upkeep->quiver_cnt = 0;
+	p->upkeep->quiver_cnt = 0;
 
-	/* Copy the current quiver */
-	for (i = 0; i < z_info->quiver_size; i++)
-		if (upkeep->quiver[i])
-			old_quiver[i] = upkeep->quiver[i];
-		else
-			old_quiver[i] = NULL;
-
-	/* First, allocate inscribed items */
+	/* Copy the current quiver and then leave it empty. */
 	for (i = 0; i < z_info->quiver_size; i++) {
-		struct object *current;
+		if (p->upkeep->quiver[i]) {
+			old_quiver[i] = p->upkeep->quiver[i];
+			p->upkeep->quiver[i] = NULL;
+		} else {
+			old_quiver[i] = NULL;
+		}
+	}
 
-		/* Start with an empty slot */
-		upkeep->quiver[i] = NULL;
+	/* Fill quiver.  First, allocate inscribed items. */
+	for (current = p->gear, j = 0; current; current = current->next, ++j) {
+		int prefslot;
 
-		/* Find the first quiver object with the correct label */
-		for (current = gear; current; current = current->next) {
-			/* Ignore non-ammo */
-			if (!tval_is_ammo(current)) continue;
+		/* Skip already assigned (i.e. equipped) items. */
+		if (assigned[j]) continue;
 
-			/* Allocate inscribed objects if it's the right slot */
-			if (current->note) {
-				const char *s = strchr(quark_str(current->note), '@');
-				if (s && s[1] == 'f') {
-					int choice = s[2] - '0';
+		prefslot  = preferred_quiver_slot(current);
+		if (prefslot >= 0 && prefslot < z_info->quiver_size
+				&& !p->upkeep->quiver[prefslot]) {
+			/*
+			 * The preferred slot is empty.  Split the stack if
+			 * necessary.  Don't allow splitting if it could
+			 * result in overfilling the pack by more than one slot.
+			 */
+			int mult = tval_is_ammo(current) ?
+				1 : z_info->thrown_quiver_mult;
+			struct object *to_quiver;
 
-					/* Correct slot, fill it straight away */
-					if (choice == i) {
-						upkeep->quiver[i] = current;
-						upkeep->quiver_cnt += current->number;
+			if (current->number * mult
+					<= z_info->quiver_slot_size) {
+				to_quiver = current;
+			} else {
+				int nsplit = z_info->quiver_slot_size / mult;
 
-						/* In the quiver counts as worn */
-						object_learn_on_wield(player, current);
-
-						/* Done with this slot */
-						break;
-					}
+				assert(nsplit < current->number);
+				if (nsplit > 0 && n_stack_split
+						<= n_pack_remaining) {
+					/*
+					 * Split off the portion that goes to
+					 * the pack.  Since the stack in the
+					 * quiver is earlier in the gear list it
+					 * will prefer to remain in the quiver
+					 * in future calls to calc_inventory()
+					 * and will be the preferred target for
+					 * combine_pack().
+					 */
+					to_quiver = current;
+					gear_insert_end(p, object_split(current,
+						current->number - nsplit));
+					++n_stack_split;
+				} else {
+					to_quiver = NULL;
 				}
 			}
+
+			if (to_quiver) {
+				p->upkeep->quiver[prefslot] = to_quiver;
+				p->upkeep->quiver_cnt += to_quiver->number * mult;
+
+				/* That part of the gear has been dealt with. */
+				assigned[j] = true;
+			}
 		}
 	}
 
-	/* Now fill the rest of the slots in order */
-	for (i = 0; i < z_info->quiver_size; i++) {
-		struct object *current, *first = NULL;
-		int j;
+	/* Now fill the rest of the slots in order. */
+	for (i = 0; i < z_info->quiver_size; ++i) {
+		struct object *first = NULL;
+		int jfirst = -1;
 
-		/* If the slot is full, move on */
-		if (upkeep->quiver[i]) continue;
+		/* If the slot is full, move on. */
+		if (p->upkeep->quiver[i]) continue;
 
-		/* Find the first quiver object not yet allocated */
-		for (current = gear; current; current = current->next) {
-			bool already = false;
+		/* Find the quiver object that should go there. */
+		j = 0;
+		current = p->gear;
+		while (1) {
+			if (!current) break;
+			assert(j < n_max);
 
-			/* Ignore non-ammo */
-			if (!tval_is_ammo(current)) continue;
-
-			/* Ignore stuff already quivered */
-			for (j = 0; j < z_info->quiver_size; j++)
-				if (upkeep->quiver[j] == current)
-					already = true;
-			if (already) continue;
-
-			/* Choose the first in order */
-			if (earlier_object(first, current, false)) {
-				first = current;
+			/*
+			 * Only try to assign if not assigned, ammo, and,
+			 * if necessary to split, have room for the split
+			 * stacks.
+			 */
+			if (!assigned[j] && tval_is_ammo(current)
+					&& (current->number
+					<= z_info->quiver_slot_size
+					|| (z_info->quiver_slot_size > 0
+					&& n_stack_split
+					<= n_pack_remaining))) {
+				/* Choose the first in order. */
+				if (earlier_object(first, current, false)) {
+					first = current;
+					jfirst = j;
+				}
 			}
+
+			current = current->next;
+			++j;
 		}
 
-		/* Stop looking if there's nothing left */
+		/* Stop looking if there's nothing left in the gear. */
 		if (!first) break;
 
-		/* If we have an item, slot it */
-		upkeep->quiver[i] = first;
-		upkeep->quiver_cnt += first->number;
+		/* Put the item in the slot, splitting (if needed) to fit. */
+		if (first->number > z_info->quiver_slot_size) {
+			assert(z_info->quiver_slot_size > 0
+				&& n_stack_split <= n_pack_remaining);
+			/* As above, split off the portion going to the pack. */
+			gear_insert_end(p, object_split(first,
+				first->number - z_info->quiver_slot_size));
+		}
+		p->upkeep->quiver[i] = first;
+		p->upkeep->quiver_cnt += first->number;
 
-		/* In the quiver counts as worn */
-		object_learn_on_wield(player, first);
+		/* That part of the gear has been dealt with. */
+		assigned[jfirst] = true;
 	}
 
 	/* Note reordering */
-	if (character_dungeon)
-		for (i = 0; i < z_info->quiver_size; i++)
-			if (old_quiver[i] && (upkeep->quiver[i] != old_quiver[i])) {
+	if (character_dungeon) {
+		for (i = 0; i < z_info->quiver_size; i++) {
+			if (old_quiver[i] && p->upkeep->quiver[i] != old_quiver[i]) {
 				msg("You re-arrange your quiver.");
 				break;
 			}
+		}
+	}
 
 	/* Copy the current pack */
-	for (i = 0; i < z_info->pack_size; i++)
-		old_pack[i] = upkeep->inven[i];
+	for (i = 0; i < z_info->pack_size; i++) {
+		old_pack[i] = p->upkeep->inven[i];
+	}
 
 	/* Prepare to fill the inventory */
-	upkeep->inven_cnt = 0;
+	p->upkeep->inven_cnt = 0;
 
 	for (i = 0; i <= z_info->pack_size; i++) {
-		struct object *current, *first = NULL;
-		for (current = gear; current; current = current->next) {
-			bool possible = true;
-			int j;
+		struct object *first = NULL;
+		int jfirst = -1;
 
-			/* Skip equipment */
-			if (object_is_equipped(body, current))
-				possible = false;
+		/* Find the object that should go there. */
+		j = 0;
+		current = p->gear;
+		while (1) {
+			if (!current) break;
+			assert(j < n_max);
 
-			/* Skip quivered objects */
-			for (j = 0; j < z_info->quiver_size; j++)
-				if (upkeep->quiver[j] == current)
-					possible = false;
-
-			/* Skip objects already allocated to the inventory */
-			for (j = 0; j < upkeep->inven_cnt; j++)
-				if (upkeep->inven[j] == current)
-					possible = false;
-
-			/* If still possible, choose the first in order */
-			if (!possible)
-				continue;
-			else if (earlier_object(first, current, false)) {
-				first = current;
+			/* Consider it if it hasn't already been handled. */
+			if (!assigned[j]) {
+				/* Choose the first in order. */
+				if (earlier_object(first, current, false)) {
+					first = current;
+					jfirst = j;
+				}
 			}
+
+			current = current->next;
+			++j;
 		}
 
 		/* Allocate */
-		upkeep->inven[i] = first;
-		if (first)
-			upkeep->inven_cnt++;
+		p->upkeep->inven[i] = first;
+		if (first) {
+			++p->upkeep->inven_cnt;
+			assigned[jfirst] = true;
+		}
 	}
 
 	/* Note reordering */
-	if (character_dungeon && (upkeep->inven_cnt == old_inven_cnt))
-		for (i = 0; i < z_info->pack_size; i++)
-			if (old_pack[i] && (upkeep->inven[i] != old_pack[i]) &&
-				!object_is_equipped(body, old_pack[i])) {
+	if (character_dungeon && p->upkeep->inven_cnt == old_inven_cnt) {
+		for (i = 0; i < z_info->pack_size; i++) {
+			if (old_pack[i] && p->upkeep->inven[i] != old_pack[i]
+					 && !object_is_equipped(p->body, old_pack[i])) {
 				msg("You re-arrange your pack.");
 				break;
 			}
+		}
+	}
 
-	mem_free(old_quiver);
+	mem_free(assigned);
 	mem_free(old_pack);
-}
-
-static void update_inventory(struct player *p)
-{
-	calc_inventory(p->upkeep, p->gear, p->body);
+	mem_free(old_quiver);
 }
 
 /**
@@ -1214,7 +1273,7 @@ static void calc_spells(struct player *p)
 
 	const struct class_spell *spell;
 
-	s16b old_spells;
+	int16_t old_spells;
 
 	/* Hack -- must be literate */
 	if (!p->class->magic.total_spells) return;
@@ -1260,7 +1319,7 @@ static void calc_spells(struct player *p)
 		if (j >= 99) continue;
 
 		/* Get the spell */
-		spell = spell_by_index(j);
+		spell = spell_by_index(p, j);
 
 		/* Skip spells we are allowed to know */
 		if (spell->slevel <= p->lev) continue;
@@ -1294,7 +1353,7 @@ static void calc_spells(struct player *p)
 		if (j >= 99) continue;
 
 		/* Get the spell */
-		spell = spell_by_index(j);
+		spell = spell_by_index(p, j);
 
 		/* Forget it (if learned) */
 		if (p->spell_flags[j] & PY_SPELL_LEARNED) {
@@ -1325,7 +1384,7 @@ static void calc_spells(struct player *p)
 		if (j >= 99) break;
 
 		/* Get the spell */
-		spell = spell_by_index(j);
+		spell = spell_by_index(p, j);
 
 		/* Skip spells we cannot remember */
 		if (spell->slevel > p->lev) continue;
@@ -1353,7 +1412,7 @@ static void calc_spells(struct player *p)
 	/* Count spells that can be learned */
 	for (j = 0; j < num_total; j++) {
 		/* Get the spell */
-		spell = spell_by_index(j);
+		spell = spell_by_index(p, j);
 
 		/* Skip spells we cannot remember or don't exist */
 		if (!spell) continue;
@@ -1449,11 +1508,11 @@ static void calc_mana(struct player *p, struct player_state *state, bool update)
 		struct object *obj_local = slot_object(p, i);
 
 		/* Ignore non-armor */
-		if (slot_type_is(i, EQUIP_WEAPON)) continue;
-		if (slot_type_is(i, EQUIP_BOW)) continue;
-		if (slot_type_is(i, EQUIP_RING)) continue;
-		if (slot_type_is(i, EQUIP_AMULET)) continue;
-		if (slot_type_is(i, EQUIP_LIGHT)) continue;
+		if (slot_type_is(p, i, EQUIP_WEAPON)) continue;
+		if (slot_type_is(p, i, EQUIP_BOW)) continue;
+		if (slot_type_is(p, i, EQUIP_RING)) continue;
+		if (slot_type_is(p, i, EQUIP_AMULET)) continue;
+		if (slot_type_is(p, i, EQUIP_LIGHT)) continue;
 
 		/* Add weight */
 		if (obj_local)
@@ -1681,6 +1740,32 @@ int weight_remaining(struct player *p)
 }
 
 
+/**
+ * Adjust a value by a relative factor of the absolute value.  Mimics the
+ * inline calculations of value = (value * (den + num)) / num when value is
+ * positive.
+ * \param v Is a pointer to the value to adjust.
+ * \param num Is the numerator of the relative factor.  Use a negative value
+ * for a decrease in the value, and a positive value for an increase.
+ * \param den Is the denominator for the relative factor.  Must be positive.
+ * \param minv Is the minimum absolute value of v to use when computing the
+ * adjustment; use zero for this to get a pure relative adjustment.  Must be
+ * be non-negative.
+ */
+static void adjust_skill_scale(int *v, int num, int den, int minv)
+{
+	if (num >= 0) {
+		*v += (MAX(minv, ABS(*v)) * num) / den;
+	} else {
+		/*
+		 * To mimic what (value * (den * num)) / num would give for
+		 * positive value, need to round up the adjustment.
+		 */
+		*v -= (MAX(minv, ABS(*v)) * -num + den - 1) / den;
+	}
+}
+
+
 /**
  * Calculate the effect of a shapechange on player state
  */
@@ -1881,7 +1966,7 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 			for (j = 0; j < ELEM_MAX; j++) {
 				if (!known_only || obj->known->el_info[j].res_level) {
 					if (obj->el_info[j].res_level == -1)
-						vuln[i] = true;
+						vuln[j] = true;
 
 					/* OK because res_level hasn't included vulnerability yet */
 					if (obj->el_info[j].res_level > state->el_info[j].res_level)
@@ -1893,7 +1978,8 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 			state->ac += obj->ac;
 			if (!known_only || obj->known->to_a)
 				state->to_a += obj->to_a;
-			if (!slot_type_is(i, EQUIP_WEAPON) && !slot_type_is(i, EQUIP_BOW)) {
+			if (!slot_type_is(p, i, EQUIP_WEAPON)
+					&& !slot_type_is(p, i, EQUIP_BOW)) {
 				if (!known_only || obj->known->to_h) {
 					state->to_h += obj->to_h;
 				}
@@ -1947,11 +2033,6 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 		state->el_info[ELEM_HOLY_ORB].res_level = -1;
 	}
 
-	/* Combat Regeneration */
-	if (player_has(p, PF_COMBAT_REGEN) && character_dungeon) {
-		of_on(state->flags, OF_IMPAIR_HP);
-	}
-
 	/* Calculate the various stat values */
 	for (i = 0; i < STAT_MAX; i++) {
 		int add, use, ind;
@@ -2008,18 +2089,18 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 			state->to_h -= lack;
 			state->to_d -= lack;
 			if ((lack > 10) && (lack <= 15)) {
-				state->skills[SKILL_DEVICE] *= 9;
-				state->skills[SKILL_DEVICE] /= 10;
+				adjust_skill_scale(&state->skills[SKILL_DEVICE],
+					-1, 10, 0);
 			} else if ((lack > 15) && (lack <= 18)) {
-				state->skills[SKILL_DEVICE] *= 8;
-				state->skills[SKILL_DEVICE] /= 10;
+				adjust_skill_scale(&state->skills[SKILL_DEVICE],
+					-1, 5, 0);
 				state->skills[SKILL_DISARM_PHYS] *= 9;
 				state->skills[SKILL_DISARM_PHYS] /= 10;
 				state->skills[SKILL_DISARM_MAGIC] *= 9;
 				state->skills[SKILL_DISARM_MAGIC] /= 10;
 			} else if (lack > 18) {
-				state->skills[SKILL_DEVICE] *= 7;
-				state->skills[SKILL_DEVICE] /= 10;
+				adjust_skill_scale(&state->skills[SKILL_DEVICE],
+					-3, 10, 0);
 				state->skills[SKILL_DISARM_PHYS] *= 8;
 				state->skills[SKILL_DISARM_PHYS] /= 10;
 				state->skills[SKILL_DISARM_MAGIC] *= 8;
@@ -2038,14 +2119,14 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 	if (player_timed_grade_eq(p, TMD_STUN, "Heavy Stun")) {
 		state->to_h -= 20;
 		state->to_d -= 20;
-		state->skills[SKILL_DEVICE] = state->skills[SKILL_DEVICE] * 8 / 10;
+		adjust_skill_scale(&state->skills[SKILL_DEVICE], -1, 5, 0);
 		if (update) {
 			p->timed[TMD_FASTCAST] = 0;
 		}
 	} else if (player_timed_grade_eq(p, TMD_STUN, "Stun")) {
 		state->to_h -= 5;
 		state->to_d -= 5;
-		state->skills[SKILL_DEVICE] = state->skills[SKILL_DEVICE] * 9 / 10;
+		adjust_skill_scale(&state->skills[SKILL_DEVICE], -1, 10, 0);
 		if (update) {
 			p->timed[TMD_FASTCAST] = 0;
 		}
@@ -2056,7 +2137,7 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 	if (p->timed[TMD_BLESSED]) {
 		state->to_a += 5;
 		state->to_h += 10;
-		state->skills[SKILL_DEVICE] = state->skills[SKILL_DEVICE] * 105 / 100;
+		adjust_skill_scale(&state->skills[SKILL_DEVICE], 1, 20, 0);
 	}
 	if (p->timed[TMD_SHIELD]) {
 		state->to_a += 50;
@@ -2067,12 +2148,12 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 	}
 	if (p->timed[TMD_HERO]) {
 		state->to_h += 12;
-		state->skills[SKILL_DEVICE] = state->skills[SKILL_DEVICE] * 105 / 100;
+		adjust_skill_scale(&state->skills[SKILL_DEVICE], 1, 20, 0);
 	}
 	if (p->timed[TMD_SHERO]) {
 		state->skills[SKILL_TO_HIT_MELEE] += 75;
 		state->to_a -= 10;
-		state->skills[SKILL_DEVICE] = state->skills[SKILL_DEVICE] * 9 / 10;
+		adjust_skill_scale(&state->skills[SKILL_DEVICE], -1, 10, 0);
 	}
 	if (p->timed[TMD_FAST] || p->timed[TMD_SPRINT]) {
 		state->speed += 10;
@@ -2102,16 +2183,16 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 			state->el_info[ELEM_POIS].res_level++;
 	}
 	if (p->timed[TMD_CONFUSED]) {
-		state->skills[SKILL_DEVICE] = state->skills[SKILL_DEVICE] * 75 / 100;
+		adjust_skill_scale(&state->skills[SKILL_DEVICE], -1, 4, 0);
 	}
 	if (p->timed[TMD_AMNESIA]) {
-		state->skills[SKILL_DEVICE] = state->skills[SKILL_DEVICE] * 8 / 10;
+		adjust_skill_scale(&state->skills[SKILL_DEVICE], -1, 5, 0);
 	}
 	if (p->timed[TMD_POISONED]) {
-		state->skills[SKILL_DEVICE] = state->skills[SKILL_DEVICE] * 95 / 100;
+		adjust_skill_scale(&state->skills[SKILL_DEVICE], -1, 20, 0);
 	}
 	if (p->timed[TMD_IMAGE]) {
-		state->skills[SKILL_DEVICE] = state->skills[SKILL_DEVICE] * 8 / 10;
+		adjust_skill_scale(&state->skills[SKILL_DEVICE], -1, 5, 0);
 	}
 	if (p->timed[TMD_BLOODLUST]) {
 		state->to_d += p->timed[TMD_BLOODLUST] / 2;
@@ -2125,7 +2206,7 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 	if (of_has(state->flags, OF_AFRAID)) {
 		state->to_h -= 20;
 		state->to_a += 8;
-		state->skills[SKILL_DEVICE] = state->skills[SKILL_DEVICE] * 95 / 100;
+		adjust_skill_scale(&state->skills[SKILL_DEVICE], -1, 20, 0);
 	}
 
 	/* Analyze weight */
@@ -2185,7 +2266,7 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 		if (!state->heavy_shoot) {
 			state->num_shots += extra_shots;
 			state->ammo_mult += extra_might;
-			if (player_has(p, PF_FAST_SHOT) && (state->ammo_tval == TV_ARROW)) {
+			if (player_has(p, PF_FAST_SHOT)) {
 				state->num_shots += p->lev / 3;
 			}
 		}
@@ -2229,7 +2310,7 @@ void calc_bonuses(struct player *p, struct player_state *state, bool known_only,
 	}
 
 	/* Movement speed */
-	state->num_moves = 1 + extra_moves;
+	state->num_moves = extra_moves;
 
 	return;
 }
@@ -2307,6 +2388,11 @@ static void update_bonuses(struct player *p)
 		p->upkeep->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
 	}
 
+	/* Notice changes to the weight limit. */
+	if (weight_limit(&p->state) != weight_limit(&state)) {
+		p->upkeep->redraw |= (PR_INVEN);
+	}
+
 	/* Hack -- handle partial mode */
 	if (!p->upkeep->only_partial) {
 		/* Take note when "heavy bow" changes */
@@ -2440,13 +2526,13 @@ void notice_stuff(struct player *p)
 	/* Deal with ignore stuff */
 	if (p->upkeep->notice & PN_IGNORE) {
 		p->upkeep->notice &= ~(PN_IGNORE);
-		ignore_drop();
+		ignore_drop(p);
 	}
 
 	/* Combine the pack */
 	if (p->upkeep->notice & PN_COMBINE) {
 		p->upkeep->notice &= ~(PN_COMBINE);
-		combine_pack();
+		combine_pack(p);
 	}
 
 	/* Dump the monster messages */
@@ -2469,7 +2555,7 @@ void update_stuff(struct player *p)
 
 	if (p->upkeep->update & (PU_INVEN)) {
 		p->upkeep->update &= ~(PU_INVEN);
-		update_inventory(p);
+		calc_inventory(p);
 	}
 
 	if (p->upkeep->update & (PU_BONUS)) {
@@ -2532,7 +2618,7 @@ void update_stuff(struct player *p)
 
 struct flag_event_trigger
 {
-	u32b flag;
+	uint32_t flag;
 	game_event_type event;
 };
 
@@ -2577,7 +2663,7 @@ static const struct flag_event_trigger redraw_events[] =
 void redraw_stuff(struct player *p)
 {
 	size_t i;
-	u32b redraw = p->upkeep->redraw;
+	uint32_t redraw = p->upkeep->redraw;
 
 	/* Redraw stuff */
 	if (!redraw) return;
