@@ -25,31 +25,34 @@
 /**
  * Calculates the total number of points earned (wow - NRM)
  */
-static long total_points(void)
+static long total_points(const struct player *p)
 {
-	return (player->max_exp + (100 * player->max_depth));
+	return p->max_exp + 100 * p->max_depth;
 }
 
 
 /**
  * Read in a highscore file.
  */
-size_t highscore_read(high_score scores[], size_t sz)
+size_t highscore_read(struct high_score scores[], size_t sz)
 {
 	char fname[1024];
 	ang_file *scorefile;
 	size_t i;
 
 	/* Wipe current scores */
-	memset(scores, 0, sz * sizeof(high_score));
+	memset(scores, 0, sz * sizeof(struct high_score));
 
 	path_build(fname, sizeof(fname), ANGBAND_DIR_SCORES, "scores.raw");
+	safe_setuid_grab();
 	scorefile = file_open(fname, MODE_READ, FTYPE_TEXT);
+	safe_setuid_drop();
 
-	if (!scorefile) return true;
+	if (!scorefile) return 0;
 
 	for (i = 0; i < sz; i++)
-		if (file_read(scorefile, (char *)&scores[i], sizeof(high_score)) <= 0)
+		if (file_read(scorefile, (char *)&scores[i],
+					  sizeof(struct high_score)) <= 0)
 			break;
 
 	file_close(scorefile);
@@ -62,8 +65,8 @@ size_t highscore_read(high_score scores[], size_t sz)
  * Just determine where a new score *would* be placed
  * Return the location (0 is best) or -1 on failure
  */
-size_t highscore_where(const high_score *entry, const high_score scores[],
-					   size_t sz)
+size_t highscore_where(const struct high_score *entry,
+					   const struct high_score scores[], size_t sz)
 {
 	size_t i;
 
@@ -71,6 +74,14 @@ size_t highscore_where(const high_score *entry, const high_score scores[],
 	for (i = 0; i < sz; i++) {
 		long entry_pts = strtoul(entry->pts, NULL, 0);
 		long score_pts = strtoul(scores[i].pts, NULL, 0);
+		bool entry_winner = streq(entry->how, "Ripe Old Age");
+		bool score_winner = streq(scores[i].how, "Ripe Old Age");
+
+		if (entry_winner && !score_winner)
+			return i;
+
+		if (!entry_winner && score_winner)
+			continue;
 
 		if (entry_pts >= score_pts)
 			return i;
@@ -83,18 +94,22 @@ size_t highscore_where(const high_score *entry, const high_score scores[],
 	return sz - 1;
 }
 
-size_t highscore_add(const high_score *entry, high_score scores[], size_t sz)
+/**
+ * Place an entry into a high score array
+ */
+size_t highscore_add(const struct high_score *entry, struct high_score scores[],
+					 size_t sz)
 {
 	size_t slot = highscore_where(entry, scores, sz);
 
 	memmove(&scores[slot + 1], &scores[slot],
-			sizeof(high_score) * (sz - 1 - slot));
-	memcpy(&scores[slot], entry, sizeof(high_score));
+			sizeof(struct high_score) * (sz - 1 - slot));
+	memcpy(&scores[slot], entry, sizeof(struct high_score));
 
 	return slot;
 }
 
-static size_t highscore_count(const high_score scores[], size_t sz)
+static size_t highscore_count(const struct high_score scores[], size_t sz)
 {
 	size_t i;
 	for (i = 0; i < sz; i++)
@@ -107,9 +122,8 @@ static size_t highscore_count(const high_score scores[], size_t sz)
 
 /**
  * Actually place an entry into the high score file
- * Return the location (0 is best) or -1 on "failure"
  */
-static void highscore_write(const high_score scores[], size_t sz)
+static void highscore_write(const struct high_score scores[], size_t sz)
 {
 	size_t n;
 
@@ -120,6 +134,7 @@ static void highscore_write(const high_score scores[], size_t sz)
 	char cur_name[1024];
 	char new_name[1024];
 	char lok_name[1024];
+	bool exists;
 
 	path_build(old_name, sizeof(old_name), ANGBAND_DIR_SCORES, "scores.old");
 	path_build(cur_name, sizeof(cur_name), ANGBAND_DIR_SCORES, "scores.raw");
@@ -132,7 +147,10 @@ static void highscore_write(const high_score scores[], size_t sz)
 
 
 	/* Lock scores */
-	if (file_exists(lok_name)) {
+	safe_setuid_grab();
+	exists = file_exists(lok_name);
+	safe_setuid_drop();
+	if (exists) {
 		msg("Lock file in place for scorefile; not writing.");
 		return;
 	}
@@ -157,11 +175,13 @@ static void highscore_write(const high_score scores[], size_t sz)
 		msg("Failed to open new scorefile for writing.");
 
 		file_close(lok);
+		safe_setuid_grab();
 		file_delete(lok_name);
+		safe_setuid_drop();
 		return;
 	}
 
-	file_write(scorefile, (const char *)scores, sizeof(high_score)*n);
+	file_write(scorefile, (const char *)scores, sizeof(struct high_score)*n);
 	file_close(scorefile);
 
 	/* Now move things around */
@@ -185,18 +205,33 @@ static void highscore_write(const high_score scores[], size_t sz)
 
 
 
-void build_score(high_score *entry, const char *died_from, time_t *death_time)
+/**
+ * Fill in a score record for the given player.
+ *
+ * \param entry points to the record to fill in.
+ * \param p is the player whose score should be recorded.
+ * \param died_from is the reason for death.  In typical use, that will be
+ * p->died_from, but when the player isn't dead yet, the caller may want to
+ * use something else:  "nobody (yet!)" is traditional.
+ * \param death_time points to the time at which the player died.  May be NULL
+ * when the player isn't dead.
+ *
+ * Bug:  takes a player argument, but still accesses a bit of global state,
+ * player_uid, referring to the player
+ */
+void build_score(struct high_score *entry, const struct player *p,
+		const char *died_from, const time_t *death_time)
 {
-	memset(entry, 0, sizeof(high_score));
+	memset(entry, 0, sizeof(struct high_score));
 
 	/* Save the version */
 	strnfmt(entry->what, sizeof(entry->what), "%s", buildid);
 
 	/* Calculate and save the points */
-	strnfmt(entry->pts, sizeof(entry->pts), "%9u", total_points());
+	strnfmt(entry->pts, sizeof(entry->pts), "%9u", total_points(p));
 
 	/* Save the current gold */
-	strnfmt(entry->gold, sizeof(entry->gold), "%9u", player->au);
+	strnfmt(entry->gold, sizeof(entry->gold), "%9u", p->au);
 
 	/* Save the current turn */
 	strnfmt(entry->turns, sizeof(entry->turns), "%9u", turn);
@@ -209,18 +244,18 @@ void build_score(high_score *entry, const char *died_from, time_t *death_time)
 		my_strcpy(entry->day, "TODAY", sizeof(entry->day));
 
 	/* Save the player name (15 chars) */
-	strnfmt(entry->who, sizeof(entry->who), "%-.15s", player->full_name);
+	strnfmt(entry->who, sizeof(entry->who), "%-.15s", p->full_name);
 
 	/* Save the player info XXX XXX XXX */
 	strnfmt(entry->uid, sizeof(entry->uid), "%7u", player_uid);
-	strnfmt(entry->p_r, sizeof(entry->p_r), "%2d", player->race->ridx);
-	strnfmt(entry->p_c, sizeof(entry->p_c), "%2d", player->class->cidx);
+	strnfmt(entry->p_r, sizeof(entry->p_r), "%2d", p->race->ridx);
+	strnfmt(entry->p_c, sizeof(entry->p_c), "%2d", p->class->cidx);
 
 	/* Save the level and such */
-	strnfmt(entry->cur_lev, sizeof(entry->cur_lev), "%3d", player->lev);
-	strnfmt(entry->cur_dun, sizeof(entry->cur_dun), "%3d", player->depth);
-	strnfmt(entry->max_lev, sizeof(entry->max_lev), "%3d", player->max_lev);
-	strnfmt(entry->max_dun, sizeof(entry->max_dun), "%3d", player->max_depth);
+	strnfmt(entry->cur_lev, sizeof(entry->cur_lev), "%3d", p->lev);
+	strnfmt(entry->cur_dun, sizeof(entry->cur_dun), "%3d", p->depth);
+	strnfmt(entry->max_lev, sizeof(entry->max_lev), "%3d", p->max_lev);
+	strnfmt(entry->max_dun, sizeof(entry->max_dun), "%3d", p->max_depth);
 
 	/* No cause of death */
 	my_strcpy(entry->how, died_from, sizeof(entry->how));
@@ -229,11 +264,14 @@ void build_score(high_score *entry, const char *died_from, time_t *death_time)
 
 
 /**
- * Enters a players name on a hi-score table, if "legal".
+ * Enter a player's name on a hi-score table, if "legal".
  *
+ * \param p is the player to enter
+ * \param death_time points to the time at which the player died; may be NULL
+ * for a player that's not dead yet
  * Assumes "signals_ignore_tstp()" has been called.
  */
-void enter_score(time_t *death_time)
+void enter_score(const struct player *p, const time_t *death_time)
 {
 	int j;
 
@@ -241,7 +279,7 @@ void enter_score(time_t *death_time)
 	for (j = 0; j < OPT_MAX; ++j) {
 		if (option_type(j) != OP_SCORE)
 			continue;
-		if (!player->opts.opt[j])
+		if (!p->opts.opt[j])
 			continue;
 
 		msg("Score not registered for cheaters.");
@@ -250,21 +288,20 @@ void enter_score(time_t *death_time)
 	}
 
 	/* Add a new entry, if allowed */
-	if (player->noscore & (NOSCORE_WIZARD | NOSCORE_DEBUG)) {
+	if (p->noscore & (NOSCORE_WIZARD | NOSCORE_DEBUG)) {
 		msg("Score not registered for wizards.");
 		event_signal(EVENT_MESSAGE_FLUSH);
-	} else if (!player->total_winner &&
-			   streq(player->died_from, "Interrupting")) {
+	} else if (!p->total_winner && streq(p->died_from, "Interrupting")) {
 		msg("Score not registered due to interruption.");
 		event_signal(EVENT_MESSAGE_FLUSH);
-	} else if (!player->total_winner && streq(player->died_from, "Quitting")) {
+	} else if (!p->total_winner && streq(p->died_from, "Quitting")) {
 		msg("Score not registered due to quitting.");
 		event_signal(EVENT_MESSAGE_FLUSH);
 	} else {
-		high_score entry;
-		high_score scores[MAX_HISCORES];
+		struct high_score entry;
+		struct high_score scores[MAX_HISCORES];
 
-		build_score(&entry, player->died_from, death_time);
+		build_score(&entry, p, p->died_from, death_time);
 
 		highscore_read(scores, N_ELEMENTS(scores));
 		highscore_add(&entry, scores, N_ELEMENTS(scores));
