@@ -65,7 +65,7 @@ struct hint *hints;
 
 static const char *obj_flags[] = {
 	"NONE",
-	#define OF(a) #a,
+	#define OF(a, b) #a,
 	#include "list-object-flags.h"
 	#undef OF
 	NULL
@@ -96,7 +96,7 @@ static struct store *store_new(int idx) {
 /**
  * Get rid of stores at cleanup. Gets rid of everything.
  */
-void cleanup_stores(void)
+static void cleanup_stores(void)
 {
 	struct owner *o, *o_next;
 	struct object_buy *buy, *buy_next;
@@ -111,8 +111,8 @@ void cleanup_stores(void)
 		struct store *store = &stores[i];
 
 		/* Free the store inventory */
-		object_pile_free(store->stock_k);
-		object_pile_free(store->stock);
+		object_pile_free(NULL, NULL, store->stock_k);
+		object_pile_free(NULL, NULL, store->stock);
 		mem_free(store->always_table);
 		mem_free(store->normal_table);
 
@@ -372,7 +372,9 @@ void store_reset(void) {
 		s = &stores[i];
 		s->stock_num = 0;
 		store_shuffle(s);
-		object_pile_free(s->stock);
+		object_pile_free(NULL, NULL, s->stock_k);
+		object_pile_free(NULL, NULL, s->stock);
+		s->stock_k = NULL;
 		s->stock = NULL;
 		if (i == STORE_HOME)
 			continue;
@@ -424,7 +426,18 @@ static bool store_can_carry(struct store *store, struct object_kind *kind) {
 	return store_is_staple(store, kind);
 }
 
-
+/**
+ * Check if an object is such that selling it should reduce the stock.
+ */
+static bool store_sale_should_reduce_stock(struct store *store,
+		struct object *obj)
+{
+	if (obj->artifact || obj->ego) return true;
+	if (tval_is_weapon(obj) && (obj->to_h || obj->to_d))
+		return true;
+	if (tval_is_armor(obj) && obj->to_a) return true;
+	return !store_is_staple(store, obj->kind);
+}
 
 
 /**
@@ -562,7 +575,7 @@ static bool store_will_buy(struct store *store, const struct object *obj)
 
 		/* OK if the object is known to have the flag */
 		if (of_has(obj->flags, buy->flag) &&
-			object_flag_is_known(obj, buy->flag))
+			object_flag_is_known(player, obj, buy->flag))
 			return true;
 	}
 
@@ -830,7 +843,7 @@ static void store_object_absorb(struct object *old, struct object *new)
 	object_origin_combine(old, new);
 
 	/* Fully absorbed */
-	object_delete(&new);
+	object_delete(NULL, NULL, &new);
 }
 
 
@@ -852,14 +865,14 @@ bool store_check_num(struct store *store, const struct object *obj)
 	if (store->sidx == STORE_HOME) {
 		for (stock_obj = store->stock; stock_obj; stock_obj = stock_obj->next) {
 			/* Can the new object be combined with the old one? */
-			if (object_similar(stock_obj, obj, OSTACK_PACK))
+			if (object_mergeable(stock_obj, obj, OSTACK_PACK))
 				return true;
 		}
 	} else {
 		/* Normal stores do special stuff */
 		for (stock_obj = store->stock; stock_obj; stock_obj = stock_obj->next) {
 			/* Can the new object be combined with the old one? */
-			if (object_similar(stock_obj, obj, OSTACK_STORE))
+			if (object_mergeable(stock_obj, obj, OSTACK_STORE))
 				return true;
 		}
 	}
@@ -883,7 +896,7 @@ void home_carry(struct object *obj)
 	/* Check each existing object (try to combine) */
 	for (temp_obj = store->stock; temp_obj; temp_obj = temp_obj->next) {
 		/* The home acts just like the player */
-		if (object_similar(temp_obj, obj, OSTACK_PACK)) {
+		if (object_mergeable(temp_obj, obj, OSTACK_PACK)) {
 			/* Save the new number of items */
 			object_absorb(temp_obj, obj);
 			return;
@@ -915,7 +928,7 @@ void home_carry(struct object *obj)
 struct object *store_carry(struct store *store, struct object *obj)
 {
 	unsigned int i;
-	u32b value;
+	uint32_t value;
 	struct object *temp_obj, *known_obj = obj->known;
 
 	struct object_kind *kind = obj->kind;
@@ -964,7 +977,7 @@ struct object *store_carry(struct store *store, struct object *obj)
 
 	for (temp_obj = store->stock; temp_obj; temp_obj = temp_obj->next) {
 		/* Can the existing items be incremented? */
-		if (object_similar(temp_obj, obj, OSTACK_STORE)) {
+		if (object_mergeable(temp_obj, obj, OSTACK_STORE)) {
 			/* Absorb (some of) the object */
 			store_object_absorb(temp_obj->known, known_obj);
 			obj->known = NULL;
@@ -988,7 +1001,7 @@ struct object *store_carry(struct store *store, struct object *obj)
 }
 
 
-void store_delete(struct store *s, struct object *obj, int amt)
+static void store_delete(struct store *s, struct object *obj, int amt)
 {
 	struct object *known_obj = obj->known;
 
@@ -997,9 +1010,9 @@ void store_delete(struct store *s, struct object *obj, int amt)
 		known_obj->number -= amt;
 	} else {
 		pile_excise(&s->stock, obj);
-		object_delete(&obj);
+		object_delete(NULL, NULL, &obj);
 		pile_excise(&s->stock_k, known_obj);
-		object_delete(&known_obj);
+		object_delete(NULL, NULL, &known_obj);
 		assert(s->stock_num);
 		s->stock_num--;
 	}
@@ -1007,9 +1020,11 @@ void store_delete(struct store *s, struct object *obj, int amt)
 
 
 /**
- * Find a given object kind in the store.
+ * Find a given object kind in the store.  If fexclude is not NULL, exclude
+ * any object, o, for which (*fexclude)(s, o) is true.
  */
-static struct object *store_find_kind(struct store *s, struct object_kind *k) {
+static struct object *store_find_kind(struct store *s, struct object_kind *k,
+		bool (*fexclude)(struct store *, struct object *)) {
 	struct object *obj;
 
 	assert(s);
@@ -1017,8 +1032,8 @@ static struct object *store_find_kind(struct store *s, struct object_kind *k) {
 
 	/* Check if it's already in stock */
 	for (obj = s->stock; obj; obj = obj->next) {
-		if (obj->kind == k && !obj->ego)
-			return obj;
+		if (obj->kind == k && (fexclude == NULL ||
+			!((*fexclude)(s, obj)))) return obj;
 	}
 
 	return NULL;
@@ -1201,7 +1216,7 @@ static bool store_create_random(struct store *store)
 		/* Reject if item is 'damaged' (negative combat mods, curses) */
 		if ((tval_is_weapon(obj) && ((obj->to_h < 0) || (obj->to_d < 0)))
 			|| (tval_is_armor(obj) && (obj->to_a < 0)) || (obj->curses)) {
-			object_delete(&obj);
+			object_delete(NULL, NULL, &obj);
 			continue;
 		}
 
@@ -1213,24 +1228,24 @@ static bool store_create_random(struct store *store)
 
 		/* Know everything the player knows, no origin yet */
 		obj->known->notice |= OBJ_NOTICE_ASSESSED;
-		object_set_base_known(obj);
+		object_set_base_known(player, obj);
 		obj->known->notice |= OBJ_NOTICE_ASSESSED;
 		player_know_object(player, obj);
 		obj->origin = ORIGIN_NONE;
 
 		/* Black markets have expensive tastes */
 		if ((store->sidx == STORE_B_MARKET) && !black_market_ok(obj)) {
-			object_delete(&known_obj);
+			object_delete(NULL, NULL, &known_obj);
 			obj->known = NULL;
-			object_delete(&obj);
+			object_delete(NULL, NULL, &obj);
 			continue;
 		}
 
 		/* No "worthless" items */
 		if (object_value_real(obj, 1) < 1)  {
-			object_delete(&known_obj);
+			object_delete(NULL, NULL, &known_obj);
 			obj->known = NULL;
-			object_delete(&obj);
+			object_delete(NULL, NULL, &obj);
 			continue;
 		}
 
@@ -1239,9 +1254,9 @@ static bool store_create_random(struct store *store)
 
 		/* Attempt to carry the object */
 		if (!store_carry(store, obj)) {
-			object_delete(&known_obj);
+			object_delete(NULL, NULL, &known_obj);
 			obj->known = NULL;
-			object_delete(&obj);
+			object_delete(NULL, NULL, &obj);
 			continue;
 		}
 
@@ -1269,7 +1284,7 @@ static struct object *store_create_item(struct store *store,
 	/* Know everything the player knows, no origin yet */
 	obj->known = known_obj;
 	obj->known->notice |= OBJ_NOTICE_ASSESSED;
-	object_set_base_known(obj);
+	object_set_base_known(player, obj);
 	obj->known->notice |= OBJ_NOTICE_ASSESSED;
 	player_know_object(player, obj);
 	obj->origin = ORIGIN_NONE;
@@ -1354,7 +1369,8 @@ static void store_maint(struct store *s)
 		size_t i;
 		for (i = 0; i < s->always_num; i++) {
 			struct object_kind *kind = s->always_table[i];
-			struct object *obj = store_find_kind(s, kind);
+			struct object *obj = store_find_kind(s, kind,
+				store_sale_should_reduce_stock);
 
 			/* Create the item if it doesn't exist */
 			if (!obj)
@@ -1652,21 +1668,22 @@ void do_cmd_buy(struct command *cmd)
 	object_copy_amt(bought, obj, amt);
 
 	/* Ensure we have room */
-	if (bought->number > inven_carry_num(bought, false)) {
+	if (bought->number > inven_carry_num(player, bought)) {
 		msg("You cannot carry that many items.");
-		object_delete(&bought);
+		object_delete(NULL, NULL, &bought);
 		return;
 	}
 
 	/* Describe the object (fully) */
-	object_desc(o_name, sizeof(o_name), bought, ODESC_PREFIX | ODESC_FULL);
+	object_desc(o_name, sizeof(o_name), bought, ODESC_PREFIX | ODESC_FULL,
+		player);
 
 	/* Extract the price for the entire stack */
 	price = price_item(store, bought, false, bought->number);
 
 	if (price > player->au) {
 		msg("You cannot afford that purchase.");
-		object_delete(&bought);
+		object_delete(NULL, NULL, &bought);
 		return;
 	}
 
@@ -1680,7 +1697,8 @@ void do_cmd_buy(struct command *cmd)
 	player->upkeep->notice |= (PN_COMBINE | PN_IGNORE);
 
 	/* Describe the object (fully) again for the message */
-	object_desc(o_name, sizeof(o_name), bought, ODESC_PREFIX | ODESC_FULL);
+	object_desc(o_name, sizeof(o_name), bought, ODESC_PREFIX | ODESC_FULL,
+		player);
 
 	/* Message */
 	if (one_in_(3)) msgt(MSG_STORE5, "%s", ONE_OF(comment_accept));
@@ -1703,7 +1721,7 @@ void do_cmd_buy(struct command *cmd)
 	bought->known = known_obj;
 
 	/* Learn flavor, any effect and all the runes */
-	object_flavor_aware(bought);
+	object_flavor_aware(player, bought);
 	bought->known->effect = bought->effect;
 	while (!object_fully_known(bought)) {
 		object_learn_unknown_rune(player, bought);
@@ -1716,8 +1734,9 @@ void do_cmd_buy(struct command *cmd)
 	/* Handle stuff */
 	handle_stuff(player);
 
-	/* Remove the bought objects from the store if it's not a staple */
-	if (!store_is_staple(store, obj->kind)) {
+	/* Remove the bought objects from the store if it's not a readily
+	 * replaced staple item */
+	if (store_sale_should_reduce_stock(store, obj)) {
 		/* Reduce or remove the item */
 		store_delete(store, obj, amt);
 
@@ -1779,9 +1798,9 @@ void do_cmd_retrieve(struct command *cmd)
 	object_copy_amt(picked_item, obj, amt);
 
 	/* Ensure we have room */
-	if (picked_item->number > inven_carry_num(picked_item, false)) {
+	if (picked_item->number > inven_carry_num(player, picked_item)) {
 		msg("You cannot carry that many items.");
-		object_delete(&picked_item);
+		object_delete(NULL, NULL, &picked_item);
 		return;
 	}
 
@@ -1871,7 +1890,7 @@ void do_cmd_sell(struct command *cmd)
 	}
 
 	/* Get the label */
-	label = gear_to_label(obj);
+	label = gear_to_label(player, obj);
 
 	price = price_item(store, &dummy_item, true, amt);
 
@@ -1901,7 +1920,7 @@ void do_cmd_sell(struct command *cmd)
 	object_wipe(&dummy_item);
 
 	/* Know flavor of consumables */
-	object_flavor_aware(obj);
+	object_flavor_aware(player, obj);
 	obj->known->effect = obj->effect;
 	while (!object_fully_known(obj)) {
 		object_learn_unknown_rune(player, obj);
@@ -1909,13 +1928,14 @@ void do_cmd_sell(struct command *cmd)
 	}
 
 	/* Take a proper copy of the now known-about object. */
-	sold_item = gear_object_for_use(obj, amt, false, &none_left);
+	sold_item = gear_object_for_use(player, obj, amt, false, &none_left);
 
 	/* Get the "actual" value */
 	value = object_value_real(sold_item, amt);
 
 	/* Get the description all over again */
-	object_desc(o_name, sizeof(o_name), sold_item, ODESC_PREFIX | ODESC_FULL);
+	object_desc(o_name, sizeof(o_name), sold_item,
+		ODESC_PREFIX | ODESC_FULL, player);
 
 	/* Describe the result (in message buffer) */
 	if (OPT(player, birth_no_selling)) {
@@ -1929,7 +1949,7 @@ void do_cmd_sell(struct command *cmd)
 
 	/* Autoinscribe if we still have any */
 	if (!none_left)
-		apply_autoinscription(obj);
+		apply_autoinscription(player, obj);
 
 	/* Set ignore flag */
 	player->upkeep->notice |= PN_IGNORE;
@@ -1941,13 +1961,13 @@ void do_cmd_sell(struct command *cmd)
 	handle_stuff(player);
 
 	/* The store gets that (known) object */
-	if (! store_carry(store, sold_item)) {
+	if (!store_carry(store, sold_item)) {
 		/* The store rejected it; delete. */
 		if (sold_item->known) {
-			object_delete(&sold_item->known);
+			object_delete(NULL, NULL, &sold_item->known);
 			sold_item->known = NULL;
 		}
-		object_delete(&sold_item);
+		object_delete(NULL, NULL, &sold_item);
 	}
 
 	event_signal(EVENT_STORECHANGED);
@@ -2003,13 +2023,14 @@ void do_cmd_stash(struct command *cmd)
 	}
 
 	/* Get where the object is now */
-	label = gear_to_label(obj);
+	label = gear_to_label(player, obj);
 
 	/* Now get the real item */
-	dropped = gear_object_for_use(obj, amt, false, &none_left);
+	dropped = gear_object_for_use(player, obj, amt, false, &none_left);
 
 	/* Describe */
-	object_desc(o_name, sizeof(o_name), dropped, ODESC_PREFIX | ODESC_FULL);
+	object_desc(o_name, sizeof(o_name), dropped,
+		ODESC_PREFIX | ODESC_FULL, player);
 
 	/* Message */
 	msg("You drop %s (%c).", o_name, label);
