@@ -20,6 +20,7 @@
 #include "cave.h"
 #include "cmd-core.h"
 #include "game-input.h"
+#include "init.h"
 #include "mon-desc.h"
 #include "mon-util.h"
 #include "monster.h"
@@ -272,7 +273,7 @@ int cmp_distance(const void *a, const void *b)
  * Help select a location.  This function picks the closest from a set in 
  *(roughly) a given direction.
  */
-s16b target_pick(int y1, int x1, int dy, int dx, struct point_set *targets)
+int16_t target_pick(int y1, int x1, int dy, int dx, struct point_set *targets)
 {
 	int i, v;
 
@@ -327,13 +328,13 @@ bool target_accept(int y, int x)
 	struct object *obj;
 
 	/* Player grids are always interesting */
-	if (square(cave, grid).mon < 0) return true;
+	if (square(cave, grid)->mon < 0) return true;
 
 	/* Handle hallucination */
 	if (player->timed[TMD_IMAGE]) return false;
 
 	/* Obvious monsters */
-	if (square(cave, grid).mon > 0) {
+	if (square(cave, grid)->mon > 0) {
 		struct monster *mon = square_monster(cave, grid);
 		if (monster_is_obvious(mon)) {
 			return true;
@@ -346,7 +347,8 @@ bool target_accept(int y, int x)
 	/* Scan all objects in the grid */
 	for (obj = square_object(player->cave, grid); obj; obj = obj->next) {
 		/* Memorized object */
-		if ((obj->kind == unknown_item_kind) || !ignore_known_item_ok(obj)) {
+		if (obj->kind == unknown_item_kind
+				 || !ignore_known_item_ok(player, obj)) {
 			return true;
 		}
 	}
@@ -422,16 +424,30 @@ bool target_sighted(void)
 #define TS_INITIAL_SIZE	20
 
 /**
- * Return a target set of target_able monsters.
+ * Return a target set of interesting locations including monsters, objects,
+ * traps, and features.
+ *
+ * \mode If mode includes TARGET_KILL, only target_able monsters matching pred
+ *       are included
+ * \pred The monster predicate used to filter monsters (optional)
+ * \restrict_to_panel Restricts the interesting points to the current panel
  */
-struct point_set *target_get_monsters(int mode, monster_predicate pred)
+struct point_set *target_get_monsters(int mode, monster_predicate pred,
+		bool restrict_to_panel)
 {
 	int y, x;
 	int min_y, min_x, max_y, max_x;
 	struct point_set *targets = point_set_new(TS_INITIAL_SIZE);
 
-	/* Get the current panel */
-	get_panel(&min_y, &min_x, &max_y, &max_x);
+	if (restrict_to_panel) {
+		/* Get the current panel */
+		get_panel(&min_y, &min_x, &max_y, &max_x);
+	} else {
+		min_y = player->grid.y - z_info->max_range;
+		max_y = player->grid.y + z_info->max_range + 1;
+		min_x = player->grid.x - z_info->max_range;
+		max_x = player->grid.x + z_info->max_range + 1;
+	}
 
 	/* Scan for targets */
 	for (y = min_y; y < max_y; y++) {
@@ -482,7 +498,7 @@ bool target_set_closest(int mode, monster_predicate pred)
 	target_set_monster(NULL);
 
 	/* Get ready to do targetting */
-	targets = target_get_monsters(mode, pred);
+	targets = target_get_monsters(mode, pred, false);
 
 	/* If nothing was prepared, then return */
 	if (point_set_size(targets) < 1) {
@@ -502,7 +518,7 @@ bool target_set_closest(int mode, monster_predicate pred)
 	}
 
 	/* Target the monster */
-	monster_desc(m_name, sizeof(m_name), mon, MDESC_CAPITAL);
+	monster_desc(m_name, sizeof(m_name), mon, MDESC_CAPITAL | MDESC_COMMA);
 	if (!(mode & TARGET_QUIET))
 		msg("%s is targeted.", m_name);
 
