@@ -21,12 +21,24 @@ int teardown_tests(void *state) {
 	for (i = 0; i < PY_MAX_LEVEL / 5; i++) {
 		string_free((char *)c->title[i]);
 	}
+	while (c->start_items) {
+		struct start_item *si = c->start_items;
+
+		c->start_items = si->next;
+		mem_free(si->eopts);
+		mem_free(si);
+	}
+	/*
+	 * If tests are set up that lead to spell allocation, those will also
+	 * have to be freed here before releasing the books.
+	 */
+	mem_free(c->magic.books);
 	mem_free(c);
 	parser_destroy(state);
 	return 0;
 }
 
-int test_name0(void *state) {
+static int test_name0(void *state) {
 	enum parser_error r = parser_parse(state, "name:Ranger");
 	struct player_class *c;
 
@@ -37,7 +49,7 @@ int test_name0(void *state) {
 	ok;
 }
 
-int test_stats0(void *state) {
+static int test_stats0(void *state) {
 	enum parser_error r = parser_parse(state, "stats:3:-3:2:-2:1");
 	struct player_class *c;
 
@@ -52,7 +64,7 @@ int test_stats0(void *state) {
 	ok;
 }
 
-int test_skill_disarm0(void *state) {
+static int test_skill_disarm0(void *state) {
 	enum parser_error r = parser_parse(state, "skill-disarm-phys:30:8");
 	struct player_class *c;
 
@@ -64,7 +76,7 @@ int test_skill_disarm0(void *state) {
 	ok;
 }
 
-int test_skill_device0(void *state) {
+static int test_skill_device0(void *state) {
 	enum parser_error r = parser_parse(state, "skill-device:32:10");
 	struct player_class *c;
 
@@ -76,7 +88,7 @@ int test_skill_device0(void *state) {
 	ok;
 }
 
-int test_skill_save0(void *state) {
+static int test_skill_save0(void *state) {
 	enum parser_error r = parser_parse(state, "skill-save:28:10");
 	struct player_class *c;
 
@@ -88,7 +100,7 @@ int test_skill_save0(void *state) {
 	ok;
 }
 
-int test_skill_stealth0(void *state) {
+static int test_skill_stealth0(void *state) {
 	enum parser_error r = parser_parse(state, "skill-stealth:3:0");
 	struct player_class *c;
 
@@ -100,7 +112,7 @@ int test_skill_stealth0(void *state) {
 	ok;
 }
 
-int test_skill_search0(void *state) {
+static int test_skill_search0(void *state) {
 	enum parser_error r = parser_parse(state, "skill-search:24:0");
 	struct player_class *c;
 
@@ -112,7 +124,7 @@ int test_skill_search0(void *state) {
 	ok;
 }
 
-int test_skill_melee0(void *state) {
+static int test_skill_melee0(void *state) {
 	enum parser_error r = parser_parse(state, "skill-melee:56:30");
 	struct player_class *c;
 
@@ -124,7 +136,7 @@ int test_skill_melee0(void *state) {
 	ok;
 }
 
-int test_skill_shoot0(void *state) {
+static int test_skill_shoot0(void *state) {
 	enum parser_error r = parser_parse(state, "skill-shoot:72:45");
 	struct player_class *c;
 
@@ -136,7 +148,7 @@ int test_skill_shoot0(void *state) {
 	ok;
 }
 
-int test_skill_throw0(void *state) {
+static int test_skill_throw0(void *state) {
 	enum parser_error r = parser_parse(state, "skill-throw:72:45");
 	struct player_class *c;
 
@@ -148,7 +160,7 @@ int test_skill_throw0(void *state) {
 	ok;
 }
 
-int test_skill_dig0(void *state) {
+static int test_skill_dig0(void *state) {
 	enum parser_error r = parser_parse(state, "skill-dig:0:0");
 	struct player_class *c;
 
@@ -160,7 +172,7 @@ int test_skill_dig0(void *state) {
 	ok;
 }
 
-int test_hitdie0(void *state) {
+static int test_hitdie0(void *state) {
 	enum parser_error r = parser_parse(state, "hitdie:4");
 	struct player_class *c;
 
@@ -171,7 +183,7 @@ int test_hitdie0(void *state) {
 	ok;
 }
 
-int test_max_attacks0(void *state) {
+static int test_max_attacks0(void *state) {
 	enum parser_error r = parser_parse(state, "max-attacks:5");
 	struct player_class *c;
 
@@ -182,7 +194,7 @@ int test_max_attacks0(void *state) {
 	ok;
 }
 
-int test_min_weight0(void *state) {
+static int test_min_weight0(void *state) {
 	enum parser_error r = parser_parse(state, "min-weight:35");
 	struct player_class *c;
 
@@ -193,7 +205,7 @@ int test_min_weight0(void *state) {
 	ok;
 }
 
-int test_strength_multiplier0(void *state) {
+static int test_strength_multiplier0(void *state) {
 	enum parser_error r = parser_parse(state, "strength-multiplier:4");
 	struct player_class *c;
 
@@ -204,7 +216,7 @@ int test_strength_multiplier0(void *state) {
 	ok;
 }
 
-int test_title0(void *state) {
+static int test_title0(void *state) {
 	enum parser_error r0 = parser_parse(state, "title:Runner");
 	enum parser_error r1 = parser_parse(state, "title:Strider");
 	struct player_class *c;
@@ -218,9 +230,8 @@ int test_title0(void *state) {
 	ok;
 }
 
-/* Causes segfault: lookup_sval() requires z_info/k_info */
-int test_equip0(void *state) {
-	enum parser_error r = parser_parse(state, "E:magic book:2:2:5");
+static int test_equip0(void *state) {
+	enum parser_error r = parser_parse(state, "equip:magic book:2:2:5:none");
 	struct player_class *c;
 
 	eq(r, PARSE_ERROR_NONE);
@@ -230,10 +241,11 @@ int test_equip0(void *state) {
 	eq(c->start_items[0].sval, 2);
 	eq(c->start_items[0].min, 2);
 	eq(c->start_items[0].max, 5);
+	eq(c->start_items[0].eopts, NULL);
 	ok;
 }
 
-int test_flags0(void *state) {
+static int test_flags0(void *state) {
 	enum parser_error r = parser_parse(state, "player-flags:BLESS_WEAPON | CHOOSE_SPELLS");
 	struct player_class *c;
 
@@ -244,7 +256,7 @@ int test_flags0(void *state) {
 	ok;
 }
 
-int test_flags1(void *state) {
+static int test_flags1(void *state) {
 	enum parser_error r = parser_parse(state, "obj-flags:SEE_INVIS | IMPAIR_HP");
 	struct player_class *c;
 
@@ -255,7 +267,7 @@ int test_flags1(void *state) {
 	ok;
 }
 
-int test_magic0(void *state) {
+static int test_magic0(void *state) {
 	enum parser_error r = parser_parse(state, "magic:3:400:9");
 	struct player_class *c;
 
@@ -264,6 +276,7 @@ int test_magic0(void *state) {
 	require(c);
 	eq(c->magic.spell_first, 3);
 	eq(c->magic.spell_weight, 400);
+	noteq(c->magic.books, NULL);
 	ok;
 }
 
@@ -285,9 +298,9 @@ struct test tests[] = {
 	{ "min_weight0", test_min_weight0 },
 	{ "strength_multiplier0", test_strength_multiplier0 },
 	{ "title0", test_title0 },
-	/* { "equip0", test_equip0 }, */
+	{ "equip0", test_equip0 },
 	{ "flags0", test_flags0 },
 	{ "flags1", test_flags1 },
-	//{ "magic0", test_magic0 },
+	{ "magic0", test_magic0 },
 	{ NULL, NULL }
 };
