@@ -9,9 +9,10 @@
 #include "h-basic.h"
 #include "config.h"
 #include "init.h"
+#include "test-utils.h"
 #include "z-util.h"
 
-#ifdef SOUND_SDL
+#if defined(SOUND_SDL) || defined(SOUND_SDL2)
 #include "sound.h"
 #include "snd-sdl.h"
 
