@@ -20,9 +20,11 @@
 #include "cave.h"
 #include "effects.h"
 #include "init.h"
+#include "mon-attack.h"
 #include "mon-util.h"
 #include "obj-knowledge.h"
 #include "player-attack.h"
+#include "player-quest.h"
 #include "player-timed.h"
 #include "player-util.h"
 #include "trap.h"
@@ -139,7 +141,7 @@ static bool square_verify_trap(struct chunk *c, struct loc grid, int vis)
     /* No traps in this location. */
     if (!trap_exists) {
 		/* No traps */
-		sqinfo_off(square(c, grid).info, SQUARE_TRAP);
+		sqinfo_off(square(c, grid)->info, SQUARE_TRAP);
 
 		/* Take note */
 		square_note_spot(c, grid);
@@ -170,7 +172,7 @@ void square_free_trap(struct chunk *c, struct loc grid)
  */
 bool square_remove_all_traps(struct chunk *c, struct loc grid)
 {
-	struct trap *trap = square(c, grid).trap;
+	struct trap *trap = square(c, grid)->trap;
 	bool were_there_traps = trap == NULL ? false : true;
 
 	assert(square_in_bounds(c, grid));
@@ -203,7 +205,7 @@ bool square_remove_trap(struct chunk *c, struct loc grid, int t_idx_remove)
 
 	/* Look at the traps in this grid */
 	struct trap *prev_trap = NULL;
-	struct trap *trap = square(c, grid).trap;
+	struct trap *trap = square(c, grid)->trap;
 
 	assert(square_in_bounds(c, grid));
 	while (trap) {
@@ -298,7 +300,7 @@ static int pick_trap(struct chunk *c, int feat, int trap_level)
 		/* Check legality of trapdoors. */
 		if (trf_has(kind->flags, TRF_DOWN)) {
 			/* No trap doors on quest levels */
-			if (is_quest(player->depth)) continue;
+			if (is_quest(player, player->depth)) continue;
 
 			/* No trap doors on the deepest level */
 			if (player->depth >= z_info->max_depth - 1)
@@ -347,16 +349,21 @@ void place_trap(struct chunk *c, struct loc grid, int t_idx, int trap_level)
 {
 	struct trap *new_trap;
 
-    /* We've been called with an illegal index; choose a random trap */
-    if ((t_idx <= 0) || (t_idx >= z_info->trap_max)) {
+	/* We've been called with an illegal index; choose a random trap */
+	if ((t_idx <= 0) || (t_idx >= z_info->trap_max)) {
 		/* Require the correct terrain */
 		if (!square_player_trap_allowed(c, grid)) return;
 
-		t_idx = pick_trap(c, square(c, grid).feat, trap_level);
-    }
+		t_idx = pick_trap(c, square(c, grid)->feat, trap_level);
+	}
 
-    /* Failure */
-    if (t_idx < 0) return;
+	/* Failure */
+	if (t_idx < 0) return;
+	/* Don't allow trap doors in single combat arenas. */
+	if (player && player->upkeep && player->upkeep->arena_level
+			&& trf_has(trap_info[t_idx].flags, TRF_DOWN)) {
+		return;
+	}
 
 	/* Allocate a new trap for this grid (at the front of the list) */
 	new_trap = mem_zalloc(sizeof(*new_trap));
@@ -371,7 +378,7 @@ void place_trap(struct chunk *c, struct loc grid, int t_idx, int trap_level)
 	trf_copy(new_trap->flags, trap_info[t_idx].flags);
 
 	/* Toggle on the trap marker */
-	sqinfo_on(square(c, grid).info, SQUARE_TRAP);
+	sqinfo_on(square(c, grid)->info, SQUARE_TRAP);
 
 	/* Redraw the grid */
 	square_note_spot(c, grid);
@@ -443,7 +450,7 @@ bool square_reveal_trap(struct chunk *c, struct loc grid, bool always,
  */
 void square_memorize_traps(struct chunk *c, struct loc grid)
 {
-	struct trap *trap = square(c, grid).trap;
+	struct trap *trap = square(c, grid)->trap;
 	struct trap *current = NULL;
 	if (c != cave) return;
 
@@ -469,17 +476,6 @@ void square_memorize_traps(struct chunk *c, struct loc grid)
 	}
 }
 
-/**
- * Determine if a trap affects the player.
- * Always miss 5% of the time, Always hit 5% of the time.
- * Otherwise, match trap power against player armor.
- */
-bool trap_check_hit(int power)
-{
-	return test_hit(power, player->state.ac + player->state.to_a, true);
-}
-
-
 /**
  * Hit a trap. 
  */
@@ -516,7 +512,7 @@ extern void hit_trap(struct loc grid, int delayed)
 
 		/* Give a message */
 		if (trap->kind->msg)
-			msg(trap->kind->msg);
+			msg("%s", trap->kind->msg);
 
 		/* Test for save due to flag */
 		for (flag = of_next(trap->kind->save_flags, FLAG_START);
@@ -528,7 +524,8 @@ extern void hit_trap(struct loc grid, int delayed)
 			}
 
 		/* Test for save due to armor */
-		if (trf_has(trap->kind->flags, TRF_SAVE_ARMOR) && !trap_check_hit(125))
+		if (trf_has(trap->kind->flags, TRF_SAVE_ARMOR)
+			&& !check_hit(player, 125))
 			saved = true;
 
 		/* Test for save due to saving throw */
@@ -539,10 +536,10 @@ extern void hit_trap(struct loc grid, int delayed)
 		/* Save, or fire off the trap */
 		if (saved) {
 			if (trap->kind->msg_good)
-				msg(trap->kind->msg_good);
+				msg("%s", trap->kind->msg_good);
 		} else {
 			if (trap->kind->msg_bad)
-				msg(trap->kind->msg_bad);
+				msg("%s", trap->kind->msg_bad);
 			effect = trap->kind->effect;
 			effect_do(effect, source_trap(trap), NULL, &ident, false, 0, 0, 0, NULL);
 
@@ -552,7 +549,7 @@ extern void hit_trap(struct loc grid, int delayed)
 			/* Do any extra effects */
 			if (trap->kind->effect_xtra && one_in_(2)) {
 				if (trap->kind->msg_xtra)
-					msg(trap->kind->msg_xtra);
+					msg("%s", trap->kind->msg_xtra);
 				effect = trap->kind->effect_xtra;
 				effect_do(effect, source_trap(trap), NULL, &ident, false,
 						  0, 0, 0, NULL);
@@ -565,7 +562,8 @@ extern void hit_trap(struct loc grid, int delayed)
 		/* Some traps drop you a dungeon level */
 		if (trf_has(trap->kind->flags, TRF_DOWN))
 			dungeon_change_level(player,
-								 dungeon_get_next_level(player->depth, 1));
+				dungeon_get_next_level(player,
+				player->depth, 1));
 
 		/* Some traps drop you onto them */
 		if (trf_has(trap->kind->flags, TRF_PIT))
@@ -582,11 +580,17 @@ extern void hit_trap(struct loc grid, int delayed)
 
 		/* Trap becomes visible (always XXX) */
 		trf_on(trap->flags, TRF_VISIBLE);
-		square_memorize(cave, grid);
 	}
 
     /* Verify traps (remove marker if appropriate) */
-    (void)square_verify_trap(cave, grid, 0);
+    if (square_verify_trap(cave, grid, 0)) {
+	/* At least one trap left.  Memorize the visible ones and the grid. */
+	square_memorize_traps(cave, grid);
+	square_memorize(cave, grid);
+    }
+    if (square_isseen(cave, grid)) {
+	square_light_spot(cave, grid);
+    }
 }
 
 /**
@@ -602,7 +606,7 @@ bool square_set_trap_timeout(struct chunk *c, struct loc grid, bool domsg,
 	assert(square_in_bounds(c, grid));
 
 	/* Look at the traps in this grid */
-	current_trap = square(c, grid).trap;
+	current_trap = square(c, grid)->trap;
 	while (current_trap) {
 		/* Get the next trap (may be NULL) */
 		struct trap *next_trap = current_trap->next;
@@ -641,7 +645,7 @@ bool square_set_trap_timeout(struct chunk *c, struct loc grid, bool domsg,
  */
 int square_trap_timeout(struct chunk *c, struct loc grid, int t_idx)
 {
-	struct trap *current_trap = square(c, grid).trap;
+	struct trap *current_trap = square(c, grid)->trap;
 	while (current_trap) {
 		/* Get the next trap (may be NULL) */
 		struct trap *next_trap = current_trap->next;
