@@ -89,6 +89,19 @@ const char *keycode_find_desc(keycode_t kc)
 }
 
 
+/**
+ * Given a keycode, return whether it corresponds to a printable character.
+ */
+bool keycode_isprint(keycode_t kc)
+{
+	/*
+	 * Exclude ESCAPE (not part of the Unicode standard).  Otherwise,
+	 * treat the keycode as a Unicode code point.
+	 */
+	return kc != ESCAPE && utf32_isprint(kc);
+}
+
+
 /**
  * Convert a hexidecimal-digit into a decimal
  */
@@ -105,7 +118,7 @@ static int dehex(char c)
 void keypress_from_text(struct keypress *buf, size_t len, const char *str)
 {
 	size_t cur = 0;
-	byte mods = 0;
+	uint8_t mods = 0;
 
 	memset(buf, 0, len * sizeof *buf);
 
@@ -113,7 +126,7 @@ void keypress_from_text(struct keypress *buf, size_t len, const char *str)
 	{ \
 		int p = (pos); \
 		keycode_t c = (cod); \
-		byte m = (mod); \
+		uint8_t m = (mod); \
 \
 		if ((m & KC_MOD_CONTROL) && ENCODE_KTRL(c)) { \
 			m &= ~KC_MOD_CONTROL; \
@@ -152,6 +165,7 @@ void keypress_from_text(struct keypress *buf, size_t len, const char *str)
 				case '\\': STORE(buf, cur++, mods, '\\'); break;
 				case '^': STORE(buf, cur++, mods, '^'); break;
 				case '[': STORE(buf, cur++, mods, '['); break;
+				case '{': STORE(buf, cur++, mods, '{'); break;
 				default: STORE(buf, cur++, mods, *str); break;
 			}
 
@@ -263,6 +277,7 @@ void keypress_to_text(char *buf, size_t len, const struct keypress *src,
 				}
 				case '^': strnfcat(buf, len, &end, "\\^"); break;
 				case '[': strnfcat(buf, len, &end, "\\["); break;
+				case '{': strnfcat(buf, len, &end, "\\{"); break;
 				default: {
 					if (i < 127)
 						strnfcat(buf, len, &end, "%c", i);
@@ -337,3 +352,28 @@ bool char_matches_key(wchar_t c, keycode_t key)
 	text_mbstowcs(keychar, k, 1);
 	return (c == keychar[0]);
 }
+
+/**
+ * Check if a UI event matches a certain keycode ('a', 'b', etc)
+ */
+bool event_is_key(ui_event e, keycode_t key)
+{
+	return e.type == EVT_KBRD && e.key.code == key;
+}
+
+/**
+ * Check if a UI event matches a certain mouse button (1, 2, 3)
+ */
+bool event_is_mouse(ui_event e, uint8_t button)
+{
+	return e.type == EVT_MOUSE && e.mouse.button == button;
+}
+
+/**
+ * Check if a UI event matches a certain mouse button (1, 2, 3) and has
+ * specific modifiers (KC_MOD_*)
+ */
+bool event_is_mouse_m(ui_event e, uint8_t button, uint8_t mods)
+{
+	return e.type == EVT_MOUSE && e.mouse.button == button && (e.mouse.mods & mods);
+}
