@@ -26,17 +26,18 @@
 #include "obj-util.h"
 #include "player-calcs.h"
 #include "player-path.h"
-#include "randname.h"
 #include "savefile.h"
 #include "target.h"
 #include "ui-command.h"
 #include "ui-context.h"
 #include "ui-curse.h"
 #include "ui-display.h"
+#include "ui-effect.h"
 #include "ui-help.h"
 #include "ui-keymap.h"
 #include "ui-knowledge.h"
 #include "ui-map.h"
+#include "ui-menu.h"
 #include "ui-object.h"
 #include "ui-output.h"
 #include "ui-player.h"
@@ -47,7 +48,7 @@
 #include "ui-target.h"
 
 static bool inkey_xtra;
-u32b inkey_scan;		/* See the "inkey()" function */
+uint32_t inkey_scan;		/* See the "inkey()" function */
 bool inkey_flag;		/* See the "inkey()" function */
 
 /**
@@ -352,7 +353,7 @@ ui_event inkey_m(void)
  */
 static void msg_flush(int x)
 {
-	byte a = COLOUR_L_BLUE;
+	uint8_t a = COLOUR_L_BLUE;
 
 	/* Pause for response */
 	Term_putstr(x, 0, -1, a, "-more-");
@@ -402,7 +403,7 @@ void display_message(game_event_type unused, game_event_data *data, void *user)
 	int n;
 	char *t;
 	char buf[1024];
-	byte color;
+	uint8_t color;
 	int w, h;
 
 	int type;
@@ -413,7 +414,12 @@ void display_message(game_event_type unused, game_event_data *data, void *user)
 	type = data->message.type;
 	msg = data->message.msg;
 
-	if (type == MSG_BELL || !msg || !Term || !character_generated)
+	if (Term && type == MSG_BELL) {
+		Term_xtra(TERM_XTRA_NOISE, 0);
+		return;
+	}
+
+	if (!msg || !Term || !character_generated)
 		return;
 
 	/* Obtain the size */
@@ -545,9 +551,10 @@ void clear_from(int row)
 }
 
 /**
- * The default "keypress handling function" for askfor_aux, this takes the
- * given keypress, input buffer, length, etc, and does the appropriate action
- * for each keypress, such as moving the cursor left or inserting a character.
+ * The default "keypress handling function" for askfor_aux()/askfor_aux_ext(),
+ * this takes the given keypress, input buffer, length, etc, and does the
+ * appropriate action for that keypress, such as moving the cursor left or
+ * inserting a character.
  *
  * It should return true when editing of the buffer is "complete" (e.g. on
  * the press of RETURN).
@@ -555,6 +562,8 @@ void clear_from(int row)
 bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len,
 						 struct keypress keypress, bool firsttime)
 {
+	size_t ulen = utf8_strlen(buf);
+
 	switch (keypress.code)
 	{
 		case ESCAPE:
@@ -565,51 +574,71 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len,
 		
 		case KC_ENTER:
 		{
-			*curs = *len;
+			*curs = ulen;
 			return true;
 		}
 		
 		case ARROW_LEFT:
 		{
-			if (firsttime) *curs = 0;
-			if (*curs > 0) (*curs)--;
+			if (firsttime) {
+				*curs = 0;
+			} else if (*curs > 0) {
+				(*curs)--;
+			}
 			break;
 		}
 		
 		case ARROW_RIGHT:
 		{
-			if (firsttime) *curs = *len - 1;
-			if (*curs < *len) (*curs)++;
+			if (firsttime) {
+				*curs = ulen;
+			} else if (*curs < ulen) {
+				(*curs)++;
+			}
 			break;
 		}
 		
 		case KC_BACKSPACE:
 		case KC_DELETE:
 		{
+			char *ocurs, *oshift;
+
 			/* If this is the first time round, backspace means "delete all" */
 			if (firsttime) {
 				buf[0] = '\0';
 				*curs = 0;
 				*len = 0;
-
 				break;
 			}
 
 			/* Refuse to backspace into oblivion */
 			if ((keypress.code == KC_BACKSPACE && *curs == 0) ||
-				(keypress.code == KC_DELETE && *curs >= *len))
+				(keypress.code == KC_DELETE && *curs >= ulen))
 				break;
 
-			/* Move the string from k to nul along to the left by 1 */
-			if (keypress.code == KC_BACKSPACE)
-				memmove(&buf[*curs - 1], &buf[*curs], *len - *curs);
-			else
-				memmove(&buf[*curs], &buf[*curs+1], *len - *curs -1);
-
-			/* Decrement */
-			if(keypress.code == KC_BACKSPACE)
+			/*
+			 * Move the string from k to nul along to the left
+			 * by 1.  First, have to get offset corresponding to
+			 * the cursor position.
+			 */
+			ocurs = utf8_fskip(buf, *curs, NULL);
+			assert(ocurs);
+			if (keypress.code == KC_BACKSPACE) {
+				/* Get offset of the previous character. */
+				oshift = utf8_rskip(ocurs, 1, buf);
+				assert(oshift);
+				memmove(oshift, ocurs, *len - (ocurs - buf));
+				/* Decrement. */
 				(*curs)--;
-			(*len)--;
+				*len -= ocurs - oshift;
+			} else {
+				/* Get offset of the next character. */
+				oshift = utf8_fskip(buf + *curs, 1, NULL);
+				assert(oshift);
+				memmove(ocurs, oshift, *len - (oshift - buf));
+				/* Decrement */
+				*len -= oshift - ocurs;
+			}
 
 			/* Terminate */
 			buf[*len] = '\0';
@@ -619,10 +648,18 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len,
 		
 		default:
 		{
-			bool atnull = (buf[*curs] == 0);
+			bool atnull = (*curs == ulen);
+			char encoded[5];
+			size_t n_enc = 0;
+			char *ocurs;
 
-			if (!isprint(keypress.code)) {
-				bell("Illegal edit key!");
+			if (keycode_isprint(keypress.code)) {
+				n_enc = utf32_to_utf8(encoded,
+					N_ELEMENTS(encoded), &keypress.code,
+					1, NULL);
+			}
+			if (n_enc == 0) {
+				bell();
 				break;
 			}
 
@@ -634,20 +671,29 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len,
 				atnull = 1;
 			}
 
-			if (atnull) {
-				/* Make sure we have enough room for a new character */
-				if ((*curs + 1) >= buflen) break;
-			} else {
-				/* Make sure we have enough room to add a new character */
-				if ((*len + 1) >= buflen) break;
-
-				/* Move the rest of the buffer along to make room */
-				memmove(&buf[*curs+1], &buf[*curs], *len - *curs);
+			/* Make sure we have enough room for the new character */
+			if (*len + n_enc >= buflen) {
+				break;
 			}
 
-			/* Insert the character */
-			buf[(*curs)++] = (char)keypress.code;
-			(*len)++;
+			/* Insert the encoded character. */
+			if (atnull) {
+				ocurs = buf + *len;
+			} else {
+				ocurs = utf8_fskip(buf, *curs, NULL);
+				assert(ocurs);
+				/*
+				 * Move the rest of the buffer along to make
+				 * room.
+				 */
+				memmove(ocurs + n_enc, ocurs,
+					*len - (ocurs - buf));
+			}
+			memcpy(ocurs, encoded, n_enc);
+
+			/* Update position and length. */
+			(*curs)++;
+			*len += n_enc;
 
 			/* Terminate */
 			buf[*len] = '\0';
@@ -661,6 +707,34 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len,
 }
 
 
+/**
+ * Handle a mouse event during editing of a string.  This is the default mouse
+ * event handler for askfor_aux_ext().
+ *
+ * \param buf is the buffer with the string to be edited.
+ * \param buflen is the maximum number of characters that may be stored in buf.
+ * \param curs is the pointer to the position of the cursor in the buffer.
+ * \param len is the pointer to position of the first null character in the
+ * buffer.
+ * \param mouse is a description of the mouse event to handle.
+ * \param firsttime is whether or not this is the first call to the keypress or
+ * mouse handler in this editing session.
+ * \return zero if the editing session should continue, one if the editing
+ * session should end and the current contents of the buffer be accepted, or
+ * two if the editing session should end and the current contents of the buffer
+ * be rejected.
+ *
+ * askfor_aux_mouse() is very simple.  Any mouse click terminates the editing
+ * session, and if that click is with the second button, the result of the
+ * editing is rejected.
+ */
+int askfor_aux_mouse(char *buf, size_t buflen, size_t *curs, size_t *len,
+		struct mouseclick mouse, bool firsttime)
+{
+	return (mouse.button == 2) ? 2 : 1;
+}
+
+
 /**
  * Get some input at the cursor location.
  *
@@ -741,6 +815,127 @@ bool askfor_aux(char *buf, size_t len, bool (*keypress_h)(char *, size_t, size_t
 }
 
 
+/**
+ * Act like askfor_aux() but allow customization of what happens with mouse
+ * input.
+ *
+ * \param buf is the buffer with the string to edit.
+ * \param len is the maximum number of characters buf can hold.
+ * \param keypress_h is the function to call to handle a keypress.  It may be
+ * NULL.  In that case, askfor_aux_keypress() is used.  The function takes
+ * six arguments and should return whether or not to end this editing
+ * session.  The first argument is the buffer with the string to be edited.  The
+ * second argument is the maximum number of characters that can be stored in
+ * that buffer.  The third argument is a pointer to the position of the cursor
+ * in the buffer.  The fourth argument is a pointer to the position of the
+ * first null character in the buffer.  The fifth argument is a description of
+ * the keypress to be handled.  The sixth argument is whether or not this is
+ * the first call to the keypress handler or mouse handler in this editing
+ * session.
+ * \param mouse_h is the function to call to handle a mouse click.  It may be
+ * NULL.  In that case, askfor_aux_mouse() is used.  The function takes six
+ * arguments and should either return zero (this editing should session should
+ * continue), one (this editing session should end and the result in the buffer
+ * be accepted), or a non-zero value other than one (this editing session should
+ * end and the result in the buffer should not be accepted).  The first argument
+ * is the buffer with the string to be edited.  The second argument is the
+ * maximum number of characters that can be stored in that buffer.  The third
+ * argument is a pointer to the position of the cursor in the buffer.  The
+ * fourth argument is a pointer to the position of the first null character in
+ * the buffer.  The fifth argument is a description of the keypress to be
+ * handled.  The sixth argument is whether or not this is the first call to the
+ * keypress handler or mouse handler in this editing session.
+ */
+bool askfor_aux_ext(char *buf, size_t len,
+	bool (*keypress_h)(char *, size_t, size_t *, size_t *, struct keypress, bool),
+	int (*mouse_h)(char *, size_t, size_t *, size_t *, struct mouseclick, bool))
+{
+	size_t k = 0;		/* Cursor position */
+	size_t nul = 0;		/* Position of the null byte in the string */
+	bool firsttime = true;
+	bool done = false;
+	bool accepted = true;
+	int y, x;
+
+	if (keypress_h == NULL) {
+		keypress_h = askfor_aux_keypress;
+	}
+	if (mouse_h == NULL) {
+		mouse_h = askfor_aux_mouse;
+	}
+
+	/* Locate the cursor */
+	Term_locate(&x, &y);
+
+	/* Paranoia */
+	if (x < 0 || x >= 80) x = 0;
+
+	/* Restrict the length */
+	if (x + len > 80) len = 80 - x;
+
+	/* Truncate the default entry */
+	buf[len-1] = '\0';
+
+	/* Get the position of the null byte */
+	nul = strlen(buf);
+
+	/* Display the default answer */
+	Term_erase(x, y, (int)len);
+	Term_putstr(x, y, -1, COLOUR_YELLOW, buf);
+
+	/* Process input */
+	while (!done) {
+		ui_event in;
+
+		/* Place cursor */
+		Term_gotoxy(x + k, y);
+
+		/*
+		 * Get input.  Emulate what inkey() does without the coercing
+		 * mouse events to look like keystrokes.
+		 */
+		while (1) {
+			in = inkey_ex();
+			if (in.type == EVT_KBRD || in.type == EVT_MOUSE) {
+				break;
+			}
+			if (in.type == EVT_BUTTON) {
+				in.type = EVT_KBRD;
+				break;
+			}
+			if (in.type == EVT_ESCAPE) {
+				in.type = EVT_KBRD;
+				in.key.code = ESCAPE;
+				in.key.mods = 0;
+				break;
+			}
+		}
+
+		/* Pass on to the appropriate handler. */
+		if (in.type == EVT_KBRD) {
+			done = keypress_h(buf, len, &k, &nul, in.key,
+				firsttime);
+			accepted = (in.key.code != ESCAPE);
+		} else if (in.type == EVT_MOUSE) {
+			int result = mouse_h(buf, len, &k, &nul, in.mouse,
+				firsttime);
+
+			if (result != 0) {
+				done = true;
+				accepted = (result == 1);
+			}
+		}
+
+		/* Update the entry */
+		Term_erase(x, y, (int)len);
+		Term_putstr(x, y, -1, COLOUR_WHITE, buf);
+
+		/* Not the first time round anymore */
+		firsttime = false;
+	}
+
+	return accepted;
+}
 
 
 /**
@@ -758,9 +953,7 @@ static bool get_name_keypress(char *buf, size_t buflen, size_t *curs,
 	{
 		case '*':
 		{
-			*len = randname_make(RANDNAME_TOLKIEN, 4, 8, buf, buflen,
-								 name_sections);
-			my_strcap(buf);
+			*len = player_random_name(buf, buflen);
 			*curs = 0;
 			result = false;
 			break;
@@ -778,6 +971,90 @@ static bool get_name_keypress(char *buf, size_t buflen, size_t *curs,
 }
 
 
+/**
+ * Handle a mouse event during editing of a string:  presents a context menu
+ * with options appropriate for handling editing a character's name.
+ *
+ * \param buf is the buffer with the string to be edited.
+ * \param buflen is the maximum number of characters that may be stored in buf.
+ * \param curs is the pointer to the position of the cursor in the buffer.
+ * \param len is the pointer to position of the first null character in the
+ * buffer.
+ * \param mouse is a description of the mouse event to handle.
+ * \param firsttime is whether or not this is the first call to the keypress or
+ * mouse handler in this editing session.
+ * \return zero if the editing session should continue, one if the editing
+ * session should end and the current contents of the buffer be accepted, or
+ * two if the editing session should end and the current contents of the buffer
+ * be rejected.
+ */
+static int handle_name_mouse(char *buf, size_t buflen, size_t *curs,
+		size_t *len, struct mouseclick mouse, bool firsttime)
+{
+	enum { ACT_CTX_NAME_ACCEPT, ACT_CTX_NAME_RANDOM, ACT_CTX_NAME_CLEAR };
+	int result = 2;
+	char *labels;
+	struct menu *m;
+	int action;
+
+	/*
+	 * A mouse click with the second button ends the editing session and
+	 * indicates that the result of editing should be rejected.
+	 */
+	if (mouse.button == 2) {
+		return result;
+	}
+
+	/* By default, don't end the editing session. */
+	result = 0;
+
+	/* Present a context menu with the possible actions. */
+	labels = string_make(lower_case);
+	m = menu_dynamic_new();
+
+	m->selections = labels;
+	menu_dynamic_add_label(m, "Accept", 'a', ACT_CTX_NAME_ACCEPT, labels);
+	menu_dynamic_add_label(m, "Set to random name", 'r',
+		ACT_CTX_NAME_RANDOM, labels);
+	menu_dynamic_add_label(m, "Clear name", 'c', ACT_CTX_NAME_CLEAR,
+		labels);
+
+	screen_save();
+
+	menu_dynamic_calc_location(m, mouse.x, mouse.y);
+	region_erase_bordered(&m->boundary);
+
+	action = menu_dynamic_select(m);
+
+	menu_dynamic_free(m);
+	string_free(labels);
+
+	screen_load();
+
+	/* Do what was requested. */
+	switch (action) {
+	case ACT_CTX_NAME_ACCEPT:
+		/* End the editing session and accept the result. */
+		result = 1;
+		break;
+
+	case ACT_CTX_NAME_RANDOM:
+		*len = player_random_name(buf, buflen);
+		*curs = 0;
+		break;
+
+	case ACT_CTX_NAME_CLEAR:
+		assert(buflen > 0);
+		buf[0] = '\0';
+		*len = 0;
+		*curs = 0;
+		break;
+	}
+
+	return result;
+}
+
+
 /**
  * Gets a name for the character, reacting to name changes.
  *
@@ -797,7 +1074,7 @@ bool get_character_name(char *buf, size_t buflen)
 	my_strcpy(buf, player->full_name, buflen);
 
 	/* Ask the user for a string */
-	res = askfor_aux(buf, buflen, get_name_keypress);
+	res = askfor_aux_ext(buf, buflen, get_name_keypress, handle_name_mouse);
 
 	/* Clear prompt */
 	prt("", 0, 0);
@@ -819,7 +1096,7 @@ bool get_character_name(char *buf, size_t buflen)
  * See "askfor_aux" for some notes about "buf" and "len", and about
  * the return value of this function.
  */
-bool textui_get_string(const char *prompt, char *buf, size_t len)
+static bool textui_get_string(const char *prompt, char *buf, size_t len)
 {
 	bool res;
 
@@ -844,7 +1121,7 @@ bool textui_get_string(const char *prompt, char *buf, size_t len)
 /**
  * Request a "quantity" from the user
  */
-int textui_get_quantity(const char *prompt, int max)
+static int textui_get_quantity(const char *prompt, int max)
 {
 	int amt = 1;
 
@@ -893,18 +1170,21 @@ int textui_get_quantity(const char *prompt, int max)
  *
  * Note that "[y/n]" is appended to the prompt.
  */
-bool textui_get_check(const char *prompt)
+static bool textui_get_check(const char *prompt)
 {
 	ui_event ke;
 
 	char buf[80];
 
+	/*
+	 * Hack -- Build a "useful" prompt; do this first so prompts built by
+	 * format() won't run afoul of event_signal()'s side effects.
+	 */
+	strnfmt(buf, 78, "%.70s[y/n] ", prompt);
+
 	/* Paranoia */
 	event_signal(EVENT_MESSAGE_FLUSH);
 
-	/* Hack -- Build a "useful" prompt */
-	strnfmt(buf, 78, "%.70s[y/n] ", prompt);
-
 	/* Prompt for it */
 	prt(buf, 0, 0);
 	ke = inkey_m();
@@ -984,7 +1264,7 @@ static bool get_file_text(const char *suggested_name, char *path, size_t len)
 			/* Make sure it's actually a filename */
 			if (buf[0] == '\0' || buf[0] == ' ') return false;
 	} else {
-		int len;
+		int old_len;
 		time_t ltime;
 		struct tm *today;
 
@@ -996,8 +1276,8 @@ static bool get_file_text(const char *suggested_name, char *path, size_t len)
 
 		/* Overwrite the ".txt" that was added */
 		assert(strlen(buf) >= 4);
-		len = strlen(buf) - 4;
-		strftime(buf + len, sizeof(buf) - len, "-%Y-%m-%d-%H-%M.txt", today);
+		old_len = strlen(buf) - 4;
+		strftime(buf + old_len, sizeof(buf) - len, "-%Y-%m-%d-%H-%M.txt", today);
 
 		/* Prompt the user to confirm or cancel the file dump */
 		if (!get_check(format("Confirm writing to %s? ", buf))) return false;
@@ -1042,7 +1322,7 @@ bool (*get_file)(const char *suggested_name, char *path, size_t len) = get_file_
  * -------
  * Returns true unless the character is "Escape"
  */
-bool textui_get_com(const char *prompt, char *command)
+static bool textui_get_com(const char *prompt, char *command)
 {
 	ui_event ke;
 	bool result;
@@ -1087,12 +1367,12 @@ bool get_com_ex(const char *prompt, ui_event *command)
  *
  * This function is stupid.  XXX XXX XXX
  */
-void pause_line(struct term *term)
+void pause_line(struct term *tm)
 {
-	prt("", term->hgt - 1, 0);
-	put_str("[Press any key to continue]", term->hgt - 1, (Term->wid - 27) / 2);
+	prt("", tm->hgt - 1, 0);
+	put_str("[Press any key to continue]", tm->hgt - 1, (tm->wid - 27) / 2);
 	(void)anykey();
-	prt("", term->hgt - 1, 0);
+	prt("", tm->hgt - 1, 0);
 }
 
 static int dir_transitions[10][10] =
@@ -1124,7 +1404,7 @@ static int dir_transitions[10][10] =
  * This function tracks and uses the "global direction", and uses
  * that as the "desired direction", if it is set.
  */
-bool textui_get_rep_dir(int *dp, bool allow_5)
+static bool textui_get_rep_dir(int *dp, bool allow_5)
 {
 	int dir = 0;
 
@@ -1204,7 +1484,7 @@ bool textui_get_rep_dir(int *dp, bool allow_5)
 		}
 
 		/* Oops */
-		if (!dir) bell("Illegal repeatable direction!");
+		if (!dir) bell();
 	}
 
 	/* Clear the prompt */
@@ -1230,7 +1510,7 @@ bool textui_get_rep_dir(int *dp, bool allow_5)
  * Note that "Force Target", if set, will pre-empt user interaction,
  * if there is a usable target already set.
  */
-bool textui_get_aim_dir(int *dp)
+static bool textui_get_aim_dir(int *dp)
 {
 	/* Global direction */
 	int dir = 0;
@@ -1305,7 +1585,7 @@ bool textui_get_aim_dir(int *dp)
 		}
 
 		/* Error */
-		if (!dir) bell("Illegal aim direction!");
+		if (!dir) bell();
 	}
 
 	/* No direction */
@@ -1331,6 +1611,7 @@ void textui_input_init(void)
 	get_aim_dir_hook = textui_get_aim_dir;
 	get_spell_from_book_hook = textui_get_spell_from_book;
 	get_spell_hook = textui_get_spell;
+	get_effect_from_list_hook = textui_get_effect_from_list;
 	get_item_hook = textui_get_item;
 	get_curse_hook = textui_get_curse;
 	get_panel_hook = textui_get_panel;
@@ -1367,7 +1648,7 @@ static int textui_get_count(void)
 			count = count * 10 + D2I(ke.code);
 
 			if (count >= 9999) {
-				bell("Invalid repeat count!");
+				bell();
 				count = 9999;
 			}
 		} else {
