@@ -25,9 +25,21 @@
 #include "ui-term.h"
 
 /**
- * Holds a generic command - if cmd is set to other than CMD_NULL 
- * it simply pushes that command to the game, otherwise the hook 
- * function will be called.
+ * Holds a generic command.  If hook is not NULL, that function
+ * will be called.  Otherwise, if cmd is not CMD_NULL, that command
+ * will be pushed to the game.  If prereq is not NULL, it is tested
+ * before pushing cmd or invoking hook.  If cmd is CMD_NULL and hook
+ * is NULL, the command is an access point to a nested set of commands.
+ * In that case, a positive value for nested_keymap sets the mapping
+ * of keys to the nested commands; otherwise, those commands go through
+ * the same lookup as directly entered commands.  When nested_keymap is
+ * positive, nested_prompt is the prompt to solicit the key and should
+ * be non-NULL for the commands to be activated.  nested_error is the
+ * error message displayed if the entered key isn't recognized and may
+ * be NULL.  nested_name is used to look up the struct command_list for
+ * presentation of the nested commands through a menu.  nested_cached_idx
+ * is for an optimization so that lookup only has to be done once;
+ * initialize it to -1 for an access point to nested commands.
  */
 struct cmd_info
 {
@@ -36,6 +48,11 @@ struct cmd_info
 	cmd_code cmd;
 	void (*hook)(void);
 	bool (*prereq)(void);
+	int nested_keymap;
+	const char *nested_prompt;
+	const char *nested_error;
+	const char *nested_name;
+	int nested_cached_idx;
 };
 
 /**
@@ -46,9 +63,11 @@ struct command_list
 	const char *name;
 	struct cmd_info *list;
 	size_t len;
+	int menu_level;
+	int keymap;
 };
 
-#define SCAN_INSTANT ((u32b) -1)
+#define SCAN_INSTANT ((uint32_t) -1)
 #define SCAN_OFF 0
 
 extern struct cmd_info cmd_item[];
@@ -60,9 +79,9 @@ extern struct cmd_info cmd_hidden[];
 extern struct command_list cmds_all[];
 
 extern struct keypress *inkey_next;
-extern u32b inkey_scan;
+extern uint32_t inkey_scan;
 extern bool inkey_flag;
-extern u16b lazymove_delay;
+extern uint8_t lazymove_delay;
 extern bool msg_flag;
 extern bool arg_force_name;
 
@@ -77,13 +96,18 @@ void message_flush(game_event_type unused, game_event_data *data, void *user);
 void clear_from(int row);
 bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len,
 						 struct keypress keypress, bool firsttime);
+int askfor_aux_mouse(char *buf, size_t buflen, size_t *curs, size_t *len,
+	struct mouseclick mouse, bool firsttime);
 bool askfor_aux(char *buf, size_t len, bool (*keypress_h)(char *, size_t, size_t *, size_t *, struct keypress, bool));
+bool askfor_aux_ext(char *buf, size_t len,
+	bool (*keypress_h)(char *, size_t, size_t *, size_t *, struct keypress, bool),
+	int (*mouse_h)(char *, size_t, size_t *, size_t *, struct mouseclick, bool));
 bool get_character_name(char *buf, size_t buflen);
 char get_char(const char *prompt, const char *options, size_t len,
 			  char fallback);
 extern bool (*get_file)(const char *suggested_name, char *path, size_t len);
 bool get_com_ex(const char *prompt, ui_event *command);
-void pause_line(struct term *term);
+void pause_line(struct term *tm);
 void textui_input_init(void);
 ui_event textui_get_command(int *count);
 bool key_confirm_command(unsigned char c);
