@@ -20,7 +20,10 @@
 #include "angband.h"
 #include "cave.h"
 #include "cmds.h"
+#include "effects.h"
+#include "effects-info.h"
 #include "game-input.h"
+#include "game-world.h"
 #include "grafmode.h"
 #include "init.h"
 #include "mon-lore.h"
@@ -38,12 +41,14 @@
 #include "player-calcs.h"
 #include "player-history.h"
 #include "player-util.h"
+#include "project.h"
 #include "store.h"
 #include "target.h"
 #include "trap.h"
 #include "ui-context.h"
 #include "ui-equip-cmp.h"
 #include "ui-history.h"
+#include "ui-knowledge.h"
 #include "ui-menu.h"
 #include "ui-mon-list.h"
 #include "ui-mon-lore.h"
@@ -104,7 +109,7 @@ typedef struct {
 	wchar_t *(*xchar)(int oid);
 
 	/* Get color attr for OID (by address) */
-	byte *(*xattr)(int oid);
+	uint8_t *(*xattr)(int oid);
 
 	/* Returns optional extra prompt */
 	const char *(*xtra_prompt)(int oid);
@@ -134,8 +139,8 @@ static join_t *default_join;
 /**
  * Clipboard variables for copy & paste in visual mode
  */
-static byte attr_idx = 0;
-static byte char_idx = 0;
+static uint8_t attr_idx = 0;
+static wchar_t char_idx = 0;
 
 /**
  * ------------------------------------------------------------------------
@@ -214,7 +219,7 @@ static int logical_height(int height)
  * Display tiles.
  */
 static void display_tiles(int col, int row, int height, int width,
-						  byte attr_top, byte char_left)
+		uint8_t attr_top, wchar_t char_left)
 {
 	int i, j;
 
@@ -229,8 +234,8 @@ static void display_tiles(int col, int row, int height, int width,
 	for (i = 0; i < height; i++) {
 		/* Display columns until done */
 		for (j = 0; j < width; j++) {
-			byte a;
-			unsigned char c;
+			uint8_t a;
+			wchar_t c;
 			int x = col + actual_width(j);
 			int y = row + actual_height(i);
 			int ia, ic;
@@ -238,8 +243,8 @@ static void display_tiles(int col, int row, int height, int width,
 			ia = attr_top + i;
 			ic = char_left + j;
 
-			a = (byte)ia;
-			c = (unsigned char)ic;
+			a = (uint8_t)ia;
+			c = (wchar_t)ic;
 
 			/* Display symbol */
 			big_pad(x, y, a, c);
@@ -251,8 +256,8 @@ static void display_tiles(int col, int row, int height, int width,
 /**
  * Place the cursor at the correct position for tile picking
  */
-static void place_tile_cursor(int col, int row, byte a, byte c, byte attr_top,
-							  byte char_left)
+static void place_tile_cursor(int col, int row, uint8_t a, wchar_t c,
+		uint8_t attr_top, wchar_t char_left)
 {
 	int i = a - attr_top;
 	int j = c - char_left;
@@ -289,13 +294,12 @@ static void remove_tiles(int col, int row, bool *picker_ptr, int width,
  *  Do tile picker command -- Change tiles
  */
 static bool tile_picker_command(ui_event ke, bool *tile_picker_ptr,
-								int height, int width, byte *attr_top_ptr,
-								byte *char_left_ptr, byte *cur_attr_ptr,
-								byte *cur_char_ptr, int col, int row,
-								int *delay)
+		int height, int width, uint8_t *attr_top_ptr,
+		wchar_t *char_left_ptr, uint8_t *cur_attr_ptr,
+		wchar_t *cur_char_ptr, int col, int row, int *delay)
 {
-	static byte attr_old = 0;
-	static char char_old = 0;
+	static uint8_t attr_old = 0;
+	static wchar_t char_old = 0;
 
 	/* These are the distance we want to maintain between the
 	 * cursor and borders. */
@@ -309,8 +313,8 @@ static bool tile_picker_command(ui_event ke, bool *tile_picker_ptr,
 	if (*tile_picker_ptr &&  (ke.type == EVT_MOUSE)) {
 		int eff_width = actual_width(width);
 		int eff_height = actual_height(height);
-		byte a = *cur_attr_ptr;
-		byte c = *cur_char_ptr;
+		uint8_t a = *cur_attr_ptr;
+		wchar_t c = *cur_char_ptr;
 
 		int my = logical_height(ke.mouse.y - row);
 		int mx = logical_width(ke.mouse.x - col);
@@ -395,8 +399,8 @@ static bool tile_picker_command(ui_event ke, bool *tile_picker_ptr,
 				*tile_picker_ptr = true;
 				bigcurs = true;
 
-				*attr_top_ptr = (byte)MAX(0, (int)*cur_attr_ptr - frame_top);
-				*char_left_ptr = (char)MAX(0, (int)*cur_char_ptr - frame_left);
+				*attr_top_ptr = (uint8_t)MAX(0, (int)*cur_attr_ptr - frame_top);
+				*char_left_ptr = (wchar_t)MAX(0, (int)*cur_char_ptr - frame_left);
 
 				attr_old = *cur_attr_ptr;
 				char_old = *cur_char_ptr;
@@ -426,13 +430,13 @@ static bool tile_picker_command(ui_event ke, bool *tile_picker_ptr,
 			if (attr_idx) {
 				/* Set the char */
 				*cur_attr_ptr = attr_idx;
-				*attr_top_ptr = (byte)MAX(0, (int)*cur_attr_ptr - frame_top);
+				*attr_top_ptr = (uint8_t)MAX(0, (int)*cur_attr_ptr - frame_top);
 			}
 
 			if (char_idx) {
 				/* Set the char */
 				*cur_char_ptr = char_idx;
-				*char_left_ptr = (char)MAX(0, (int)*cur_char_ptr - frame_left);
+				*char_left_ptr = (wchar_t)MAX(0, (int)*cur_char_ptr - frame_left);
 			}
 
 			return true;
@@ -441,8 +445,8 @@ static bool tile_picker_command(ui_event ke, bool *tile_picker_ptr,
 		default:
 		{
 			int d = target_dir(ke.key);
-			byte a = *cur_attr_ptr;
-			byte c = *cur_char_ptr;
+			uint8_t a = *cur_attr_ptr;
+			wchar_t c = *cur_char_ptr;
 
 			if (!*tile_picker_ptr)
 				break;
@@ -495,7 +499,7 @@ static bool tile_picker_command(ui_event ke, bool *tile_picker_ptr,
 /**
  * Display glyph and colours
  */
-static void display_glyphs(int col, int row, int height, int width, byte a, 
+static void display_glyphs(int col, int row, int height, int width, uint8_t a,
 			   wchar_t c)
 {
 	int i;
@@ -518,11 +522,11 @@ static void display_glyphs(int col, int row, int height, int width, byte a,
  * Do glyph picker command -- Change glyphs
  */
 static bool glyph_command(ui_event ke, bool *glyph_picker_ptr,
-			  int height, int width, byte *cur_attr_ptr,
+			  int height, int width, uint8_t *cur_attr_ptr,
 			  wchar_t *cur_char_ptr, int col, int row)
 {
-	static byte attr_old = 0;
-	static char char_old = 0;
+	static uint8_t attr_old = 0;
+	static wchar_t char_old = 0;
 	
 	/* Get mouse movement */
 	if (*glyph_picker_ptr && (ke.type == EVT_MOUSE)) {
@@ -646,7 +650,7 @@ static bool glyph_command(ui_event ke, bool *glyph_picker_ptr,
 	    default:
 	    {
 		    int d = target_dir(ke.key);
-		    byte a = *cur_attr_ptr;
+		    uint8_t a = *cur_attr_ptr;
 		    
 		    if (!*glyph_picker_ptr)
 				break;
@@ -678,7 +682,7 @@ static void display_group_member(struct menu *menu, int oid,
 						bool cursor, int row, int col, int wid)
 {
 	const member_funcs *o_funcs = menu->menu_data;
-	byte attr = curs_attrs[CURS_KNOWN][cursor == oid];
+	uint8_t attr = curs_attrs[CURS_KNOWN][cursor == oid];
 
 	(void)wid;
 
@@ -692,7 +696,7 @@ static void display_group_member(struct menu *menu, int oid,
 	/* Do visual mode */
 	if (o_funcs->is_visual && o_funcs->xattr) {
 		wchar_t c = *o_funcs->xchar(oid);
-		byte a = *o_funcs->xattr(oid);
+		uint8_t a = *o_funcs->xattr(oid);
 
 		c_put_str(attr, format("%d/%d", a, c), row, 60);
 	}
@@ -742,8 +746,8 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 	bool tiles = (current_graphics_mode != NULL);
 	bool tile_picker = false;
 	bool glyph_picker = false;
-	byte attr_top = 0;
-	byte char_left = 0;
+	uint8_t attr_top = 0;
+	wchar_t char_left = 0;
 
 	int delay = 0;
 
@@ -858,10 +862,10 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 
 			/* Print dividers: horizontal and vertical */
 			for (i = 0; i < 79; i++)
-				Term_putch(i, 5, COLOUR_WHITE, '=');
+				Term_putch(i, 5, COLOUR_WHITE, L'=');
 
 			for (i = 0; i < browser_rows; i++)
-				Term_putch(g_name_len + 1, 6 + i, COLOUR_WHITE, '|');
+				Term_putch(g_name_len + 1, 6 + i, COLOUR_WHITE, L'|');
 
 
 			/* Reset redraw flag */
@@ -933,17 +937,17 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 		if (tile_picker) {
 		        bigcurs = true;
 			display_tiles(g_name_len + 3, 7, browser_rows - 1,
-				      wid - (g_name_len + 3), attr_top, 
+				      wid - (g_name_len + 3), attr_top,
 				      char_left);
 			place_tile_cursor(g_name_len + 3, 7, 
 					  *o_funcs.xattr(oid),
-					  (byte) *o_funcs.xchar(oid), 
+					  *o_funcs.xchar(oid),
 					  attr_top, char_left);
 		}
 
 		if (glyph_picker) {
 		        display_glyphs(g_name_len + 3, 7, browser_rows - 1,
-				       wid - (g_name_len + 3), 
+				       wid - (g_name_len + 3),
 				       *o_funcs.xattr(oid),
 				       *o_funcs.xchar(oid));
 		}
@@ -975,18 +979,19 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 		if (o_funcs.xattr && o_funcs.xchar) {
 			if (tiles) {
 				if (tile_picker_command(ke, &tile_picker, 
-										browser_rows - 1,
-										wid - (g_name_len + 3),
-										&attr_top, &char_left,
-										o_funcs.xattr(oid),
-										(byte *) o_funcs.xchar(oid), 
-										g_name_len + 3, 7, &delay))
+						browser_rows - 1,
+						wid - (g_name_len + 3),
+						&attr_top, &char_left,
+						o_funcs.xattr(oid),
+						o_funcs.xchar(oid),
+						g_name_len + 3, 7, &delay))
 					continue;
 			} else {
 				if (glyph_command(ke, &glyph_picker, 
-								  browser_rows - 1, wid - (g_name_len + 3), 
-								  o_funcs.xattr(oid), o_funcs.xchar(oid), 
-								  g_name_len + 3, 7))
+						browser_rows - 1, wid - (g_name_len + 3),
+						o_funcs.xattr(oid),
+						o_funcs.xchar(oid),
+						g_name_len + 3, 7))
 					continue;
 			}
 		}
@@ -1145,8 +1150,8 @@ static void display_monster(int col, int row, bool cursor, int oid)
 	struct monster_lore *lore = &l_list[r_idx];
 
 	/* Choose colors */
-	byte attr = curs_attrs[CURS_KNOWN][(int)cursor];
-	byte a = monster_x_attr[race->ridx];
+	uint8_t attr = curs_attrs[CURS_KNOWN][(int)cursor];
+	uint8_t a = monster_x_attr[race->ridx];
 	wchar_t c = monster_x_char[race->ridx];
 
 	if ((tile_height != 1) && (a & 0x80)) {
@@ -1211,7 +1216,7 @@ static wchar_t *m_xchar(int oid)
 	return &monster_x_char[default_join[oid].oid];
 }
 
-static byte *m_xattr(int oid)
+static uint8_t *m_xattr(int oid)
 {
 	return &monster_x_attr[default_join[oid].oid];
 }
@@ -1415,7 +1420,8 @@ static void get_artifact_display_name(char *o_name, size_t namelen, int a_idx)
 	object_wipe(known_obj);
 	object_copy(known_obj, obj);
 	obj->known = known_obj;
-	object_desc(o_name, namelen, obj, ODESC_PREFIX | ODESC_BASE | ODESC_SPOIL);
+	object_desc(o_name, namelen, obj,
+		ODESC_PREFIX | ODESC_BASE | ODESC_SPOIL, NULL);
 	object_wipe(known_obj);
 	object_wipe(obj);
 }
@@ -1425,7 +1431,7 @@ static void get_artifact_display_name(char *o_name, size_t namelen, int a_idx)
  */
 static void display_artifact(int col, int row, bool cursor, int oid)
 {
-	byte attr = curs_attrs[CURS_KNOWN][(int)cursor];
+	uint8_t attr = curs_attrs[CURS_KNOWN][(int)cursor];
 	char o_name[80];
 
 	get_artifact_display_name(o_name, sizeof o_name, oid);
@@ -1545,7 +1551,7 @@ static void desc_art_fake(int a_idx)
 
 	tb = object_info(obj, OINFO_NONE);
 	object_desc(header, sizeof(header), obj,
-			ODESC_PREFIX | ODESC_FULL | ODESC_CAPITAL);
+		ODESC_PREFIX | ODESC_FULL | ODESC_CAPITAL, player);
 	if (fake) {
 		object_wipe(known_obj);
 		object_wipe(obj);
@@ -1597,7 +1603,7 @@ static bool artifact_is_known(int a_idx)
 	if (player->wizard)
 		return true;
 
-	if (!a_info[a_idx].created)
+	if (!is_artifact_created(&a_info[a_idx]))
 		return false;
 
 	/* Check all objects to see if it exists but hasn't been IDed */
@@ -1648,13 +1654,20 @@ static void do_cmd_knowledge_artifacts(const char *name, int row)
 
 	int *artifacts;
 	int a_count = 0;
+	char title[40];
 
 	artifacts = mem_zalloc(z_info->a_max * sizeof(int));
 
 	/* Collect valid artifacts */
 	a_count = collect_known_artifacts(artifacts, z_info->a_max);
 
-	display_knowledge("artifacts", artifacts, a_count, obj_f, art_f, NULL);
+	if (OPT(player, birth_randarts)) {
+		strnfmt(title, sizeof(title), "artifacts (seed %08x)",
+			seed_randart);
+	} else {
+		strnfmt(title, sizeof(title), "artifacts");
+	}
+	display_knowledge(title, artifacts, a_count, obj_f, art_f, NULL);
 	mem_free(artifacts);
 }
 
@@ -1674,7 +1687,7 @@ static void display_ego_item(int col, int row, bool cursor, int oid)
 	struct ego_item *ego = &e_info[default_item_id(oid)];
 
 	/* Choose a color */
-	byte attr = curs_attrs[0 != (int)ego->everseen][0 != (int)cursor];
+	uint8_t attr = curs_attrs[0 != (int)ego->everseen][0 != (int)cursor];
 
 	/* Display the name */
 	c_prt(attr, ego->name, row, col);
@@ -1792,10 +1805,10 @@ static void display_object(int col, int row, bool cursor, int oid)
 
 	/* Choose a color */
 	bool aware = (!kind->flavor || kind->aware);
-	byte attr = curs_attrs[(int)aware][(int)cursor];
+	uint8_t attr = curs_attrs[(int)aware][(int)cursor];
 
 	/* Graphics versions of the object_char and object_attr defines */
-	byte a = object_kind_attr(kind);
+	uint8_t a = object_kind_attr(kind);
 	wchar_t c = object_kind_char(kind);
 
 	/* Don't display special artifacts */
@@ -1856,11 +1869,11 @@ static void desc_obj_fake(int k_idx)
 
 	tb = object_info(obj, OINFO_FAKE);
 	object_desc(header, sizeof(header), obj,
-			ODESC_PREFIX | ODESC_CAPITAL);
+		ODESC_PREFIX | ODESC_CAPITAL, player);
 
 	textui_textblock_show(tb, area, header);
-	object_delete(&known_obj);
-	object_delete(&obj);
+	object_delete(NULL, NULL, &known_obj);
+	object_delete(NULL, NULL, &obj);
 	textblock_free(tb);
 
 	/* Restore the old trackee */
@@ -1931,7 +1944,7 @@ static wchar_t *o_xchar(int oid)
 		return &flavor_x_char[kind->flavor->fidx];
 }
 
-static byte *o_xattr(int oid)
+static uint8_t *o_xattr(int oid)
 {
 	struct object_kind *kind = objkind_byid(oid);
 	if (!kind) return NULL;
@@ -2083,7 +2096,7 @@ static const char *rune_group_text[] =
  */
 static void display_rune(int col, int row, bool cursor, int oid )
 {
-	byte attr = curs_attrs[CURS_KNOWN][(int)cursor];
+	uint8_t attr = curs_attrs[CURS_KNOWN][(int)cursor];
 	const char *inscrip = quark_str(rune_note(oid));
 
 	c_prt(attr, rune_name(oid), row, col);
@@ -2110,9 +2123,9 @@ static void rune_lore(int oid)
 	char *title = string_make(rune_name(oid));
 
 	my_strcap(title);
-	textblock_append_c(tb, COLOUR_L_BLUE, title);
+	textblock_append_c(tb, COLOUR_L_BLUE, "%s", title);
 	textblock_append(tb, "\n");
-	textblock_append(tb, rune_desc(oid));
+	textblock_append(tb, "%s", rune_desc(oid));
 	textblock_append(tb, "\n");
 	textui_textblock_show(tb, SCREEN_REGION, NULL);
 	textblock_free(tb);
@@ -2163,7 +2176,7 @@ static void rune_xtra_act(struct keypress ch, int oid)
 
 			/* Add the autoinscription */
 			rune_set_note(oid, note_text);
-			rune_autoinscribe(oid);
+			rune_autoinscribe(player, oid);
 
 			/* Redraw gear */
 			player->upkeep->redraw |= (PR_INVEN | PR_EQUIP);
@@ -2237,7 +2250,7 @@ static const char *feature_group_text[] =
 static void display_feature(int col, int row, bool cursor, int oid )
 {
 	struct feature *feat = &f_info[oid];
-	byte attr = curs_attrs[CURS_KNOWN][(int)cursor];
+	uint8_t attr = curs_attrs[CURS_KNOWN][(int)cursor];
 
 	c_prt(attr, feat->name, row, col);
 
@@ -2283,7 +2296,7 @@ static const char *fkind_name(int gid)
 static enum grid_light_level f_uik_lighting = LIGHTING_LIT;
 
 /* XXX needs *better* retooling for multi-light terrain */
-static byte *f_xattr(int oid)
+static uint8_t *f_xattr(int oid)
 {
 	return &feat_x_attr[f_uik_lighting][oid];
 }
@@ -2299,9 +2312,9 @@ static void feat_lore(int oid)
 
 	if (feat->desc) {
 		my_strcap(title);
-		textblock_append_c(tb, COLOUR_L_BLUE, title);
+		textblock_append_c(tb, COLOUR_L_BLUE, "%s", title);
 		textblock_append(tb, "\n");
-		textblock_append(tb, feat->desc);
+		textblock_append(tb, "%s", feat->desc);
 		textblock_append(tb, "\n");
 		textui_textblock_show(tb, SCREEN_REGION, NULL);
 		textblock_free(tb);
@@ -2400,7 +2413,7 @@ static const char *trap_group_text[] =
 static void display_trap(int col, int row, bool cursor, int oid )
 {
 	struct trap_kind *trap = &trap_info[oid];
-	byte attr = curs_attrs[CURS_KNOWN][(int)cursor];
+	uint8_t attr = curs_attrs[CURS_KNOWN][(int)cursor];
 
 	c_prt(attr, trap->desc, row, col);
 
@@ -2468,7 +2481,7 @@ static const char *tkind_name(int gid)
 static enum grid_light_level t_uik_lighting = LIGHTING_LIT;
 
 /* XXX needs *better* retooling for multi-light terrain */
-static byte *t_xattr(int oid)
+static uint8_t *t_xattr(int oid)
 {
 	return &trap_x_attr[t_uik_lighting][oid];
 }
@@ -2484,9 +2497,9 @@ static void trap_lore(int oid)
 
 	if (trap->text) {
 		my_strcap(title);
-		textblock_append_c(tb, COLOUR_L_BLUE, title);
+		textblock_append_c(tb, COLOUR_L_BLUE, "%s", title);
 		textblock_append(tb, "\n");
-		textblock_append(tb, trap->text);
+		textblock_append(tb, "%s", trap->text);
 		textblock_append(tb, "\n");
 		textui_textblock_show(tb, SCREEN_REGION, NULL);
 		textblock_free(tb);
@@ -2555,13 +2568,626 @@ static void do_cmd_knowledge_traps(const char *name, int row)
 }
 
 
+/**
+ * ------------------------------------------------------------------------
+ * SHAPECHANGE
+ * ------------------------------------------------------------------------ */
+
+/**
+ * Counts the number of interesting shapechanges and returns it.
+ */
+static int count_interesting_shapes(void)
+{
+	int count = 0;
+	struct player_shape *s;
+
+	for (s = shapes; s; s = s->next) {
+		if (! streq(s->name, "normal")) {
+			++count;
+		}
+	}
+
+	return count;
+}
+
+
+/**
+ * Is a comparison function for an array of struct player_shape* which is
+ * compatible with sort() and puts the elements in ascending alphabetical
+ * order by name.
+ */
+static int compare_shape_names(const void *left, const void *right)
+{
+	const struct player_shape * const *sleft = left;
+	const struct player_shape * const *sright = right;
+
+	return my_stricmp((*sleft)->name, (*sright)->name);
+}
+
+
+static void shape_lore_helper_append_to_list(const char* item,
+	const char ***list, int* p_nmax, int *p_n)
+{
+	if (*p_n >= *p_nmax) {
+		if (*p_nmax == 0) {
+			*p_nmax = 4;
+		} else {
+			assert(*p_nmax > 0);
+			*p_nmax *= 2;
+		}
+		*list = mem_realloc(*list, *p_nmax * sizeof(**list));
+	}
+	(*list)[*p_n] = string_make(item);
+	++*p_n;
+}
+
+
+static const char *skill_index_to_name(int i)
+{
+	const char *name;
+
+	switch (i) {
+	case SKILL_DISARM_PHYS:
+		name = "physical disarming";
+		break;
+
+	case SKILL_DISARM_MAGIC:
+		name = "magical disarming";
+		break;
+
+	case SKILL_DEVICE:
+		name = "magic devices";
+		break;
+
+	case SKILL_SAVE:
+		name = "saving throws";
+		break;
+
+	case SKILL_SEARCH:
+		name = "searching";
+		break;
+
+	case SKILL_TO_HIT_MELEE:
+		name = "melee to hit";
+		break;
+
+	case SKILL_TO_HIT_BOW:
+		name = "shooting to hit";
+		break;
+
+	case SKILL_TO_HIT_THROW:
+		name = "throwing to hit";
+		break;
+
+	case SKILL_DIGGING:
+		name = "digging";
+		break;
+
+	default:
+		name = "unknown skill";
+		break;
+	}
+
+	return name;
+}
+
+
+static void shape_lore_append_list(textblock *tb,
+	const char * const *list, int n)
+{
+	int i;
+
+	if (n > 0) {
+		textblock_append(tb, " %s", list[0]);
+	}
+	for (i = 1; i < n; ++i) {
+		textblock_append(tb, "%s %s", (i < n - 1) ? "," : " and",
+			list[i]);
+	}
+}
+
+
+static void shape_lore_append_basic_combat(textblock *tb,
+	const struct player_shape *s)
+{
+	char toa_msg[24];
+	char toh_msg[24];
+	char tod_msg[24];
+	const char* msgs[3];
+	int n = 0;
+
+	if (s->to_a != 0) {
+		strnfmt(toa_msg, sizeof(toa_msg), "%+d to AC", s->to_a);
+		msgs[n] = toa_msg;
+		++n;
+	}
+	if (s->to_h != 0) {
+		strnfmt(toh_msg, sizeof(toh_msg), "%+d to hit", s->to_h);
+		msgs[n] = toh_msg;
+		++n;
+	}
+	if (s->to_d != 0) {
+		strnfmt(tod_msg, sizeof(tod_msg), "%+d to damage", s->to_d);
+		msgs[n] = tod_msg;
+		++n;
+	}
+	if (n > 0) {
+		textblock_append(tb, "Adds");
+		shape_lore_append_list(tb, msgs, n);
+		textblock_append(tb, ".\n");
+	}
+}
+
+
+static void shape_lore_append_skills(textblock *tb,
+	const struct player_shape *s)
+{
+	const char **msgs = NULL;
+	int nmax = 0, n = 0;
+	int i;
+
+	for (i = 0; i < SKILL_MAX; ++i) {
+		if (s->skills[i] != 0) {
+			shape_lore_helper_append_to_list(
+				format("%+d to %s", s->skills[i],
+					skill_index_to_name(i)),
+				&msgs, &nmax, &n);
+		}
+	}
+
+	if (n > 0) {
+		textblock_append(tb, "Adds");
+		shape_lore_append_list(tb, msgs, n);
+		textblock_append(tb, ".\n");
+		for (i = 0; i < n; ++i) {
+			string_free((char*) msgs[i]);
+		}
+	}
+
+	mem_free(msgs);
+}
+
+
+static void shape_lore_append_non_stat_modifiers(textblock *tb,
+	const struct player_shape *s)
+{
+	const char **msgs = NULL;
+	int nmax = 0, n = 0;
+	int i;
+
+	for (i = STAT_MAX; i < OBJ_MOD_MAX; ++i) {
+		if (s->modifiers[i] != 0) {
+			shape_lore_helper_append_to_list(
+				format("%+d to %s",
+					s->modifiers[i],
+					lookup_obj_property(OBJ_PROPERTY_MOD, i)->name),
+				&msgs, &nmax, &n);
+		}
+	}
+
+	if (n > 0) {
+		textblock_append(tb, "Adds");
+		shape_lore_append_list(tb, msgs, n);
+		textblock_append(tb, ".\n");
+		for (i = 0; i < n; ++i) {
+			string_free((char*) msgs[i]);
+		}
+	}
+
+	mem_free(msgs);
+}
+
+
+static void shape_lore_append_stat_modifiers(textblock *tb,
+	const struct player_shape *s)
+{
+	const char **msgs = NULL;
+	int nmax = 0, n = 0;
+	int i;
+
+	for (i = 0; i < STAT_MAX; ++i) {
+		if (s->modifiers[i] != 0) {
+			shape_lore_helper_append_to_list(
+				format("%+d to %s",
+					s->modifiers[i],
+					lookup_obj_property(OBJ_PROPERTY_MOD, i)->name),
+				&msgs, &nmax, &n);
+		}
+	}
+
+	if (n > 0) {
+		textblock_append(tb, "Adds");
+		shape_lore_append_list(tb, msgs, n);
+		textblock_append(tb, ".\n");
+		for (i = 0; i < n; ++i) {
+			string_free((char*) msgs[i]);
+		}
+	}
+
+	mem_free(msgs);
+}
+
+
+static void shape_lore_append_resistances(textblock *tb,
+	const struct player_shape *s)
+{
+	const char* vul[ELEM_MAX];
+	const char* res[ELEM_MAX];
+	const char* imm[ELEM_MAX];
+	int nvul = 0, nres = 0, nimm = 0;
+	int i;
+
+	for (i = 0; i < ELEM_MAX; ++i) {
+		if (s->el_info[i].res_level < 0) {
+			vul[nvul] = projections[i].name;
+			++nvul;
+		} else if (s->el_info[i].res_level >= 3) {
+			imm[nimm] = projections[i].name;
+			++nimm;
+		} else if (s->el_info[i].res_level != 0) {
+			res[nres] = projections[i].name;
+			++nres;
+		}
+	}
+
+	if (nvul != 0) {
+		textblock_append(tb, "Makes you vulnerable to");
+		shape_lore_append_list(tb, vul, nvul);
+		textblock_append(tb, ".\n");
+	}
+
+	if (nres != 0) {
+		textblock_append(tb, "Makes you resistant to");
+		shape_lore_append_list(tb, res, nres);
+		textblock_append(tb, ".\n");
+	}
+
+	if (nimm != 0) {
+		textblock_append(tb, "Makes you immune to");
+		shape_lore_append_list(tb, imm, nimm);
+		textblock_append(tb, ".\n");
+	}
+}
+
+
+static void shape_lore_append_protection_flags(textblock *tb,
+	const struct player_shape *s)
+{
+	const char **msgs = NULL;
+	int nmax = 0, n = 0;
+	int i;
+
+	for (i = 1; i < OF_MAX; ++i) {
+		struct obj_property *prop =
+			lookup_obj_property(OBJ_PROPERTY_FLAG, i);
+
+		if (prop->subtype == OFT_PROT &&
+			of_has(s->flags, prop->index)) {
+			shape_lore_helper_append_to_list(
+				prop->desc, &msgs, &nmax, &n);
+		}
+	}
+
+	if (n > 0) {
+		textblock_append(tb, "Provides protection from");
+		shape_lore_append_list(tb, msgs, n);
+		textblock_append(tb, ".\n");
+		for (i = 0; i < n; ++i) {
+			string_free((char*) msgs[i]);
+		}
+	}
+
+	mem_free(msgs);
+}
+
+
+static void shape_lore_append_sustains(textblock *tb,
+	const struct player_shape *s)
+{
+	const char **msgs = NULL;
+	int nmax = 0, n = 0;
+	int i;
+
+	for (i = 0; i < STAT_MAX; ++i) {
+		struct obj_property *prop =
+			lookup_obj_property(OBJ_PROPERTY_STAT, i);
+
+		if (of_has(s->flags, sustain_flag(prop->index))) {
+			shape_lore_helper_append_to_list(
+				prop->name, &msgs, &nmax, &n);
+		}
+	}
+
+	if (n > 0) {
+		textblock_append(tb, "Sustains");
+		shape_lore_append_list(tb, msgs, n);
+		textblock_append(tb, ".\n");
+		for (i = 0; i < n; ++i) {
+			string_free((char*) msgs[i]);
+		}
+	}
+
+	mem_free(msgs);
+}
+
+
+static void shape_lore_append_misc_flags(textblock *tb,
+	const struct player_shape *s)
+{
+	int n = 0;
+	int i;
+	struct player_ability *ability;
+
+	for (i = 1; i < OF_MAX; ++i) {
+		struct obj_property *prop =
+			lookup_obj_property(OBJ_PROPERTY_FLAG, i);
+
+		if ((prop->subtype == OFT_MISC || prop->subtype == OFT_MELEE ||
+			prop->subtype == OFT_BAD) &&
+			of_has(s->flags, prop->index)) {
+			textblock_append(tb, "%s%s.", (n > 0) ? "  " : "",
+				prop->desc);
+			++n;
+		}
+	}
+
+	for (ability = player_abilities; ability; ability = ability->next) {
+		if (streq(ability->type, "player") &&
+			pf_has(s->pflags, ability->index)) {
+			textblock_append(tb, "%s%s", (n > 0) ? "  " : "",
+				ability->desc);
+			++n;
+		}
+	}
+
+	if (n > 0) {
+		textblock_append(tb, "\n");
+	}
+}
+
+
+static void shape_lore_append_change_effects(textblock *tb,
+	const struct player_shape *s)
+{
+	textblock *tbe = effect_describe(s->effect, "Changing into the shape ",
+		0, false);
+
+	if (tbe) {
+		textblock_append_textblock(tb, tbe);
+		textblock_free(tbe);
+		textblock_append(tb, ".\n");
+	}
+}
+
+
+static void shape_lore_append_triggering_spells(textblock *tb,
+	const struct player_shape *s)
+{
+	int n = 0;
+	struct player_class *c;
+
+	for (c = classes; c; c = c->next) {
+		int ibook;
+
+		for (ibook = 0; ibook < c->magic.num_books; ++ibook) {
+			const struct class_book *book = c->magic.books + ibook;
+			const struct object_kind *kind =
+				lookup_kind(book->tval, book->sval);
+			int ispell;
+
+			if (!kind || !kind->name) {
+				continue;
+			}
+			for (ispell = 0; ispell < book->num_spells; ++ispell) {
+				const struct class_spell *spell =
+					book->spells + ispell;
+				const struct effect *effect;
+
+				for (effect = spell->effect;
+					effect;
+					effect = effect->next) {
+					if (effect->index == EF_SHAPECHANGE &&
+						effect->subtype == s->sidx) {
+						if (n == 0) {
+							textblock_append(tb, "\n");
+						}
+						textblock_append(tb,
+							"The %s spell, %s, from %s triggers the shapechange.",
+							c->name,
+							spell->name,
+							kind->name
+						);
+						++n;
+					}
+				}
+			}
+		}
+	}
+
+	if (n > 0) {
+		textblock_append(tb, "\n");
+	}
+}
+
+
+/**
+ * Display information about a shape change.
+ */
+static void shape_lore(const struct player_shape *s)
+{
+	textblock *tb = textblock_new();
+
+	textblock_append(tb, "%s", s->name);
+	textblock_append(tb, "\nLike all shapes, the equipment at the time of "
+		"the shapechange sets the base attributes, including damage "
+		"per blow, number of blows and resistances.  While changed, "
+		"items in your pack or on the floor (except for pickup or "
+		"eating) are inaccessible.  To switch back to your normal "
+		"shape, cast a spell or use an item command other than eat "
+		"(drop, for instance).\n");
+	shape_lore_append_basic_combat(tb, s);
+	shape_lore_append_skills(tb, s);
+	shape_lore_append_non_stat_modifiers(tb, s);
+	shape_lore_append_stat_modifiers(tb, s);
+	shape_lore_append_resistances(tb, s);
+	shape_lore_append_protection_flags(tb, s);
+	shape_lore_append_sustains(tb, s);
+	shape_lore_append_misc_flags(tb, s);
+	shape_lore_append_change_effects(tb, s);
+	shape_lore_append_triggering_spells(tb, s);
+
+	textui_textblock_show(tb, SCREEN_REGION, NULL);
+	textblock_free(tb);
+}
+
+
+static void do_cmd_knowledge_shapechange(const char *name, int row)
+{
+	region header_region = { 0, 0, -1, 5 };
+	region list_region = { 0, 6, -1, -2 };
+	int count = count_interesting_shapes();
+	struct menu* m;
+	struct player_shape **sarray;
+	const char **narray;
+	int omode;
+	int h, mark, mark_old;
+	bool displaying, redraw;
+	struct player_shape *s;
+	int i;
+
+	if (!count) {
+		return;
+	}
+
+	m = menu_new(MN_SKIN_SCROLL, menu_find_iter(MN_ITER_STRINGS));
+
+	/* Set up an easily indexable list of the interesting shapes. */
+	sarray = mem_alloc(count * sizeof(*sarray));
+	for (s = shapes, i = 0; s; s = s->next) {
+		if (streq(s->name, "normal")) {
+			continue;
+		}
+		sarray[i] = s;
+		++i;
+	}
+
+	/*
+	 * Sort them alphabetically by name and set up an array with just the
+	 * names.
+	 */
+	sort(sarray, count, sizeof(sarray[0]), compare_shape_names);
+	narray = mem_alloc(count * sizeof(*narray));
+	for (i = 0; i < count; ++i) {
+		narray[i] = sarray[i]->name;
+	}
+
+	menu_setpriv(m, count, narray);
+	menu_layout(m, &list_region);
+	m->flags |= MN_DBL_TAP;
+
+	screen_save();
+	clear_from(0);
+
+	/* Disable the roguelike commands for the duration */
+	omode = OPT(player, rogue_like_commands);
+	OPT(player, rogue_like_commands) = false;
+
+	h = 0;
+	mark = 0;
+	mark_old = -1;
+	displaying = true;
+	redraw = true;
+	while (displaying) {
+		bool recall = false;
+		int wnew, hnew;
+		ui_event ke0 = EVENT_EMPTY;
+		ui_event ke;
+
+		Term_get_size(&wnew, &hnew);
+		if (h != hnew) {
+			h = hnew;
+			redraw = true;
+		}
+
+		if (redraw) {
+			region_erase(&header_region);
+			prt("Knowledge - shapes", 2, 0);
+			prt("Name", 4, 0);
+			for (i = 0; i < MIN(80, wnew); i++) {
+				Term_putch(i, 5, COLOUR_WHITE, L'=');
+			}
+			prt("<dir>, 'r' to recall, ESC", h - 2, 0);
+			redraw = false;
+		}
+
+		if (mark_old != mark) {
+			mark_old = mark;
+			m->cursor = mark;
+		}
+
+		menu_refresh(m, false);
+
+		handle_stuff(player);
+
+		ke = inkey_ex();
+		if (ke.type == EVT_MOUSE) {
+			menu_handle_mouse(m, &ke, &ke0);
+		} else if (ke.type == EVT_KBRD) {
+			menu_handle_keypress(m, &ke, &ke0);
+		}
+		if (ke0.type != EVT_NONE) {
+			ke = ke0;
+		}
+
+		switch (ke.type) {
+			case EVT_KBRD:
+				if (ke.key.code == 'r' || ke.key.code == 'R') {
+					recall = true;
+				}
+				break;
+
+			case EVT_ESCAPE:
+				displaying = false;
+				break;
+
+			case EVT_SELECT:
+				if (mark == m->cursor) {
+					recall = true;
+				}
+				break;
+
+			case EVT_MOVE:
+				mark = m->cursor;
+				break;
+
+			default:
+				break;
+		}
+
+		if (recall) {
+			assert(mark >= 0 && mark < count);
+			shape_lore(sarray[mark]);
+		}
+	}
+
+	/* Restore roguelike option */
+	OPT(player, rogue_like_commands) = omode;
+
+	screen_load();
+
+	mem_free(narray);
+	mem_free(sarray);
+	menu_free(m);
+}
+
+
 /**
  * ------------------------------------------------------------------------
  * Main knowledge menus
  * ------------------------------------------------------------------------ */
 
 /* The first row of the knowledge_actions menu which does store knowledge */
-#define STORE_KNOWLEDGE_ROW 7
+#define STORE_KNOWLEDGE_ROW 8
 
 static void do_cmd_knowledge_store(const char *name, int row)
 {
@@ -2583,6 +3209,26 @@ static void do_cmd_knowledge_equip_cmp(const char* name, int row)
 	equip_cmp_display();
 }
 
+static bool handle_store_shortcuts(struct menu *m, const ui_event *ev, int oid)
+{
+	int i = 0;
+
+	assert(ev->type == EVT_KBRD);
+	while (1) {
+		if (! m->cmd_keys[i]) {
+			return false;
+		}
+		if (ev->key.code == (unsigned) m->cmd_keys[i]) {
+			menu_action *acts = menu_priv(m);
+
+			do_cmd_knowledge_store(
+				acts[i + STORE_KNOWLEDGE_ROW].name,
+				i + STORE_KNOWLEDGE_ROW);
+			return true;
+		}
+		++i;
+	}
+}
 
 /**
  * Definition of the "player knowledge" menu.
@@ -2596,29 +3242,22 @@ static menu_action knowledge_actions[] =
 { 0, 0, "Display monster knowledge",  	   do_cmd_knowledge_monsters  },
 { 0, 0, "Display feature knowledge",  	   do_cmd_knowledge_features  },
 { 0, 0, "Display trap knowledge",          do_cmd_knowledge_traps  },
-{ 0, 0, "Display contents of general store", do_cmd_knowledge_store     },
-{ 0, 0, "Display contents of armourer",      do_cmd_knowledge_store     },
-{ 0, 0, "Display contents of weaponsmith",   do_cmd_knowledge_store     },
-{ 0, 0, "Display contents of bookseller",    do_cmd_knowledge_store     },
-{ 0, 0, "Display contents of alchemist",     do_cmd_knowledge_store     },
-{ 0, 0, "Display contents of magic shop",    do_cmd_knowledge_store     },
-{ 0, 0, "Display contents of black market",  do_cmd_knowledge_store     },
-{ 0, 0, "Display contents of home",   	   do_cmd_knowledge_store     },
+{ 0, 0, "Display shapechange effects",     do_cmd_knowledge_shapechange },
+{ 0, 0, "Display contents of general store (1)", do_cmd_knowledge_store },
+{ 0, 0, "Display contents of armourer (2)",      do_cmd_knowledge_store },
+{ 0, 0, "Display contents of weaponsmith (3)",   do_cmd_knowledge_store },
+{ 0, 0, "Display contents of bookseller (4)",    do_cmd_knowledge_store },
+{ 0, 0, "Display contents of alchemist (5)",     do_cmd_knowledge_store },
+{ 0, 0, "Display contents of magic shop (6)",    do_cmd_knowledge_store },
+{ 0, 0, "Display contents of black market (7)",  do_cmd_knowledge_store },
+{ 0, 0, "Display contents of home (8)",          do_cmd_knowledge_store },
 { 0, 0, "Display hall of fame",       	   do_cmd_knowledge_scores    },
 { 0, 0, "Display character history",  	   do_cmd_knowledge_history   },
-{ 0, 0, "Display equipable comparison",    do_cmd_knowledge_equip_cmp },
+{ 0, 0, "Display equippable comparison",   do_cmd_knowledge_equip_cmp },
 };
 
 static struct menu knowledge_menu;
 
-
-/**
- * Keep macro counts happy.
- */
-static void cleanup_cmds(void) {
-	mem_free(obj_group_order);
-}
-
 void textui_knowledge_init(void)
 {
 	/* Initialize the menus */
@@ -2628,6 +3267,10 @@ void textui_knowledge_init(void)
 
 	menu->title = "Display current knowledge";
 	menu->selections = lower_case;
+	/* Shortcuts to get the contents of the stores by number; does prevent
+	 * the normal use of 4 and 6 to go to the previous or next menu */
+	menu->cmd_keys = "12345678";
+	menu->keys_hook = handle_store_shortcuts;
 
 	/* initialize other static variables */
 	if (!obj_group_order) {
@@ -2635,7 +3278,6 @@ void textui_knowledge_init(void)
 		int gid = -1;
 
 		obj_group_order = mem_zalloc((TV_MAX + 1) * sizeof(int));
-		atexit(cleanup_cmds);
 
 		/* Allow for missing values */
 		for (i = 0; i < TV_MAX; i++)
@@ -2650,6 +3292,13 @@ void textui_knowledge_init(void)
 }
 
 
+void textui_knowledge_cleanup(void)
+{
+	mem_free(obj_group_order);
+	obj_group_order = NULL;
+}
+
+
 /**
  * Display the "player knowledge" menu, greying out items that won't display
  * anything.
@@ -2689,6 +3338,10 @@ void textui_browse_knowledge(void)
 	else
 		knowledge_actions[4].flags = MN_ACT_GRAYED;
 
+	/* Shapechanges */
+	knowledge_actions[7].flags = (count_interesting_shapes() > 0) ?
+		0 : MN_ACT_GRAYED;
+
 	screen_save();
 	menu_layout(&knowledge_menu, &knowledge_region);
 
@@ -2764,8 +3417,8 @@ void do_cmd_messages(void)
 		for (j = 0; (j < hgt - 4) && (i + j < n); j++) {
 			const char *msg;
 			const char *str = message_str(i + j);
-			byte attr = message_color(i + j);
-			u16b count = message_count(i + j);
+			uint8_t attr = message_color(i + j);
+			uint16_t count = message_count(i + j);
 
 			if (count == 1)
 				msg = str;
@@ -2879,7 +3532,7 @@ void do_cmd_messages(void)
 
 		/* Find the next item */
 		if (ke.key.code == '-' && strlen(shower)) {
-			s16b z;
+			int16_t z;
 
 			/* Scan messages */
 			for (z = i + 1; z < n; z++) {
@@ -2926,8 +3579,9 @@ void do_cmd_inven(void)
 		screen_save();
 
 		/* Get an item to use a context command on (Display the inventory) */
-		if (get_item(&obj, "Select Item:", NULL, CMD_NULL, NULL,
-					 GET_ITEM_PARAMS)) {
+		if (get_item(&obj, "Select Item:",
+				"Error in do_cmd_inven(), please report.",
+				CMD_NULL, NULL, GET_ITEM_PARAMS)) {
 			/* Load screen */
 			screen_load();
 
@@ -2971,8 +3625,9 @@ void do_cmd_equip(void)
 		screen_save();
 
 		/* Get an item to use a context command on (Display the equipment) */
-		if (get_item(&obj, "Select Item:", NULL, CMD_NULL, NULL,
-					 GET_ITEM_PARAMS)) {
+		if (get_item(&obj, "Select Item:",
+				"Error in do_cmd_equip(), please report.",
+				CMD_NULL, NULL, GET_ITEM_PARAMS)) {
 			/* Load screen */
 			screen_load();
 
@@ -3019,8 +3674,9 @@ void do_cmd_quiver(void)
 		screen_save();
 
 		/* Get an item to use a context command on (Display the quiver) */
-		if (get_item(&obj, "Select Item:", NULL, CMD_NULL, NULL,
-					 GET_ITEM_PARAMS)) {
+		if (get_item(&obj, "Select Item:",
+				"Error in do_cmd_quiver(), please report.",
+				CMD_NULL, NULL, GET_ITEM_PARAMS)) {
 			/* Load screen */
 			screen_load();
 
@@ -3058,19 +3714,26 @@ void do_cmd_look(void)
 }
 
 
-
-/**
- * Number of basic grids per panel, vertically and horizontally
- */
-#define PANEL_SIZE	11
-
 /**
  * Allow the player to examine other sectors on the map
  */
 void do_cmd_locate(void)
 {
+	int panel_hgt, panel_wid;
 	int y1, x1;
 
+	/* Use dimensions that match those in ui-output.c. */
+	if (Term == term_screen) {
+		panel_hgt = SCREEN_HGT;
+		panel_wid = SCREEN_WID;
+	} else {
+		panel_hgt = Term->hgt / tile_height;
+		panel_wid = Term->wid / tile_width;
+	}
+	/* Bound below to avoid division by zero. */
+	panel_hgt = MAX(panel_hgt, 1);
+	panel_wid = MAX(panel_wid, 1);
+
 	/* Start at current panel */
 	y1 = Term->offset_y;
 	x1 = Term->offset_x;
@@ -3087,10 +3750,6 @@ void do_cmd_locate(void)
 		int y2 = Term->offset_y;
 		int x2 = Term->offset_x;
 		
-		/* Adjust for tiles */
-		int panel_hgt = (int)(PANEL_SIZE / tile_height);
-		int panel_wid = (int)(PANEL_SIZE / tile_width);
-
 		/* Describe the location */
 		if ((y2 == y1) && (x2 == x1)) {
 			tmp_val[0] = '\0';
@@ -3103,14 +3762,14 @@ void do_cmd_locate(void)
 		/* Prepare to ask which way to look */
 		strnfmt(out_val, sizeof(out_val),
 		        "Map sector [%d,%d], which is%s your sector.  Direction?",
-		        (y2 / panel_hgt), (x2 / panel_wid), tmp_val);
+		        (2 * y2) / panel_hgt, (2 * x2) / panel_wid, tmp_val);
 
 		/* More detail */
 		if (OPT(player, center_player)) {
 			strnfmt(out_val, sizeof(out_val),
 		        	"Map sector [%d(%02d),%d(%02d)], which is%s your sector.  Direction?",
-					(y2 / panel_hgt), (y2 % panel_hgt),
-					(x2 / panel_wid), (x2 % panel_wid), tmp_val);
+					(2 * y2) / panel_hgt, (2 * y2) % panel_hgt,
+					(2 * x2) / panel_wid, (2 * x2) % panel_wid, tmp_val);
 		}
 
 		/* Get a direction */
@@ -3124,7 +3783,7 @@ void do_cmd_locate(void)
 			dir = target_dir(command);
 
 			/* Error */
-			if (!dir) bell("Illegal direction for locate!");
+			if (!dir) bell();
 		}
 
 		/* No direction */
@@ -3143,8 +3802,8 @@ void do_cmd_locate(void)
 
 static int cmp_mexp(const void *a, const void *b)
 {
-	u16b ia = *(const u16b *)a;
-	u16b ib = *(const u16b *)b;
+	uint16_t ia = *(const uint16_t *)a;
+	uint16_t ib = *(const uint16_t *)b;
 	if (r_info[ia].mexp < r_info[ib].mexp)
 		return -1;
 	if (r_info[ia].mexp > r_info[ib].mexp)
@@ -3154,8 +3813,8 @@ static int cmp_mexp(const void *a, const void *b)
 
 static int cmp_level(const void *a, const void *b)
 {
-	u16b ia = *(const u16b *)a;
-	u16b ib = *(const u16b *)b;
+	uint16_t ia = *(const uint16_t *)a;
+	uint16_t ib = *(const uint16_t *)b;
 	if (r_info[ia].level < r_info[ib].level)
 		return -1;
 	if (r_info[ia].level > r_info[ib].level)
@@ -3165,8 +3824,8 @@ static int cmp_level(const void *a, const void *b)
 
 static int cmp_tkill(const void *a, const void *b)
 {
-	u16b ia = *(const u16b *)a;
-	u16b ib = *(const u16b *)b;
+	uint16_t ia = *(const uint16_t *)a;
+	uint16_t ib = *(const uint16_t *)b;
 	if (l_list[ia].tkills < l_list[ib].tkills)
 		return -1;
 	if (l_list[ia].tkills > l_list[ib].tkills)
@@ -3176,8 +3835,8 @@ static int cmp_tkill(const void *a, const void *b)
 
 static int cmp_pkill(const void *a, const void *b)
 {
-	u16b ia = *(const u16b *)a;
-	u16b ib = *(const u16b *)b;
+	uint16_t ia = *(const uint16_t *)a;
+	uint16_t ib = *(const uint16_t *)b;
 	if (l_list[ia].pkills < l_list[ib].pkills)
 		return -1;
 	if (l_list[ia].pkills > l_list[ib].pkills)
@@ -3278,7 +3937,7 @@ void do_cmd_query_symbol(void)
 
 	bool recall = false;
 
-	u16b *who;
+	uint16_t *who;
 
 	/* Get a character, or abort */
 	if (!get_com("Enter character to be identified, or control+[ANU]: ", &sym))
@@ -3302,7 +3961,7 @@ void do_cmd_query_symbol(void)
 	prt(buf, 0, 0);
 
 	/* Allocate the "who" array */
-	who = mem_zalloc(z_info->r_max * sizeof(u16b));
+	who = mem_zalloc(z_info->r_max * sizeof(uint16_t));
 
 	/* Collect matching monsters */
 	for (num = 0, idx = 1; idx < z_info->r_max - 1; idx++) {
