@@ -50,7 +50,7 @@
  * Panel line type
  */
 struct panel_line {
-	byte attr;
+	uint8_t attr;
 	const char *label;
 	char value[20];
 };
@@ -89,7 +89,7 @@ static void panel_free(struct panel *p) {
 /**
  * Add a new line to the panel
  */
-static void panel_line(struct panel *p, byte attr, const char *label,
+static void panel_line(struct panel *p, uint8_t attr, const char *label,
 		const char *fmt, ...) {
 	va_list vp;
 
@@ -174,7 +174,7 @@ static void release_char_sheet_config(void)
 static bool check_for_two_categories(const struct ui_entry* entry,
 	void *closure)
 {
-	char **categories = closure;
+	const char **categories = closure;
 
 	return ui_entry_has_category(entry, categories[0]) &&
 		ui_entry_has_category(entry, categories[1]);
@@ -183,13 +183,13 @@ static bool check_for_two_categories(const struct ui_entry* entry,
 
 static void configure_char_sheet(void)
 {
-	char* region_categories[] = {
+	const char* region_categories[] = {
 		"resistances",
 		"abilities",
 		"hindrances",
 		"modifiers"
 	};
-	char* test_categories[2];
+	const char* test_categories[2];
 	struct ui_entry_iterator* ui_iter;
 	int i, n;
 
@@ -268,7 +268,7 @@ static void configure_char_sheet(void)
  * Returns a "rating" of x depending on y, and sets "attr" to the
  * corresponding "attribute".
  */
-static const char *likert(int x, int y, byte *attr)
+static const char *likert(int x, int y, uint8_t *attr)
 {
 	/* Paranoia */
 	if (y <= 0) y = 1;
@@ -348,7 +348,7 @@ static void display_player_equippy(int y, int x)
 {
 	int i;
 
-	byte a;
+	uint8_t a;
 	wchar_t c;
 
 	struct object *obj;
@@ -633,7 +633,8 @@ static const char *show_adv_exp(void)
 {
 	if (player->lev < PY_MAX_LEVEL) {
 		static char buffer[30];
-		s32b advance = (player_exp[player->lev - 1] * player->expfact / 100L);
+		int32_t advance = (player_exp[player->lev - 1]
+			* player->expfact / 100L);
 		strnfmt(buffer, sizeof(buffer), "%d", advance);
 		return buffer;
 	}
@@ -670,7 +671,7 @@ static const char *show_speed(void)
 	return buffer;
 }
 
-static byte max_color(int val, int max)
+static uint8_t max_color(int val, int max)
 {
 	return val < max ? COLOUR_YELLOW : COLOUR_L_GREEN;
 }
@@ -678,7 +679,7 @@ static byte max_color(int val, int max)
 /**
  * Colours for table items
  */
-static const byte colour_table[] =
+static const uint8_t colour_table[] =
 {
 	COLOUR_RED, COLOUR_RED, COLOUR_RED, COLOUR_L_RED, COLOUR_ORANGE,
 	COLOUR_YELLOW, COLOUR_YELLOW, COLOUR_GREEN, COLOUR_GREEN, COLOUR_L_GREEN,
@@ -702,7 +703,7 @@ static struct panel *get_panel_topleft(void) {
 static struct panel *get_panel_midleft(void) {
 	struct panel *p = panel_allocate(9);
 	int diff = weight_remaining(player);
-	byte attr = diff < 0 ? COLOUR_L_RED : COLOUR_L_GREEN;
+	uint8_t attr = diff < 0 ? COLOUR_L_RED : COLOUR_L_GREEN;
 
 	panel_line(p, max_color(player->lev, player->max_lev),
 			"Level", "%d", player->lev);
@@ -767,7 +768,7 @@ static struct panel *get_panel_skills(void) {
 	struct panel *p = panel_allocate(8);
 
 	int skill;
-	byte attr;
+	uint8_t attr;
 	const char *desc;
 	int depth = cave ? cave->depth : 0;
 
@@ -813,7 +814,7 @@ static struct panel *get_panel_skills(void) {
 
 static struct panel *get_panel_misc(void) {
 	struct panel *p = panel_allocate(7);
-	byte attr = COLOUR_L_BLUE;
+	uint8_t attr = COLOUR_L_BLUE;
 
 	panel_line(p, attr, "Age", "%d", player->age);
 	panel_line(p, attr, "Height", "%d'%d\"", player->ht / 12, player->ht % 12);
@@ -922,8 +923,18 @@ void write_character_dump(ang_file *fff)
 										   z_info->store_inven_max);
 	char o_name[80];
 
-	char buf[1024];
-	char *p;
+	int n;
+	char *buf, *p;
+
+	if (!have_valid_char_sheet_config()) {
+		configure_char_sheet();
+	}
+
+	n = 80;
+	if (n < 2 * cached_config->res_cols + 1) {
+		n = 2 * cached_config->res_cols + 1;
+	}
+	buf = mem_alloc(text_wcsz() * n + 1);
 
 	/* Begin dump */
 	file_putf(fff, "  [%s Character Dump]\n\n", buildid);
@@ -940,7 +951,12 @@ void write_character_dump(ang_file *fff)
 			(void)(Term_what(x, y, &a, &c));
 
 			/* Dump it */
-			p += wctomb(p, c);
+			n = text_wctomb(p, c);
+			if (n > 0) {
+				p += n;
+			} else {
+				*p++ = ' ';
+			}
 		}
 
 		/* Back up over spaces */
@@ -973,7 +989,12 @@ void write_character_dump(ang_file *fff)
 			(void)(Term_what(x, y, &a, &c));
 
 			/* Dump it */
-			p += wctomb(p, c);
+			n = text_wctomb(p, c);
+			if (n > 0) {
+				p += n;
+			} else {
+				*p++ = ' ';
+			}
 		}
 
 		/* Back up over spaces */
@@ -1006,7 +1027,12 @@ void write_character_dump(ang_file *fff)
 			(void)(Term_what(x + 2 * cached_config->res_cols + 2, y, &a, &c));
 
 			/* Dump it */
-			p += wctomb(p, c);
+			n = text_wctomb(p, c);
+			if (n > 0) {
+				p += n;
+			} else {
+				*p++ = ' ';
+			}
 		}
 
 		/* Back up over spaces */
@@ -1030,7 +1056,7 @@ void write_character_dump(ang_file *fff)
 		file_putf(fff, "  [Last Messages]\n\n");
 		while (i-- > 0)
 		{
-			file_putf(fff, "> %s\n", message_str((s16b)i));
+			file_putf(fff, "> %s\n", message_str((int16_t)i));
 		}
 		file_putf(fff, "\nKilled by %s.\n\n", player->died_from);
 	}
@@ -1042,8 +1068,9 @@ void write_character_dump(ang_file *fff)
 		struct object *obj = slot_object(player, i);
 		if (!obj) continue;
 
-		object_desc(o_name, sizeof(o_name), obj, ODESC_PREFIX | ODESC_FULL);
-		file_putf(fff, "%c) %s\n", gear_to_label(obj), o_name);
+		object_desc(o_name, sizeof(o_name), obj,
+			ODESC_PREFIX | ODESC_FULL, player);
+		file_putf(fff, "%c) %s\n", gear_to_label(player, obj), o_name);
 		object_info_chardump(fff, obj, 5, 72);
 	}
 	file_putf(fff, "\n\n");
@@ -1054,8 +1081,9 @@ void write_character_dump(ang_file *fff)
 		struct object *obj = player->upkeep->inven[i];
 		if (!obj) break;
 
-		object_desc(o_name, sizeof(o_name), obj, ODESC_PREFIX | ODESC_FULL);
-		file_putf(fff, "%c) %s\n", gear_to_label(obj), o_name);
+		object_desc(o_name, sizeof(o_name), obj,
+			ODESC_PREFIX | ODESC_FULL, player);
+		file_putf(fff, "%c) %s\n", gear_to_label(player, obj), o_name);
 		object_info_chardump(fff, obj, 5, 72);
 	}
 	file_putf(fff, "\n\n");
@@ -1066,8 +1094,9 @@ void write_character_dump(ang_file *fff)
 		struct object *obj = player->upkeep->quiver[i];
 		if (!obj) continue;
 
-		object_desc(o_name, sizeof(o_name), obj, ODESC_PREFIX | ODESC_FULL);
-		file_putf(fff, "%c) %s\n", gear_to_label(obj), o_name);
+		object_desc(o_name, sizeof(o_name), obj,
+			ODESC_PREFIX | ODESC_FULL, player);
+		file_putf(fff, "%c) %s\n", gear_to_label(player, obj), o_name);
 		object_info_chardump(fff, obj, 5, 72);
 	}
 	file_putf(fff, "\n\n");
@@ -1082,7 +1111,8 @@ void write_character_dump(ang_file *fff)
 		for (i = 0; i < z_info->store_inven_max; i++) {
 			struct object *obj = home_list[i];
 			if (!obj) break;
-			object_desc(o_name, sizeof(o_name), obj, ODESC_PREFIX | ODESC_FULL);
+			object_desc(o_name, sizeof(o_name), obj,
+				ODESC_PREFIX | ODESC_FULL, player);
 			file_putf(fff, "%c) %s\n", I2A(i), o_name);
 
 			object_info_chardump(fff, obj, 5, 72);
@@ -1123,7 +1153,17 @@ void write_character_dump(ang_file *fff)
 		file_putf(fff, "\n");
 	}
 
+	/*
+	 * Display the randart seed, if applicable.  Use the same format as is
+	 * used when constructing the randart file name.
+	 */
+	if (OPT(player, birth_randarts)) {
+		file_putf(fff, "  [Randart seed]\n\n");
+		file_putf(fff, "%08x\n\n", seed_randart);
+	}
+
 	mem_free(home_list);
+	mem_free(buf);
 }
 
 /**
