@@ -35,7 +35,7 @@ static void display_score_page(const struct high_score scores[], int start,
 	/* Dump 5 entries */
 	for (n = 0; start < count && n < 5; start++, n++) {
 		const struct high_score *score = &scores[start];
-		byte attr;
+		uint8_t attr;
 		int clev, mlev, cdun, mdun;
 		const char *user, *gold, *when, *aged;
 		struct player_class *c;
@@ -190,7 +190,7 @@ void predict_score(bool allow_scrolling)
 
 	/* Read scores, place current score */
 	highscore_read(scores, N_ELEMENTS(scores));
-	build_score(&the_score, "nobody (yet!)", NULL);
+	build_score(&the_score, player, "nobody (yet!)", NULL);
 
 	if (player->is_dead)
 		j = highscore_where(&the_score, scores, N_ELEMENTS(scores));
