@@ -32,7 +32,7 @@
 
 struct textblock {
 	wchar_t *text;
-	byte *attrs;
+	uint8_t *attrs;
 
 	size_t strlen;
 	size_t size;
@@ -71,7 +71,7 @@ void textblock_free(textblock *tb)
  * \param tb is the textblock we need to resize.
  * \param additional_size is how many characters we want to add.
  */
-void textblock_resize_if_needed(textblock *tb, size_t additional_size)
+static void textblock_resize_if_needed(textblock *tb, size_t additional_size)
 {
 	size_t remaining = tb->size - tb->strlen;
 
@@ -83,7 +83,7 @@ void textblock_resize_if_needed(textblock *tb, size_t additional_size)
 	}
 }
 
-static void textblock_vappend_c(textblock *tb, byte attr, const char *fmt,
+static void textblock_vappend_c(textblock *tb, uint8_t attr, const char *fmt,
 		va_list vp)
 {
 	size_t temp_len = TEXTBLOCK_LEN_INITIAL;
@@ -125,7 +125,7 @@ static void textblock_vappend_c(textblock *tb, byte attr, const char *fmt,
 /**
  * Add a graphics tile to a text block.
  */
-void textblock_append_pict(textblock *tb, byte attr, int c)
+void textblock_append_pict(textblock *tb, uint8_t attr, int c)
 {
 	textblock_resize_if_needed(tb, 1);
 	tb->text[tb->strlen] = (wchar_t)c;
@@ -133,35 +133,6 @@ void textblock_append_pict(textblock *tb, byte attr, int c)
 	tb->strlen += 1;
 }
 
-/**
- * Append a UTF-8 string to the textblock.
- *
- * This is needed in order for proper file writing. Normally, textblocks convert
- * to the system's encoding when a string is appended. However, there are still
- * some strings in the game that are imported from external files as UTF-8.
- * Instead of requiring each port to provide another converter back to UTF-8,
- * we'll just use the original strings as is.
- *
- * \param tb is the textblock we are appending to.
- * \param utf8_string is the C string that is encoded as UTF-8.
- */
-void textblock_append_utf8(textblock *tb, const char *utf8_string)
-{
-	size_t i;
-	size_t new_length = strlen(utf8_string);
-
-	textblock_resize_if_needed(tb, new_length);
-
-	/* Append each UTF-8 char one at a time, so we don't trigger any
-	 * conversions (which would require multiple bytes). */
-	for (i = 0; i < new_length; i++) {
-		tb->text[tb->strlen + i] = (wchar_t)utf8_string[i];
-	}
-
-	memset(tb->attrs + tb->strlen, COLOUR_WHITE, new_length);
-	tb->strlen += new_length;
-}
-
 /**
  * Append one textblock to another.
  *
@@ -194,7 +165,7 @@ void textblock_append(textblock *tb, const char *fmt, ...)
 /**
  * Add coloured text to a text block, formatted.
  */
-void textblock_append_c(textblock *tb, byte attr, const char *fmt, ...)
+void textblock_append_c(textblock *tb, uint8_t attr, const char *fmt, ...)
 {
 	va_list vp;
 	va_start(vp, fmt);
@@ -216,7 +187,7 @@ const wchar_t *textblock_text(textblock *tb)
 /**
  * Return a pointer to the text attrs.
  */
-const byte *textblock_attrs(textblock *tb)
+const uint8_t *textblock_attrs(textblock *tb)
 {
 	return tb->attrs;
 }
@@ -341,23 +312,35 @@ void textblock_to_file(textblock *tb, ang_file *f, int indent, int wrap_at)
 	size_t *line_starts = NULL;
 	size_t *line_lengths = NULL;
 
-	size_t n_lines, i;
+	size_t n_lines, i, j;
+	char *mbbuf;
 
 	int width = wrap_at - indent;
 	assert(width > 0);
 
 	n_lines = textblock_calculate_lines(tb, &line_starts, &line_lengths, width);
+	mbbuf = mem_alloc(text_wcsz() + 1);
 
 	for (i = 0; i < n_lines; i++) {
-		/* For some reason, the %*c part of the format string was still
-		 * indenting, even when indent was zero */
-		if (indent == 0)
-			file_putf(f, "%.*ls\n", line_lengths[i], tb->text + line_starts[i]);
-		else
-			file_putf(f, "%*c%.*ls\n", indent, ' ', line_lengths[i],
-					  tb->text + line_starts[i]);
+		if (indent > 0) {
+			file_putf(f, "%*c", indent, ' ');
+		}
+		for (j = 0; j < line_lengths[i]; ++j) {
+			int nc = text_wctomb(mbbuf,
+				tb->text[line_starts[i] + j]);
+
+			if (nc > 0) {
+				mbbuf[nc] = 0;
+			} else {
+				mbbuf[0] = ' ';
+				mbbuf[1] = 0;
+			}
+			file_putf(f, "%s", mbbuf);
+		}
+		file_putf(f, "\n");
 	}
 
+	mem_free(mbbuf);
 	mem_free(line_starts);
 	mem_free(line_lengths);
 }
@@ -374,7 +357,7 @@ void textblock_to_file(textblock *tb, ang_file *f, int indent, int wrap_at)
 /**
  * Function hook to output (colored) text to the screen or to a file.
  */
-void (*text_out_hook)(byte a, const char *str);
+void (*text_out_hook)(uint8_t a, const char *str);
 
 /**
  * Hack -- Where to wrap the text when using text_out().  Use the default
@@ -412,7 +395,7 @@ ang_file *text_out_file = NULL;
  * You must be careful to end all file output with a newline character
  * to "flush" the stored line position.
  */
-void text_out_to_file(byte a, const char *str)
+void text_out_to_file(uint8_t a, const char *str)
 {
 	const char *s;
 	char buf[1024];
@@ -544,7 +527,7 @@ void text_out(const char *fmt, ...)
  * Output text to the screen (in color) or to a file depending on the
  * selected hook.
  */
-void text_out_c(byte a, const char *fmt, ...)
+void text_out_c(uint8_t a, const char *fmt, ...)
 {
 	char buf[1024];
 	va_list vp;
@@ -713,8 +696,6 @@ errr text_lines_to_file(const char *path, text_writer writer)
 
 	ang_file *new_file;
 
-	safe_setuid_grab();
-
 	/* Format filenames */
 	strnfmt(new_fname, sizeof(new_fname), "%s.new", path);
 	strnfmt(old_fname, sizeof(old_fname), "%s.old", path);
@@ -722,7 +703,6 @@ errr text_lines_to_file(const char *path, text_writer writer)
 	/* Write new file */
 	new_file = file_open(new_fname, MODE_WRITE, FTYPE_TEXT);
 	if (!new_file) {
-		safe_setuid_drop();
 		return -1;
 	}
 
@@ -743,8 +723,6 @@ errr text_lines_to_file(const char *path, text_writer writer)
 		file_delete(new_fname);
 	}
 
-	safe_setuid_drop();
-
 	return 0;
 }
 
